#############################################################################
#	Copyright  Bendor Research Pty. Ltd. 2004		All rights reserved.	#
#############################################################################
#																			#
#	Common part of the makefiles 											#
#	~~~~~~~~~~~~~~~~~~~~~~~~~~~~											#
#																			#
#	Who				When			Comment									#
#																			#
#	Zoltn Kcsi	2004 Feb 20		Original code							#
#																			#
#############################################################################

#
#	The following things are defined in a local makefile:
#
#	SOURCES		- the source files in this directory
#	MODULES		- directories containing other sources
#	USR_CLR		- any extra clean-up commands needed for the module
#
#	The following is passed down by the calling (or the main) Makefile:
#
#	PWD			- the absolute path to where the main Makefile resides
#	TARGET		- stem of the name of the target for the current module
#

#
#	Variables to export
#	~~~~~~~~~~~~~~~~~~~

export	CC AS LD MAKE CFLAGS AFLAGS LFLAGS INCDIR LIBINC LIBS DEFS

#
#	Derived variables
#	~~~~~~~~~~~~~~~~~

incs	=	$(patsubst %,-I %,$(patsubst %,$(PWD)/%,$(INCDIR)))
incs	+=	$(patsubst %,-isystem %,$(LIBINC))
objs	=	$(addsuffix .o,$(basename $(SOURCES)))
mods	=	$(shell for f in $(MODULES) ; do echo $$f/$$f.mod ; done)
fnam	=	$(patsubst $(PWD)/%,%,$(shell pwd)/$<)
libs	=	$(patsubst %,-L %,$(LIBDIR)) \
			$(patsubst %,-l %,$(LIBS))

#
#	Compilation rules
#	~~~~~~~~~~~~~~~~~


$(TARGET):	$(MODULES) $(TARGET).mod
	@echo "" > /dev/null

.PHONY: $(MODULES)
     
$(MODULES):
	@if [[ "" != "$@" ]] ; then \
		$(MAKE) -s -r -C $@ PWD=$(PWD) TARGET=$@ $@ ; \
	 fi
	
$(TARGET).mod:	$(objs) $(mods)
	@echo "Building    " $(patsubst $(PWD)/%,%,$(shell pwd)/$(TARGET).mod)
	@echo $(patsubst %,$(shell pwd)/%,$(objs)) | tr " " "\n" > $(TARGET).mod
	@if [[ "" != "$(MODULES)" ]] ; then \
		cat $(mods) >> $(TARGET).mod ; \
	 fi
	
%.o:	%.c
	@echo "Compiling   " $(fnam) 
	@$(CC) $(patsubst -I%,-isystem%,$(shell gtk-config --cflags)) \
		   $(CFLAGS) $(defs) $(incs) -MMD -c -o $@ $< 2> $*.err
	@rm -f $*.err

clean:	.mod_clean
	@echo "Clean."
	
.mod_clean:
	@for f in $(MODULES) ; \
		do $(MAKE) -s -r -C $$f PWD=$(PWD) .mod_clean ; \
	 done
	@echo Cleaning up $(patsubst ./%,%,$(patsubst $(PWD)%,.%,$(shell pwd)))
	@rm -f	*.o *.lst *.d *.mod *.err lib*.a
	@$(USR_CLR)
	
#
#	Dependencies
#	~~~~~~~~~~~~

ifneq "$(objs)" ""
-include $(addsuffix .d,$(basename $(objs)))
endif

ifneq "$(lobj)" ""
-include $(addsuffix .d,$(basename $(lobj)))
endif
