/****************************************************************************/
/* Copyright 2004,  Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	GUI actions box creation												*/
/*	~~~~~~~~~~~~~~~~~~~~~~~~												*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zoltn Kcsi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <types.h>
#include <proxan.h>
#include <job.h>
#include <memory.h>
#include <styles.h>
#include <gtkwrappers.h>

#include "actions.h"
#include "dialog.h"
#include "options.h"
#include "file.h"

/****************************************************************************/
/*						Local definitions									*/
/****************************************************************************/

#define	BIGBORDER		5
#define	BORDER			2

/****************************************************************************/
/*						Prototypes											*/
/****************************************************************************/

static	void	GuiQuit( void );
static	void	GuiLdSett( void );
static	void	GuiStSett( void );
static	void	GuiStDefl( void );
static	void	GuiHelp( void );
static	void	GuiAbout( void );

static	void	GuiSaveOptions( char *file, void *unused );

static	void	OptionGuiQuit( GtkWidget *win );
static	void	GuiOptions( void );

/****************************************************************************/
/*						Local data											*/
/****************************************************************************/

static	GtkWidget	*lopti;				/* Load settings file				*/
static	GtkWidget	*sopti;				/* Save settings file				*/
static	GtkWidget	*opgui;				/* Options GUI window show/hide		*/
static	GtkWidget	*quit;				/* Quit program						*/
static	GtkWidget	*help;				/* Help the user					*/

static	GtkWidget	*opt_win = NULL;	/* Options top window				*/

/****************************************************************************/
/*						Public functions									*/
/****************************************************************************/

/*
*	Creates the action buttons
*	~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*ActionsCreate( void )
{
GtkWidget	*act, *row, *col, *fra;

	act = GuiVbox( FALSE, BIGBORDER );
	GuiLabel( "Actions", big_style, act, FALSE, TRUE );
	GuiBoxAdd( row = GuiHbox( TRUE, BIGBORDER ), act, FALSE, TRUE );
		
	GuiBoxAdd( ActionImage(), 		row, FALSE, TRUE  );
	GuiBoxAdd( ActionsProcopts(),	row, FALSE, TRUE );
	
	GuiBoxAdd( col = GuiVbox( FALSE, BORDER ), 				row, FALSE, TRUE );
	
	GuiLabel( 			"Options", med_style, 				col, FALSE, FALSE );
	lopti = GuiButton(	"Load",		GuiLdSett,	NULL, 		col, FALSE, FALSE );
	sopti = GuiButton(	"Save",		GuiStSett,	NULL, 		col, FALSE, FALSE );
			GuiButton(	"Default",	GuiStDefl,	NULL, 		col, FALSE, FALSE );
	opgui = GuiButton(	"Show",		GuiOptions,	NULL, 		col, FALSE, FALSE );
	
	GuiBoxAdd( col = GuiVbox( FALSE, BORDER ), 				row, FALSE, TRUE );
	
	GuiLabel(			"Misc",		med_style, 				col, FALSE, TRUE );
	quit  = GuiButton(	"Quit",		GuiQuit,	NULL,		col, FALSE, TRUE );
	help  = GuiButton(	"Help",		GuiHelp,	NULL,		col, FALSE, TRUE );
	help  = GuiButton(	"About",	GuiAbout,	NULL,		col, FALSE, TRUE );
	
	fra = GuiFrame( NULL, NULL, act, GTK_SHADOW_OUT );
	return( fra );
}

/****************************************************************************/
/*						Local functions										*/
/****************************************************************************/


/*
*	This function closes down the GUI
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	When this is called, the GuiInit() function will return
*/

static	void	GuiQuit( void )
{
	gtk_main_quit();
}

static	void	GuiLdSett( void )
{
	// Fire up a file selection menu and pass the LoadSettings
	// function as callback for successful selection
	
	GuiFileGetName( "Select options file", NULL, OptionsLoadFile, NULL );
}

static	void	GuiStSett( void )
{
	// Fire up a file selection menu and pass the LoadSettings
	// function as callback for successful selection
	
	GuiFileGetName( "Select options file", NULL, GuiSaveOptions, NULL );
}

static	void	GuiSaveOptions( char *file, void *unused )
{
	if ( ! file ) return;
	
	if ( ! OptionsSaveFile( file ) ) {
	
		GuiDialogBox( "Could not save options" );
	}
}

static	void	GuiStDefl( void )
{
char	*opts;
char	*home;

	if ( ( home = getenv( "HOME" ) ) ) {
	
		opts = Malloc( strlen( home ) + strlen( RESOURCE_FILE ) + 2 );
		sprintf( opts, "%s/%s", home, RESOURCE_FILE );
	
		if ( ! OptionsSaveFile( opts ) ) {
	
			GuiDialogBox( "Could not save options to the default file" );
		}
		
		Free( opts );
	}
	else {
	
		GuiDialogBox( "Could not obtain your home directory" );
	}
}

static	void	GuiHelp( void )
{
	GuiDialogBox( "On-line help is not yet available\n"
				  "Please read the manual or forward your question\n"
				  "to proxan@bendor.com.au" );
}

static	void	GuiAbout( void )
{
	GuiDialogBox( "This will be some nice information box one day" );
}

/*
*	User clicked on the options button
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	GuiOptions( void )
{
	if ( ! opt_win ) {
	
		opt_win = gtk_window_new( GTK_WINDOW_TOPLEVEL );
		gtk_widget_show( opt_win );
		gtk_window_set_title( GTK_WINDOW( opt_win ), "Proxan Options" );
		gtk_container_set_border_width( GTK_CONTAINER( opt_win ), 5 );
		gtk_signal_connect( GTK_OBJECT( opt_win ), "delete_event", 
							GTK_SIGNAL_FUNC( OptionGuiQuit ), NULL );
		gtk_container_add( GTK_CONTAINER( opt_win ), OptionsGuiCreate() );
	}
	else {
	
		gtk_widget_destroy( opt_win );
		OptionsCleanupGui();
		opt_win = NULL;
	}
	
	gtk_label_set_text( GTK_LABEL( GTK_BIN( opgui )->child ), 
						opt_win ? "Hide" : "Show" );
}

/*
*	User wants to destroy the options window
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	OptionGuiQuit( GtkWidget *win )
{
	gtk_widget_destroy( win );
	OptionsCleanupGui();
	opt_win = NULL;
	gtk_label_set_text( GTK_LABEL( GTK_BIN( opgui )->child ), "Show" );
}
