/****************************************************************************/
/* Copyright 2004,  Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	GUI actions box creation												*/
/*	~~~~~~~~~~~~~~~~~~~~~~~~												*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zoltn Kcsi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#include <stdio.h>

#include <types.h>
#include <job.h>
#include <gui.h>
#include <styles.h>
#include <gtkwrappers.h>
#include <dlinklist.h>
#include <dialog.h>
#include <file.h>

#include "actions.h"
#include "options.h"

/****************************************************************************/
/****************************************************************************/

#define	BORDER		5
#define	MAXSTEPS	5

/****************************************************************************/
/****************************************************************************/

static	void	GuiLdImage( void );
static	void	GuiJobOpen( char *name, void *unused );
static	void	GuiPrImage( void );
static	void	GuiStartProcessing( JOB job );
static	int		GuiJobEvent( JOB job, int event, int param );
static	void	ProcoptSig( GtkWidget *but );
static	void	GuiAll( void );
static	void	GuiLock( void );
static	void	JobBroadcast( JOB except, int cmd, int param );
static	void	GuiButtonsSensitive( int sens );

/****************************************************************************/
/****************************************************************************/

static	GtkWidget	*procwid[ MAXSTEPS];
static	int			proc_mask;
static	GtkWidget	*start;				/* Widget for the process button	*/
static	GtkWidget	*oplck;				/* Widget for the opt lock button	*/
static	GtkWidget	*doall;				/* Widget for the do all button		*/
static	DLINK		jobs;				/* Currently active jobs			*/
static	JOB			active;
static	JOB			process;
static	int			locking;
static	int			multi;
static	GtkWidget	*jobname;			/* Name of the current job			*/

/****************************************************************************/
/****************************************************************************/
	
/*
*	Creates the image control buttons
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*ActionImage( void )
{
GtkWidget	*box;
GtkWidget	*tbx;

int			i;

	// Initialise variables
	
	DlinkInit( &jobs );
	active		= NULL;
	process		= NULL;
	locking		= FALSE;
	multi		= FALSE;
	proc_mask	= 0;
			
	box	  = GuiVbox( FALSE, 2 );
			GuiLabel( "Image", med_style, 				box, FALSE, FALSE );
			GuiButton(	"Open",		GuiLdImage, NULL,	box, FALSE, FALSE );
	start = GuiButton(	"Process", 	GuiPrImage,	NULL,	box, FALSE, FALSE );
			GuiBoxAdd( tbx = GuiVbox( FALSE, 0 ), 		box, FALSE, FALSE );
	doall = GuiToggle(	"Do all",	GuiAll,		NULL,	tbx, FALSE, FALSE );
	oplck = GuiToggle(	"Opt.lock", GuiLock,	NULL, 	tbx, FALSE, FALSE );
	
	// We don't have an image yet, make the process button inactive
	
	gtk_widget_set_sensitive( start, FALSE );
	return( box );
}

/*
*	Creates the processing level buttons
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*ActionsProcopts( void )
{
GtkWidget	*box;

	box = GuiVbox( FALSE, 0 );
	GuiLabel( "Process", med_style, box, FALSE, FALSE );
	
	procwid[ 0 ] = GuiToggle( "Colour",  ProcoptSig, NULL, box, FALSE, FALSE );
	procwid[ 1 ] = GuiToggle( "Gamma",   ProcoptSig, NULL, box, FALSE, FALSE );
	procwid[ 2 ] = GuiToggle( "Extract", ProcoptSig, NULL, box, FALSE, FALSE );
	procwid[ 3 ] = GuiToggle( "Pairing", ProcoptSig, NULL, box, FALSE, FALSE );
	procwid[ 4 ] = GuiToggle( "Stats",   ProcoptSig, NULL, box, FALSE, FALSE );
	return( box );
}

/*
*	The options call this when the colours change
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	GuiColourChange( void )
{
	JobBroadcast( NULL, JOB_CMD_COLOURS, 0 );	
}

/****************************************************************************/
/****************************************************************************/

/*
*	Loads an image
*	~~~~~~~~~~~~~~
*/

static	void	GuiLdImage( void )
{
	// Fire up a file selection menu and pass the GuiJobOpen
	// function as callback for successful selection
	
	GuiFileGetName( "Select image", NULL, GuiJobOpen, NULL );
}

/*
*	Select all changed state
*	~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	GuiAll( void )
{
	multi = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( doall ) );
}

/*
*	Lock options changed state
*	~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	GuiLock( void )
{
	locking = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( oplck ) );
}


/*
*	The user selected a filename for the job
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	GuiJobOpen( char *name, void *unused )
{
JOB		job;

	if ( ( job = JobCreate( name, GuiJobEvent ) ) ) {
	
		// We managed to create a new job.
		// If no job is processing, then tell the new
		// job that it is now the active one
		
		if ( ! process ) {
	
			gtk_widget_set_sensitive( start, TRUE );
			if ( active ) JobControl( active, JOB_CMD_ACTIVE, FALSE );
			JobControl( job, JOB_CMD_ACTIVE, TRUE );
			JobControl( job, JOB_CMD_ACTIVE_ENABLE, TRUE );
		}
		
		// Add the new job to the job list
		
		DlinkAddTail( &jobs, job );
	}
	else {
	
		GuiDialogBox( "Could not open the file!" );
	}
}

/*
*	User pressed the start button
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	GuiPrImage( void )
{
	if ( process ) {
	
		JobControl( process, JOB_CMD_ABORT, 0 );
	}
	else { 
		
		if ( multi ) {
	
			GuiStartProcessing( DlinkFirst( &jobs ) );
		}
		else if ( active ) {
		
			GuiStartProcessing( active );
		}
	}
}

/*
*	Initiates processing a job
*	~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	GuiStartProcessing( JOB job )
{
	// Command all jobs to disable their activate button
	
	JobBroadcast( NULL, JOB_CMD_ACTIVE_ENABLE, FALSE );
	
	// Freeze the options (if it is active)
	
	OptionsSensitive( FALSE );
	GuiButtonsSensitive( FALSE );

	// Send the job the start command
	
 	JobControl( job, JOB_CMD_OPTION_LOAD, 0 );
 	JobControl( job, JOB_CMD_PROCESS, proc_mask );
}

/*
*	A job is reporting an event or a request
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	For requests it returns TRUE if the request was granted and
*	FALSE if it was denied.
*	For events it always returns TRUE.
*/

int		GuiJobEvent( JOB job, int event, int param )
{
	switch ( event ) {
	
		case JOB_REQ_CLOSE:
		
			// User wants to close the job. If it is processing, 
			// We have to abort and wait until it reports that it
			// finished. Otherwise we can close it
			
			if ( process == job ) {
			
				GuiDialogBox( "You can't close a job while it is processing" );
				return( FALSE );
			}
			else {
			
				if ( job == active ) active = NULL;
				JobControl( job, JOB_CMD_CLOSE, 0 );
				DlinkRemove( &jobs, job );
				
				if ( ! DlinkFirst( &jobs ) ) {
				
					gtk_widget_set_sensitive( start, FALSE );
				}
				
				return( TRUE );
			}
				
		case JOB_STAT_ACTIVE:
		
			// Job became inactive
			
			if ( param ) {
				
				active = job;
				if ( locking ) JobControl( job, JOB_CMD_OPTION_SAVE, 0 );
			}
			else {
			
				if ( active == job ) active = NULL;
			}
			
			return( TRUE );
			
		case JOB_REQ_ACTIVE:
		
			// Job requests to become active

			if ( process ) return( FALSE );
			
			if ( param ) {
			
				if ( active && active != job ) {
				
					JobControl( active, JOB_CMD_ACTIVE, FALSE );
				}
				
				JobControl( job, JOB_CMD_ACTIVE, TRUE );
			}
			else if ( active == job ) {
			
				JobControl( job, JOB_CMD_ACTIVE, FALSE );
			}
							
			return( TRUE );
		
		case JOB_STAT_STARTED:
		
			// The job started processing. We lock its active status until
			// it finishes its work. We also change the Process button to
			// Cancel
			
			process = job;
			gtk_label_set_text( GTK_LABEL( GTK_BIN(start)->child ), "Cancel" );
			break;
			
		case JOB_STAT_FINISHED:
		
			// The job finished processing
			
			if ( multi && DlinkNext( job ) ) {
			
				GuiStartProcessing( DlinkNext( job ) );
			}
			else {
			
				// Allow all jobs to become active
				
				process = NULL;
				gtk_label_set_text( GTK_LABEL( GTK_BIN(start)->child ), 
									"Process" );
				OptionsSensitive( TRUE );
				GuiButtonsSensitive( TRUE );
				JobBroadcast( NULL, JOB_CMD_ACTIVE_ENABLE, TRUE );
			}
			
			break;
				
		case JOB_STAT_ABORTED:
			
			// Job processing was aborted for some reason
			// We unlock all jobs and the options
			
			process = NULL;
			gtk_label_set_text( GTK_LABEL( GTK_BIN(start)->child ), "Process" );
			OptionsSensitive( TRUE );
			GuiButtonsSensitive( TRUE );
			JobBroadcast( NULL, JOB_CMD_ACTIVE_ENABLE, TRUE );
			break;
	}
	
	return( TRUE );
}

/*
*	Signal handler for the processing options
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	ProcoptSig( GtkWidget *but )
{
int		i, sta;

	for ( i = 0 ; i < MAXSTEPS ; i++ ) {
	
		if ( but == procwid[ i ] ) {
		
			sta = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( but ) );
			
			if ( sta && i ) {
			
				gtk_toggle_button_set_active( 
					GTK_TOGGLE_BUTTON( procwid[ i-1 ] ), TRUE );
			}
			
			if ( ! sta && i < MAXSTEPS-1 && procwid[ i+1 ] ) {
			
				gtk_toggle_button_set_active( 
					GTK_TOGGLE_BUTTON( procwid[ i+1 ] ), FALSE );
			}
			
			proc_mask &= ~( 1 << i );
			proc_mask |= sta ? ( 1 << i ) : 0;
		}
	}
}

/*
*	Broadcasts a command to all jobs except one
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	JobBroadcast( JOB except, int cmd, int param )
{
JOB		jp;

	for ( jp = DlinkFirst( &jobs ) ; jp ; jp = DlinkNext( jp ) ) {
	
		if ( jp == except ) continue;
		JobControl( jp, cmd, param );
	}
}

/*
*	Makes various buttons sensitive
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	GuiButtonsSensitive( int sens )
{
int		i;

	for ( i = 0 ; i < MAXSTEPS ; i++ ) {
	
		if ( procwid[ i ] ) gtk_widget_set_sensitive( procwid[ i ], sens );
	}
	
	gtk_widget_set_sensitive( oplck, sens );
	gtk_widget_set_sensitive( doall, sens );
}

