/****************************************************************************/
/* Copyright 2004,  Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	GUI functionality														*/
/*	~~~~~~~~~~~~~~~~~														*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zoltn Kcsi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <setjmp.h>
#include <glib.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>

#include <types.h>
#include <styles.h>
#include <options.h>
#include <gui.h>
#include <progress.h>
#include "progress.h"

/****************************************************************************/
/*						Local data											*/
/****************************************************************************/

static	GtkWidget	*bar;
static	GtkWidget	*fra;
static	GtkWidget	*nam;
static	jmp_buf		state;
static	int			cancel;

/****************************************************************************/
/*						Public interface functions							*/
/****************************************************************************/

/*
*	Sets the filename in the progress box
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters
*		name	The text to write
*
*	Return value
*		-
*/

void	ProgressText( char *name )
{
	gtk_label_set_text( GTK_LABEL( nam ), name );
}

/*
*	This function initialises the progress bar
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters
*		title		The text written onto the progress bar
*
*	Return value
*		TRUE if the process was aborted midway (by means of a longjump)
*		The process should release everything it allocated and report
*		failure
*
*		FALSE when it just returns from its call
*/

void	ProgressInit( char *title, jmp_buf stat )
{
	if ( ! gui ) return;
	memcpy( state, stat, sizeof( state ) );
	gtk_frame_set_label( GTK_FRAME( fra ), title );
	gtk_progress_set_percentage( GTK_PROGRESS( bar ), 0 );
	cancel = FALSE;
}


/*
*	This function updates the progress bar
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters
*		now		The progress indicator's current value
*		last	The progress indicator's end value
*
*	Return value
*		-
*/

void	ProgressUpdate( int now, int last )
{
	if ( ! gui ) return;
	
	if ( cancel ) {
	
		gtk_progress_set_percentage( GTK_PROGRESS( bar ), 0.0 );
		gtk_frame_set_label( GTK_FRAME( fra ), "* ABORTED *" );
		cancel = FALSE;
		longjmp( state, 1 );
	}
	
	if ( now > last ) now = last;
	gtk_progress_set_percentage( GTK_PROGRESS( bar ), (double) now / last );
	while ( gtk_events_pending() ) gtk_main_iteration();
}


/*
*	This function indicates that processing is finished
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters
*		-
*
*	Return value
*		-
*/

void	ProgressDone( void )
{
	if ( ! gui ) return;
	cancel = FALSE;
	gtk_frame_set_label( GTK_FRAME( fra ), "* DONE *" );
	gtk_progress_set_percentage( GTK_PROGRESS( bar ), 1.0 );
}


/*
*	This function indicates that processing should be aborted
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters
*		-
*
*	Return value
*		-
*
*	If there is processing running then the next call to ProgressUpdate()
*	will result in ProgressInit() returning again with a TRUE flag
*	ProgressAbort() in itself does NOT abort the processing!
*/

void	ProgressAbort( void )
{
	if ( ! gui ) return;
	cancel = TRUE;
}

/****************************************************************************/
/*					Module internal interfgace functions					*/
/****************************************************************************/

/*
*	Creates the progress bar
*	~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*ProgbarCreate( void )
{
GtkWidget	*box;
GtkWidget	*lab;
GtkWidget	*out;
GtkWidget	*con;

	if ( ! gui ) return( NULL );
		
	box = gtk_vbox_new( FALSE, 0 );
	out = gtk_frame_new( NULL );
	lab = gtk_label_new( "Progress" );
	nam = gtk_label_new( "" );
	fra = gtk_frame_new( "" );
	bar = gtk_progress_bar_new();
	con	= gtk_vbox_new( FALSE, 0 );
	
	gtk_box_pack_start( GTK_BOX( con ), bar, FALSE, FALSE, 0 );
	gtk_box_pack_start( GTK_BOX( con ), nam, FALSE, FALSE, 0 );
	
	gtk_frame_set_label_align( GTK_FRAME( fra ), 0.5, 0 );
	gtk_frame_set_shadow_type( GTK_FRAME( fra ), GTK_SHADOW_ETCHED_OUT );
	gtk_container_add( GTK_CONTAINER( fra ), con );
	
	gtk_widget_set_style( lab, big_style );

	gtk_frame_set_label_align( GTK_FRAME( out ), 0.5, 0 );
	gtk_frame_set_shadow_type( GTK_FRAME( out ), GTK_SHADOW_OUT );
	
	gtk_box_pack_start( GTK_BOX( box ), lab, FALSE, FALSE, 0 );
	gtk_box_pack_start( GTK_BOX( box ), fra, FALSE, FALSE, 0 );
	gtk_container_add( GTK_CONTAINER( out ), box );
	
	gtk_widget_show( nam );
	gtk_widget_show( con );
	gtk_widget_show( fra );
	gtk_widget_show( lab );
	gtk_widget_show( bar );
	gtk_widget_show( box );
	gtk_widget_show( out );
	
	cancel = FALSE;
	return( out );
}
