/****************************************************************************/
/* Copyright 2004,  Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	GUI functionality														*/
/*	~~~~~~~~~~~~~~~~~														*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zoltn Kcsi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <glib.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>

#include <types.h>
#include <styles.h>
#include "styles.h"

/****************************************************************************/
/*						Public data											*/
/****************************************************************************/

GtkStyle	*sml_style;					/* Style for normal operation		*/
GtkStyle	*med_style;					/* Style for headings				*/
GtkStyle	*big_style;					/* Style for main headings			*/
GtkStyle	*dia_style;					/* Dialogue box style				*/

GdkCursor	*cursor_sml;				/* Small containment cursor			*/
GdkCursor	*cursor_med;				/* Medium containment cursor		*/
GdkCursor	*cursor_lge;				/* Large containment cursor			*/

/****************************************************************************/
/*						Local data											*/
/****************************************************************************/

static	GdkColor	piros = { 0, 0xc0c0, 0xa0a0, 0x9090 };


static	GdkColor	bigforegnd = { 0, 0x6666, 0x3d3d, 0x3d3d };
static	GdkColor	foreground = { 0, 0x3333, 0x0000, 0x0000 };
static	GdkColor	background = { 0, 0xe8e8, 0xd8d8, 0xc0c0 };
static	GdkColor	preselect  = { 0, 0xf8f8, 0xf4f4, 0xd8d8 };
static	GdkColor	insensitive= { 0, 0xc0c0, 0xb0b0, 0xb0b0 };
static	GdkFont		*font = NULL;

static	GdkColor	dia_foreground = { 0, 0xffff, 0xffff, 0x0000 };
static	GdkColor	dia_background = { 0, 0xc0c0, 0x0000, 0x0000 };
static	GdkFont		*dia_font = NULL;

/*
*	Cursor shapes for containment editing
*/

static	char	mask_8[] = {
	
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xe0, 0x0f, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xf0, 0x1f, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xf8, 0x3f, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xfc, 0x7f, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xbc, 0x7a, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x7c, 0x7d, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xbe, 0xfa, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x7c, 0x7d, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xbc, 0x7a, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xfc, 0x7f, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xf8, 0x3f, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xf0, 0x1f, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xe0, 0x0f, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
};

static char	data_8[] = {

	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xe0, 0x0f, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xf0, 0x1e, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x30, 0x1f, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x30, 0x1f, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xf8, 0x30, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xf0, 0x18, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x30, 0x1f, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xf0, 0x1f, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xe0, 0x0f, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
};

static	char	mask_16[] = {

	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xf8, 0x3f, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xfe, 0xff, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x80, 0xff, 0xff, 0x03, 0x00, 0x00, 
	0x00, 0x00, 0xc0, 0xff, 0xff, 0x07, 0x00, 0x00, 
	0x00, 0x00, 0xe0, 0xaf, 0xea, 0x0f, 0x00, 0x00, 
	0x00, 0x00, 0xf0, 0x57, 0xd5, 0x1f, 0x00, 0x00, 
	0x00, 0x00, 0xf0, 0xab, 0xaa, 0x1f, 0x00, 0x00, 
	0x00, 0x00, 0xf8, 0x55, 0x55, 0x3f, 0x00, 0x00, 
	0x00, 0x00, 0xf8, 0xaa, 0xaa, 0x3e, 0x00, 0x00, 
	0x00, 0x00, 0x7c, 0x55, 0x55, 0x7d, 0x00, 0x00, 
	0x00, 0x00, 0xbc, 0xaa, 0xaa, 0x7a, 0x00, 0x00, 
	0x00, 0x00, 0x7c, 0x55, 0x55, 0x7d, 0x00, 0x00, 
	0x00, 0x00, 0xbc, 0xaa, 0xaa, 0x7a, 0x00, 0x00, 
	0x00, 0x00, 0x7c, 0x55, 0x55, 0x7d, 0x00, 0x00, 
	0x00, 0x00, 0xbe, 0xaa, 0xaa, 0xfa, 0x00, 0x00, 
	0x00, 0x00, 0x7c, 0x55, 0x55, 0x7d, 0x00, 0x00, 
	0x00, 0x00, 0xbc, 0xaa, 0xaa, 0x7a, 0x00, 0x00, 
	0x00, 0x00, 0x7c, 0x55, 0x55, 0x7d, 0x00, 0x00, 
	0x00, 0x00, 0xbc, 0xaa, 0xaa, 0x7a, 0x00, 0x00, 
	0x00, 0x00, 0x7c, 0x55, 0x55, 0x7d, 0x00, 0x00, 
	0x00, 0x00, 0xf8, 0xaa, 0xaa, 0x3e, 0x00, 0x00, 
	0x00, 0x00, 0xf8, 0x55, 0x55, 0x3f, 0x00, 0x00, 
	0x00, 0x00, 0xf0, 0xab, 0xaa, 0x1f, 0x00, 0x00, 
	0x00, 0x00, 0xf0, 0x57, 0xd5, 0x1f, 0x00, 0x00, 
	0x00, 0x00, 0xe0, 0xaf, 0xea, 0x0f, 0x00, 0x00, 
	0x00, 0x00, 0xc0, 0xff, 0xff, 0x07, 0x00, 0x00, 
	0x00, 0x00, 0x80, 0xff, 0xff, 0x03, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xfe, 0xff, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xf8, 0x3f, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
};

static char	data_16[] = {

	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xf8, 0x3f, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xfc, 0x7e, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x0f, 0xef, 0x01, 0x00, 0x00, 
	0x00, 0x00, 0x80, 0x0f, 0x8f, 0x03, 0x00, 0x00, 
	0x00, 0x00, 0xc0, 0xf1, 0xf0, 0x07, 0x00, 0x00, 
	0x00, 0x00, 0xc0, 0xf0, 0xf0, 0x06, 0x00, 0x00, 
	0x00, 0x00, 0x60, 0x0f, 0x0f, 0x0f, 0x00, 0x00, 
	0x00, 0x00, 0x70, 0x0f, 0x0f, 0x1f, 0x00, 0x00, 
	0x00, 0x00, 0xf0, 0xf0, 0xf0, 0x18, 0x00, 0x00, 
	0x00, 0x00, 0xf0, 0xf0, 0xf0, 0x18, 0x00, 0x00, 
	0x00, 0x00, 0x30, 0x0f, 0x0f, 0x1f, 0x00, 0x00, 
	0x00, 0x00, 0x30, 0x0f, 0x0f, 0x1f, 0x00, 0x00, 
	0x00, 0x00, 0xf8, 0xf0, 0xf0, 0x30, 0x00, 0x00, 
	0x00, 0x00, 0xf0, 0xf0, 0xf0, 0x18, 0x00, 0x00, 
	0x00, 0x00, 0x30, 0x0f, 0x0f, 0x1f, 0x00, 0x00, 
	0x00, 0x00, 0x30, 0x0f, 0x0f, 0x1f, 0x00, 0x00, 
	0x00, 0x00, 0xf0, 0xf0, 0xf0, 0x18, 0x00, 0x00, 
	0x00, 0x00, 0xf0, 0xf0, 0xf0, 0x1c, 0x00, 0x00, 
	0x00, 0x00, 0x60, 0x0f, 0x0f, 0x0f, 0x00, 0x00, 
	0x00, 0x00, 0xc0, 0x0f, 0x0f, 0x07, 0x00, 0x00, 
	0x00, 0x00, 0xc0, 0xf1, 0xf0, 0x07, 0x00, 0x00, 
	0x00, 0x00, 0x80, 0xf3, 0xf0, 0x03, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x0f, 0xef, 0x01, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xfc, 0x7f, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xf8, 0x3f, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
};

static	char	mask_32[] = {

	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xfe, 0xff, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0xc0, 0xff, 0xff, 0x07, 0x00, 0x00, 
	0x00, 0x00, 0xf8, 0xff, 0xff, 0x3f, 0x00, 0x00, 
	0x00, 0x00, 0xfe, 0xff, 0xff, 0xff, 0x00, 0x00, 
	0x00, 0x00, 0xff, 0xab, 0xaa, 0xff, 0x01, 0x00, 
	0x00, 0xc0, 0x7f, 0x55, 0x55, 0xfd, 0x07, 0x00, 
	0x00, 0xe0, 0xaf, 0xaa, 0xaa, 0xea, 0x0f, 0x00, 
	0x00, 0xf0, 0x57, 0x55, 0x55, 0xd5, 0x1f, 0x00, 
	0x00, 0xf8, 0xab, 0xaa, 0xaa, 0xaa, 0x3f, 0x00, 
	0x00, 0xfc, 0x55, 0x55, 0x55, 0x55, 0x7f, 0x00, 
	0x00, 0xbe, 0xaa, 0xaa, 0xaa, 0xaa, 0xfa, 0x00, 
	0x00, 0x5f, 0x55, 0x55, 0x55, 0x55, 0xf5, 0x01, 
	0x80, 0xaf, 0xaa, 0xaa, 0xaa, 0xaa, 0xea, 0x03, 
	0x80, 0x5f, 0x55, 0x55, 0x55, 0x55, 0xf5, 0x03, 
	0xc0, 0xaf, 0xaa, 0xaa, 0xaa, 0xaa, 0xea, 0x07, 
	0xe0, 0x57, 0x55, 0x55, 0x55, 0x55, 0xd5, 0x0f, 
	0xe0, 0xab, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x0f, 
	0xf0, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x1f, 
	0xf0, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x1e, 
	0xf0, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x1f, 
	0xf8, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x3e, 
	0x78, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x3d, 
	0xf8, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x3e, 
	0x7c, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x7d, 
	0xbc, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x7a, 
	0x7c, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x7d, 
	0xbc, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x7a, 
	0x7c, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x7d, 
	0xbc, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x7a, 
	0x7c, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x7d, 
	0xbe, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xfa, 
	0x7c, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x7d, 
	0xbc, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x7a, 
	0x7c, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x7d, 
	0xbc, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x7a, 
	0x7c, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x7d, 
	0xbc, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x7a, 
	0x7c, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x7d, 
	0xf8, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x3e, 
	0x78, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x3d, 
	0xf8, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x3e, 
	0xf0, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x1f, 
	0xf0, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x1e, 
	0xf0, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x1f, 
	0xe0, 0xab, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x0f, 
	0xe0, 0x57, 0x55, 0x55, 0x55, 0x55, 0xd5, 0x0f, 
	0xc0, 0xaf, 0xaa, 0xaa, 0xaa, 0xaa, 0xea, 0x07, 
	0x80, 0x5f, 0x55, 0x55, 0x55, 0x55, 0xf5, 0x03, 
	0x80, 0xaf, 0xaa, 0xaa, 0xaa, 0xaa, 0xea, 0x03, 
	0x00, 0x5f, 0x55, 0x55, 0x55, 0x55, 0xf5, 0x01, 
	0x00, 0xbe, 0xaa, 0xaa, 0xaa, 0xaa, 0xfa, 0x00, 
	0x00, 0xfc, 0x55, 0x55, 0x55, 0x55, 0x7f, 0x00, 
	0x00, 0xf8, 0xab, 0xaa, 0xaa, 0xaa, 0x3f, 0x00, 
	0x00, 0xf0, 0x57, 0x55, 0x55, 0xd5, 0x1f, 0x00, 
	0x00, 0xe0, 0xaf, 0xaa, 0xaa, 0xea, 0x0f, 0x00, 
	0x00, 0xc0, 0x7f, 0x55, 0x55, 0xfd, 0x07, 0x00, 
	0x00, 0x00, 0xff, 0xab, 0xaa, 0xff, 0x01, 0x00, 
	0x00, 0x00, 0xfe, 0xff, 0xff, 0xff, 0x00, 0x00, 
	0x00, 0x00, 0xf8, 0xff, 0xff, 0x3f, 0x00, 0x00, 
	0x00, 0x00, 0xc0, 0xff, 0xff, 0x07, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xfe, 0xff, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 
};

static char	data_32[] = {

	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xfe, 0xff, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0xc0, 0xff, 0xfe, 0x07, 0x00, 0x00, 
	0x00, 0x00, 0xf0, 0x0f, 0x0f, 0x1f, 0x00, 0x00, 
	0x00, 0x00, 0x3c, 0x0f, 0x0f, 0x7f, 0x00, 0x00, 
	0x00, 0x00, 0xff, 0xf0, 0xf0, 0xf0, 0x01, 0x00, 
	0x00, 0x80, 0xf3, 0xf0, 0xf0, 0xf0, 0x03, 0x00, 
	0x00, 0xc0, 0x0f, 0x0f, 0x0f, 0x0f, 0x07, 0x00, 
	0x00, 0xf0, 0x0f, 0x0f, 0x0f, 0x0f, 0x1f, 0x00, 
	0x00, 0xf8, 0xf0, 0xf0, 0xf0, 0xf0, 0x38, 0x00, 
	0x00, 0xf8, 0xf0, 0xf0, 0xf0, 0xf0, 0x30, 0x00, 
	0x00, 0x0c, 0x0f, 0x0f, 0x0f, 0x0f, 0x6f, 0x00, 
	0x00, 0x0e, 0x0f, 0x0f, 0x0f, 0x0f, 0xef, 0x00, 
	0x00, 0xf7, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0x01, 
	0x00, 0xf3, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0x01, 
	0x80, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x03, 
	0x80, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x03, 
	0xc0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0x06, 
	0xc0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0x06, 
	0x60, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
	0x60, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
	0xe0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0x0c, 
	0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0x18, 
	0x30, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x1f, 
	0x30, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x1f, 
	0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0x18, 
	0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0x18, 
	0x30, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x1f, 
	0x30, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x1f, 
	0xf8, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0x30, 
	0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0x18, 
	0x30, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x1f, 
	0x30, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x1f, 
	0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0x18, 
	0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0x18, 
	0x30, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x1f, 
	0x30, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x1f, 
	0xe0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0x0c, 
	0xe0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0x0c, 
	0x60, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
	0xc0, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x07, 
	0xc0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0x06, 
	0x80, 0xf1, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0x03, 
	0x80, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x03, 
	0x00, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x8f, 0x01, 
	0x00, 0xf7, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0x01, 
	0x00, 0xfe, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0x00, 
	0x00, 0x0c, 0x0f, 0x0f, 0x0f, 0x0f, 0x6f, 0x00, 
	0x00, 0x18, 0x0f, 0x0f, 0x0f, 0x0f, 0x3f, 0x00, 
	0x00, 0xf8, 0xf0, 0xf0, 0xf0, 0xf0, 0x38, 0x00, 
	0x00, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0x1e, 0x00, 
	0x00, 0xc0, 0x0f, 0x0f, 0x0f, 0x0f, 0x07, 0x00, 
	0x00, 0x80, 0x0f, 0x0f, 0x0f, 0x8f, 0x03, 0x00, 
	0x00, 0x00, 0xff, 0xf0, 0xf0, 0xf0, 0x01, 0x00, 
	0x00, 0x00, 0xfc, 0xf0, 0xf0, 0x78, 0x00, 0x00, 
	0x00, 0x00, 0xf0, 0x0f, 0x0f, 0x1f, 0x00, 0x00, 
	0x00, 0x00, 0xc0, 0xff, 0xff, 0x07, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0xfe, 0xff, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
};

/*
*	Cursor colours for containment editing
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	GdkColor	c1 = { 0, 0xffff, 0xffff, 0xffff };
static	GdkColor	c2 = { 0, 0x0000, 0x0000, 0x0000 };

/****************************************************************************/
/*						Interface functions									*/
/****************************************************************************/

/*
*	Creates all the styles that the application uses
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	GuiCreateStyles( void )
{
GdkBitmap	*dmap, *mmap;

	// Error dialog box style
	
	dia_style = gtk_style_copy( gtk_widget_get_default_style() );
	
	gdk_color_alloc( gdk_colormap_get_system(), &dia_foreground );
	gdk_color_alloc( gdk_colormap_get_system(), &dia_background );
	
	dia_font = gdk_font_load( "-*-courier-bold-r-*--18-*" );
	
	dia_style->fg[ GTK_STATE_NORMAL   ]	= dia_foreground;
	dia_style->text[ GTK_STATE_NORMAL ]	= dia_foreground;
	dia_style->bg[ GTK_STATE_NORMAL   ]	= dia_background;
	dia_style->font = dia_font ? dia_font : dia_style->font;

	// Application style
	
	sml_style = gtk_style_copy( gtk_widget_get_default_style() );
	
	gdk_color_alloc( gdk_colormap_get_system(), &bigforegnd  );
	gdk_color_alloc( gdk_colormap_get_system(), &foreground  );
	gdk_color_alloc( gdk_colormap_get_system(), &background  );
	gdk_color_alloc( gdk_colormap_get_system(), &preselect   );
	gdk_color_alloc( gdk_colormap_get_system(), &insensitive );
	gdk_color_alloc( gdk_colormap_get_system(), &piros );
	
	font = gdk_font_load( "-*-helvetica-medium-r-*--12-*" );

	sml_style->fg[ GTK_STATE_NORMAL			]	= foreground;
	sml_style->bg[ GTK_STATE_NORMAL			]	= background;
	sml_style->fg[ GTK_STATE_PRELIGHT    	]	= foreground;
	sml_style->bg[ GTK_STATE_PRELIGHT	 	]	= preselect;
	sml_style->fg[ GTK_STATE_INSENSITIVE 	]	= insensitive;
	sml_style->bg[ GTK_STATE_INSENSITIVE	]	= background;
	sml_style->fg[ GTK_STATE_ACTIVE			]	= foreground;
	sml_style->bg[ GTK_STATE_ACTIVE			]	= piros;
	sml_style->font = font ? font : sml_style->font;
	
	med_style = gtk_style_copy( sml_style );
	font = gdk_font_load( "-*-helvetica-bold-r-*--12-*" );
	med_style->fg[ GTK_STATE_NORMAL			]	= bigforegnd;
	med_style->fg[ GTK_STATE_PRELIGHT    	]	= bigforegnd;
	med_style->fg[ GTK_STATE_ACTIVE			]	= bigforegnd;
	med_style->font = font ? font : sml_style->font;

	big_style = gtk_style_copy( med_style );
	font = gdk_font_load( "-*-helvetica-bold-r-*--14-*" );
	big_style->font = font ? font : sml_style->font;
	
	// Now the cursors for containment editing
	
	dmap		= gdk_bitmap_create_from_data( NULL, data_8, 64, 64 );
	mmap		= gdk_bitmap_create_from_data( NULL, mask_8, 64, 64 );
	cursor_sml	= gdk_cursor_new_from_pixmap( dmap, mmap, &c1, &c2, 32, 32 );
	
	dmap		= gdk_bitmap_create_from_data( NULL, data_16, 64, 64 );
	mmap		= gdk_bitmap_create_from_data( NULL, mask_16, 64, 64 );
	cursor_med	= gdk_cursor_new_from_pixmap( dmap, mmap, &c1, &c2, 32, 32 );
	
	dmap		= gdk_bitmap_create_from_data( NULL, data_32, 64, 64 );
	mmap		= gdk_bitmap_create_from_data( NULL, mask_32, 64, 64 );
	cursor_lge	= gdk_cursor_new_from_pixmap( dmap, mmap, &c1, &c2, 32, 32 );
}
