/****************************************************************************/
/* Copyright 2004,  Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	GUI program top level window creation									*/
/*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~									*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zoltn Kcsi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#include <stdio.h>
#include <glib.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>

#include <types.h>
#include <gui.h>
#include <options.h>
#include <styles.h>
#include <gtkwrappers.h>
#include "actions.h"
#include "progress.h"
#include "styles.h"

/****************************************************************************/
/*							Prototypes										*/
/****************************************************************************/

static	GtkWidget	*LogoCreate( GtkWidget *win, char **xpmd );
static	gint		GuiQuit( GtkWidget *obj, gpointer gdata );

/****************************************************************************/
/*							Local data										*/
/****************************************************************************/

#include "proxan.xpm"

/****************************************************************************/
/*							Public functions								*/
/****************************************************************************/

/*
*	Creates the top level window for the application
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	TopwinCreate( void )
{
GtkWidget	*win;
GtkWidget	*box;
GtkWidget	*hor;

	// Prepare for direct image manipulation
	
	gdk_rgb_init();
	gtk_widget_set_default_colormap( gdk_rgb_get_cmap() );
	gtk_widget_set_default_visual( gdk_rgb_get_visual() );
	
	// Create the styles
	
	GuiCreateStyles();
	gtk_widget_push_style( sml_style );

	// Create the window itself
	
	win = gtk_window_new( GTK_WINDOW_TOPLEVEL );
	gtk_widget_show( win );
	gtk_window_set_title( GTK_WINDOW( win ), "Proximity Analysis" );
	gtk_container_set_border_width( GTK_CONTAINER( win ), 5 );
	gtk_signal_connect( GTK_OBJECT( win ), "delete_event", 
						GTK_SIGNAL_FUNC( GuiQuit ), NULL );
	
	// A vertical box for all major components
	
	box = GuiVbox( FALSE, 5 );
	GuiBoxAdd( LogoCreate( win, proxan ), box, TRUE, TRUE );
	GuiBoxAdd( ProgbarCreate(), box, FALSE, TRUE );
	GuiBoxAdd( ActionsCreate(), box, TRUE, TRUE );
	
	// Put the box in the window
	
	gtk_container_add( GTK_CONTAINER( win ), box );
}

/****************************************************************************/
/*							Local functions									*/
/****************************************************************************/

/*
*	This function closes down the GUI
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	When this is called, the GuiInit() function will return
*/

static	gint	GuiQuit( GtkWidget *obj, gpointer gdata )
{
	gtk_main_quit();
	return( FALSE );
}

/*
*	Creates the pixmap widget in a frame for the logo
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	GtkWidget	*LogoCreate( GtkWidget *win, char **xpmd )
{
GdkBitmap	*msk;
GdkPixmap	*map;
GtkWidget	*wid;
GtkWidget	*fra;

	map = gdk_pixmap_create_from_xpm_d( win->window, &msk, NULL, xpmd );
	wid = gtk_pixmap_new( map, msk );
	gtk_widget_show( wid );
	
	fra = gtk_frame_new( NULL );
	gtk_widget_show( fra );
	gtk_frame_set_label_align( GTK_FRAME( fra ), 0.5, 0 );
	gtk_frame_set_shadow_type( GTK_FRAME( fra ), GTK_SHADOW_OUT );
	gtk_container_add( GTK_CONTAINER( fra ), wid );
	return( fra );
}
