/****************************************************************************/
/* Copyright 2004, � Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	GUI image interface functionality										*/
/*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~										*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zolt�n K�csi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <glib.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>

#include <types.h>
#include <job.h>
#include <styles.h>
#include <objects.h>
#include <distance.h>
#include <gtkwrappers.h>
#include <dialog.h>
#include <file.h>
#include <memory.h>

#include "jobdefs.h"
#include "canvas.h"
#include "buttons.h"
#include "process.h"
#include "jobgui.h"

#define	SQR(x)	((x)*(x))

/****************************************************************************/
/*							Prototypes										*/
/****************************************************************************/

static	GtkWidget	*JobModeMenu( JOB job, GtkWidget *box, int wide, int fill);
static	GtkWidget	*JobModeMenuAdd( JOB job, char *txt );
static	int			JobModeHandler( GtkWidget *wid, JOB job );
static	void		JobModeSwitch( JOB job, void (*func)( JOB ) );
static	void		JobActionClose( GtkWidget *butt, JOB job );
static	void		JobActionActive( GtkWidget *butt, JOB job );
static	void		JobActionSave( GtkWidget *butt, JOB job );
static	void		FileSelected( char *file, void *par );
static	void		FileStats( char *file, void *par );
static	void		JobActionStats( GtkWidget *butt, JOB job );

/****************************************************************************/
/*							Public functions								*/
/****************************************************************************/

/*
*	Creates the user interface
*	~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*JobInterfaceCreate( JOB job )
{
GtkWidget	*box, *panel, *fra;

	// Create a packing box for the buttons
	
	box = GuiVbox( FALSE, 5 );
	gtk_widget_set_usize( box, BUTTONS_HSIZE, BUTTONS_VSIZE );	
	
	panel = GuiVbox( FALSE, 5 );
	GuiLabel( "Main mode", med_style, panel, FALSE, FALSE );
	
	job->butt_actv = GuiToggle( "Selected", JobActionActive, job, 
								panel, FALSE, FALSE );
	
	job->butt_mode = JobModeMenu( job, panel, FALSE, FALSE );
	
	job->butt_clse = GuiButton( "Close", JobActionClose, job, 
								panel, FALSE, FALSE );
								
	job->butt_save = GuiButton( "Save", JobActionSave, job, 
								panel, FALSE, FALSE );
								
	job->butt_data = GuiButton( "Data", JobActionStats, job, 
								panel, FALSE, FALSE );
					
	fra = GuiFrame( NULL, NULL, panel, GTK_SHADOW_OUT );
	
	GuiBoxAdd( fra, box, FALSE, FALSE );
	job->pbox_cont = JobContPanel( job, box, FALSE, FALSE );
	job->pbox_proc = JobProcPanel( job, box, FALSE, FALSE );
	job->pbox_stat = JobStatPanel( job, box, FALSE, FALSE );
	job->pbox_csel = JobCselPanel( job, box, FALSE, FALSE );
	job->mode = MODE_NONE;
	gtk_option_menu_set_history( GTK_OPTION_MENU( job->mode_menu ), 2 );
	JobModeSwitch( job, JobProcEnter );
	return( box );
}

/*
*	Enables the active button
*	~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	JobButtonActiveEnable( JOB job, int enable )
{
	if ( ! job->butt_actv ) return;
	gtk_widget_set_sensitive( job->butt_actv, enable );
}

/*
*	Sets the state of the active button
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	JobButtonActiveState( JOB job, int active )
{
	if ( ! job->butt_actv ) return;
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(job->butt_actv), active );
}

/****************************************************************************/
/*						Local functions										*/
/****************************************************************************/

/*
*	Signal handler for the close button
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	JobActionClose( GtkWidget *butt, JOB job )
{
	if ( job->signal ) (*job->signal)( job, JOB_REQ_CLOSE, 0 );
}

/*
*	Signal handler for the activate button
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	JobActionActive( GtkWidget *butt, JOB job )
{
	if ( job->signal ) {
	
		(*job->signal)( job, JOB_REQ_ACTIVE, 
		gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( butt ) ) );
	}
}

/*
*	Creates the mode menu
*	~~~~~~~~~~~~~~~~~~~~~
*/

static	GtkWidget	*JobModeMenu( JOB job, GtkWidget *box, int wide, int fill )
{
	job->mode_list = gtk_menu_new();
	job->mode_menu = gtk_option_menu_new();
	gtk_widget_show( job->mode_list );
	gtk_widget_show( job->mode_menu );
	gtk_option_menu_set_menu( GTK_OPTION_MENU( job->mode_menu ), 
							  job->mode_list );
	
	job->mode_grup = NULL;
	job->mode_csel = JobModeMenuAdd( job, "Colour pick" );
	job->mode_cont = JobModeMenuAdd( job, "Containment" );
	job->mode_proc = JobModeMenuAdd( job, "Processing" );
	job->mode_stat = JobModeMenuAdd( job, "Statistics" );
	
	GuiBoxAdd( job->mode_menu, box, wide, fill );
	return( job->mode_menu );
}

/*
*	Adds a menu item to the mode menu
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	GtkWidget	*JobModeMenuAdd( JOB job, char *txt )
{
GtkWidget	*item;

	item = gtk_radio_menu_item_new_with_label( job->mode_grup, txt );
	gtk_widget_show( item );
	job->mode_grup = gtk_radio_menu_item_group( GTK_RADIO_MENU_ITEM( item ) );
	gtk_menu_append( GTK_MENU( job->mode_list ), item );
	gtk_signal_connect( GTK_OBJECT( item ), "activate",
						GTK_SIGNAL_FUNC( JobModeHandler ), (gpointer) job );
	return( item );
}

/*
*	Signal handler to activate a mode
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	int		JobModeHandler( GtkWidget *wid, JOB job )
{
	if ( wid != gtk_menu_get_active( 
					GTK_MENU( 
						gtk_option_menu_get_menu( 
							GTK_OPTION_MENU( job->mode_menu ) ) ) ) ) {
		return( TRUE );
	}
	
	if ( wid == job->mode_cont && job->mode != MODE_CONT ) {
	
		// Containment mode is selected
		
		JobModeSwitch( job, JobContEnter );
	}
	
	if ( wid == job->mode_proc && job->mode != MODE_PROC ) {
	
		// Processing mode is selected
		
		JobModeSwitch( job, JobProcEnter );
	}
	
	if ( wid == job->mode_stat && job->mode != MODE_STAT ) {
	
		// Statistics mode is selected
		
		JobModeSwitch( job, JobStatEnter );
	}
	
	if ( wid == job->mode_csel && job->mode != MODE_CSEL ) {
	
		// Statistics mode is selected
		
		JobModeSwitch( job, JobCselEnter );
	}
	
	return( TRUE );
}

/*
*	Switches to the mode requested
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	JobModeSwitch( JOB job, void (*func)( JOB ) )
{
	JobProcLeave( job );
	JobContLeave( job );
	JobStatLeave( job );
	JobCselLeave( job );
	(*func)( job );
}

/*
*	Handler for the save button
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	JobActionSave( GtkWidget *butt, JOB job )
{
	
	GuiFileGetName( "Save image as...", NULL, FileSelected, job );
}

/*
*	Filename is selected
*	~~~~~~~~~~~~~~~~~~~~
*/

static	void	FileSelected( char *file, void *par )
{
char	*p;
JOB		job = (JOB) par;

	if ( ! TiffWrite( job->rawrgb, job->cys, job->cxs, file ) ) {
		
		GuiDialogBox( "Could not save the file!" );
	}
}

/*
*	Signal handler for the save spreadsheet button
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	JobActionStats( GtkWidget *butt, JOB job )
{
	if ( ! DlinkFirst( &job->act ) || ! DlinkFirst( &job->ref ) ) {
		
		GuiDialogBox( "No data to save!" );
		return;
	}
	
	GuiFileGetName( "File name (should end with .csv)", NULL, FileStats, job );
}
 
/*
*	Statistics data filename is selected
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	FileStats( char *file, void *par )
{
JOB		job = (JOB) par;
void	*pa, *pr;
FILE	*f;
double	x, y, fac;
int		i;

	fac = (double) job->opt.pair_mic / job->opt.pair_pix;

	if ( ! ( f = fopen( file, "w" ) ) ) {
		
		GuiDialogBox( "Can't open file for writing!" );
		return;
	}
	
	pa = DlinkFirst( &job->act );
	pr = DlinkFirst( &job->ref );
	
	fprintf( f, "\"Item\",\"Act.Size\",\"Act.X\",\"Act.Y\"," );
	fprintf( f, "\"Ref.Size\",\"Ref.X\",\"Ref.Y\"\n" );
	
	for ( i = 1 ; pa || pr ; i++ ) {
		
		fprintf( f, "%d,", i );
		
		if ( pa ) {
			
			fprintf( f, "%f,", ObjectArea( pa ) * fac * fac ); 
			ObjectCentre( pa, &x, &y );
			fprintf( f, "%f,%f,", x * fac, y * fac );
			pa = DlinkNext( pa );
		}
		else {
			
			fprintf( f, ",,," );
		}
		
		if ( pr ) {
			
			fprintf( f, "%f,", ObjectArea( pr ) * fac * fac );
			ObjectCentre( pr, &x, &y );
			fprintf( f, "%f,%f,", x * fac, y * fac );
			pr = DlinkNext( pr );
		}
		else {
			
			fprintf( f, ",,," );
		}
		
		fprintf( f, "\n" );
	}
	
	fclose( f );
}

