/****************************************************************************/
/* Copyright 2004,  Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	GUI colour select functionality											*/
/*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~											*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zoltn Kcsi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <glib.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>

#include <types.h>
#include <job.h>
#include <styles.h>
#include <objects.h>
#include <process.h>
#include <distance.h>
#include <gtkwrappers.h>
#include <options.h>
#include <memory.h>
#include "jobdefs.h"
#include "canvas.h"
#include "buttons.h"
#include "process.h"
#include "jobgui.h"

/****************************************************************************/
/*							Local macros									*/
/****************************************************************************/

#define	SQR( x )	((x)*(x))
#define	SIG( x )	((x)?(x)<0?-1:1:0)

/****************************************************************************/
/*							Prototypes										*/
/****************************************************************************/

static	int		JobMouse( GtkWidget *wid, GdkEventButton *event, JOB job );
static	void	JobPartialRefresh( JOB job, RECT *box );
static	void	JobMarkCsel( JOB job, int y, int x, int val );
static	void	JobExpose( GtkWidget *wid, GdkEventExpose *ev, JOB job );
static	void	JobCursorReset( GtkWidget *butt, JOB job );
static	void	JobCursorAct( GtkWidget *butt, JOB job );
static	void	JobCursorRef( GtkWidget *butt, JOB job );

/****************************************************************************/
/*						Module public functions								*/
/****************************************************************************/

/*
*	Creates the box that contains the colour select operation items
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*JobCselPanel( JOB job, GtkWidget *box, int wide, int fill )
{
GtkWidget	*panel;
GtkWidget	*drw;
GtkWidget	*fra;

	// Create a packing box for the panels
	
	panel = GuiVbox( FALSE, 5 );
	
	drw = gtk_drawing_area_new();
	gtk_widget_show( drw );
	gtk_widget_set_usize( drw, 50, 20 );
	gtk_signal_connect( GTK_OBJECT( drw ), "expose_event",
						GTK_SIGNAL_FUNC( JobExpose ), job );
	fra = GuiFrame( NULL, NULL, drw, GTK_SHADOW_IN );
	GuiBoxAdd( fra, panel, FALSE, FALSE );
	
	GuiButton( "Reset", JobCursorReset, job, panel, FALSE, FALSE );
	GuiButton( "Active", JobCursorAct, job, panel, FALSE, FALSE );
	GuiButton( "Reference", JobCursorRef, job, panel, FALSE, FALSE );
	
	GuiBoxAdd( panel, box, wide, fill );
	job->mouse_cid = 0;
	job->csel_cbox = drw;
	gtk_widget_hide( panel );
	return( panel );
}

/*
*	This function is called when the colour select mode is entered
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	JobCselEnter( JOB job )
{
int		y, x;
PIXEL	pix;
OBJECT	obj;
RECT	box;

	if ( job->mode == MODE_CSEL ) return;
	
	// Show interface
	
	gtk_widget_show( job->pbox_csel );
	
	// Connect the mouse events to the event handler
	
	job->mouse_cid = gtk_signal_connect( GTK_OBJECT( job->canvas ), 
										 "button_press_event",
										 GTK_SIGNAL_FUNC( JobMouse ), job );
	
	job->csel[ R ] = 0;
	job->csel[ G ] = 0;
	job->csel[ B ] = 0;
	job->ccnt = 0;
	job->mode = MODE_CSEL;
	
	gtk_widget_hide( job->csel_cbox );
	gtk_widget_show( job->csel_cbox );
	
	box.tly = 0;
	box.tlx = 0;
	box.bry = job->ys - 1;
	box.brx = job->xs - 1;
	JobPartialRefresh( job, &box );
}

/*
*	This function is called when the colour select mode is left
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	JobCselLeave( JOB job )
{
	if ( job->mode != MODE_CSEL ) return;
	
	// Disconnect the mouse events to the event handler
	
	if ( job->mouse_cid ) {
	
		gtk_signal_disconnect( GTK_OBJECT( job->canvas ), job->mouse_cid );
		job->mouse_cid = 0;
	}
					
	// Hide interface
	
	gtk_widget_hide( job->pbox_csel );
	
	job->mode = MODE_NONE;
}

/*
*	Updates the foreground for the given area and requests a
*	display update for the selected range
*/

static	void	JobPartialRefresh( JOB job, RECT *box )
{
int		y, x, i;

	for ( y = box->tly ; y <= box->bry ; y++ ) {
	
		i = ( job->xs * y + box->tlx ) * 3;
		
		for ( x = box->tlx ; x <= box->brx ; x++, i += 3 ) {
	
			job->rawrgb[ i + 0 ] = job->orig->im[ R ][ y ][ x ];
			job->rawrgb[ i + 1 ] = job->orig->im[ G ][ y ][ x ];
			job->rawrgb[ i + 2 ] = job->orig->im[ B ][ y ][ x ];
		}
	}
	
	gtk_widget_queue_draw_area( job->canvas, 
								box->tlx, box->tly, 
								box->brx - box->tlx, box->bry - box->tly );
}

/****************************************************************************/
/*							Local function									*/
/****************************************************************************/

/*
*	Signal handler for the reset button
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	JobCursorReset( GtkWidget *butt, JOB job )
{
	job->csel[ R ] = 0;
	job->csel[ G ] = 0;
	job->csel[ B ] = 0;
	job->ccnt = 0;
	gtk_widget_hide( job->csel_cbox );
	gtk_widget_show( job->csel_cbox );
}

/*
*	Signal handler for setting the active colour
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	JobCursorAct( GtkWidget *butt, JOB job )
{
int		r, g, b;

	if ( ! job->ccnt ) return;
	
	r = job->csel[ R ] / job->ccnt;
	g = job->csel[ G ] / job->ccnt;
	b = job->csel[ B ] / job->ccnt;
	
	job->new.acol = ( r << 16 ) | ( g << 8 ) | b;
	OptionValueSet( optPreColAct, &job->new.acol );
}

/*
*	Signal handler for setting the reference colour
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	JobCursorRef( GtkWidget *butt, JOB job )
{
int		r, g, b;

	if ( ! job->ccnt ) return;
	
	r = job->csel[ R ] / job->ccnt;
	g = job->csel[ G ] / job->ccnt;
	b = job->csel[ B ] / job->ccnt;
	
	job->new.rcol = ( r << 16 ) | ( g << 8 ) | b;
	OptionValueSet( optPreColRef, &job->new.rcol );
}

/*
*	Signal handler for mouse events
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	int		JobMouse( GtkWidget *wid, GdkEventButton *event, JOB job )
{
int				x, y, dx, dy;
int				r, g, b, n;
RECT			box;

	x = event->x;
	y = event->y;
	
	// See if we have to process this event at all
	
	if ( event->type != GDK_BUTTON_PRESS ) return( TRUE );
	if ( x < 0 || x >= job->xs ) return( TRUE );
	if ( y < 0 || y >= job->xs ) return( TRUE );
	if ( event->button != 1 ) return( TRUE );
	
	// We have to process the event. Collect the information.
	
	r = g = b = n = 0;
	
	for ( dy = -2 ; dy <= 2 ; dy++ ) {
	
		if ( y + dy < 0 || y + dy >= job->xs ) continue;
		
		for ( dx = -2 ; dx <= 2 ; dx++ ) {
		
			if ( x + dx < 0 || x + dx >= job->xs ) continue;
			if ( ABS( dx ) + ABS( dy ) == 4 ) continue;
			
			r += job->orig->im[ R ][ y ][ x ];
			g += job->orig->im[ G ][ y ][ x ];
			b += job->orig->im[ B ][ y ][ x ];
			n++;
		}
	}
	
	// Update the current colour of the job
	
	job->csel[ R ] += r / n;
	job->csel[ G ] += g / n;
	job->csel[ B ] += b / n;
	job->ccnt++;
	
	gtk_widget_hide( job->csel_cbox );
	gtk_widget_show( job->csel_cbox );
	
	return( TRUE );
}

////////////////////////////////////////////////////////////////////////

/*
*	Expose event handler for colour selections
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	JobExpose( GtkWidget *wid, GdkEventExpose *ev, JOB job )
{
char	rgb[ 3 ];
int		i;

	if ( job->ccnt ) {
	
		rgb[ 0 ] = job->csel[ R ] / job->ccnt;
		rgb[ 1 ] = job->csel[ G ] / job->ccnt;
		rgb[ 2 ] = job->csel[ B ] / job->ccnt;
	}
	else {
	
		rgb[ 0 ] = 0;
		rgb[ 1 ] = 0;
		rgb[ 2 ] = 0;
	}
	
	for ( i = ev->area.x ; i < ev->area.x + ev->area.width ; i++ ) {
	
		gdk_draw_rgb_image( wid->window, wid->style->fg_gc[ GTK_STATE_NORMAL ],
							i, ev->area.y, 1, ev->area.height,
							GDK_RGB_DITHER_MAX, rgb, 0 );
	}					
}

