/****************************************************************************/
/* Copyright 2004,  Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	GUI image containment functionality										*/
/*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~										*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zoltn Kcsi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <glib.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>

#include <types.h>
#include <job.h>
#include <styles.h>
#include <objects.h>
#include <process.h>
#include <distance.h>
#include <gtkwrappers.h>
#include <memory.h>
#include "jobdefs.h"
#include "canvas.h"
#include "buttons.h"
#include "process.h"
#include "jobgui.h"

/****************************************************************************/
/*							Local macros									*/
/****************************************************************************/

#define	SQR( x )	((x)*(x))
#define	SIG( x )	((x)?(x)<0?-1:1:0)

/****************************************************************************/
/*							Prototypes										*/
/****************************************************************************/

static	GtkWidget	*JobCursorCreate( JOB job );
static	void		JobCursorSig( GtkWidget *butt, JOB job );
static	void		JobCursorClear( GtkWidget *butt, JOB job );
static	void		JobCursorRestore( GtkWidget *butt, JOB job );
static	int			JobMouse( GtkWidget *wid, GdkEventMotion *event, JOB job );
static	void		JobPartialRefresh( JOB job, RECT *box );
static	void		JobMarkCont( JOB job, int y, int x, int val );

/****************************************************************************/
/*						Module public functions								*/
/****************************************************************************/

/*
*	Creates the box that contains the containment operation items
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*JobContPanel( JOB job, GtkWidget *box, int wide, int fill )
{
GtkWidget	*panel;

	// Create a packing box for the panels
	
	panel = GuiVbox( FALSE, 5 );
	GuiBoxAdd( JobCursorCreate( job ), panel, FALSE, FALSE );
	GuiButton( "Clear all", JobCursorClear, job, panel, FALSE, FALSE );
	GuiButton( "Restore", JobCursorRestore, job, panel, FALSE, FALSE );
	GuiBoxAdd( panel, box, wide, fill );
	job->mouse_cid = 0;
	gtk_widget_hide( panel );
	return( panel );
}

/*
*	This function is called when the containment mode is entered
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	JobContEnter( JOB job )
{
int		y, x;
PIXEL	pix;
OBJECT	obj;

	if ( job->mode == MODE_CONT ) return;
	
	// Show interface
	
	gtk_widget_show( job->pbox_cont );
	
	// Set mouse mask to medium
	
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( job->butt_cur1 ), TRUE );
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( job->butt_cur2 ), TRUE );
	
	// Connect the mouse events to the event handler
	
	job->mouse_cid = gtk_signal_connect( GTK_OBJECT( job->canvas ), 
										 "motion_notify_event",
										 GTK_SIGNAL_FUNC( JobMouse ), job );
	
	// For each element in the containment mask remember the old value
	// and set the current value to that
	
	for ( y = 0 ; y < job->ys ; y++ ) memset( job->mark[ y ], 0, job->xs );
	
	for ( obj = DlinkFirst( &job->con ) ; obj ; obj = DlinkNext( obj ) ) {
	
		for ( pix = ObjectFirstPixel( obj, FALSE, &y, &x ) ; 
			  pix ;
			  pix = ObjectNextPixel( obj, pix, &y, &x ) ) {
		
			job->mark[ y ][ x ] = 3;
		}
		
		for ( pix = ObjectFirstPixel( obj, TRUE, &y, &x ) ; 
			  pix ;
			  pix = ObjectNextPixel( obj, pix, &y, &x ) ) {
		
			job->mark[ y ][ x ] = 3;
		}
	}
	
	JobCursorRestore( NULL, job );
	job->mode = MODE_CONT;
}

/*
*	This function is called when the containment mode is left
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	JobContLeave( JOB job )
{
int		y, x;
int		flag;
ubyte	**temp;

	if ( job->mode != MODE_CONT ) return;
	
	// Disconnect the mouse events to the event handler
	
	if ( job->mouse_cid ) {
	
		gtk_signal_disconnect( GTK_OBJECT( job->canvas ), job->mouse_cid );
		job->mouse_cid = 0;
	}
					
	// Restore pointer for the canvas
	
	gdk_window_set_cursor( job->canvas->window, NULL );
	
	// Hide interface
	
	gtk_widget_hide( job->pbox_cont );
	
	// If there was a change of the map, turn it into an object list
	// and remove all object related job results
	
	for ( flag = FALSE, y = 0 ; y < job->ys ; y++ ) {
	
		for ( x = 0 ; x < job->xs ; x++ ) {
		
			if ( job->mark[ y ][ x ] == 2 || job->mark[ y ][ x ] == 1 ) {
			
				flag = TRUE;
			}
			
			job->mark[ y ][ x ] &= 1;
		}
	}
	
	if ( flag ) {
	
		// ObjectMapped needs a byte-sized object, so we have to
		// transform job->mark into a temporary byte array
		
		temp = MallocArray( job->ys, job->xs, 1 );
				
		for ( y = 0 ; y < job->ys ; y++ ) {
	
			for ( x = 0 ; x < job->xs ; x++ ) {
			
				temp[ y ][ x ] = job->mark[ y ][ x ];
			}
		}
		
		ObjectMapped( &job->con, temp, job->ys, job->xs );
		ObjectQuickfix( &job->con, job->ys, job->xs );
		Free( temp );
		JobConfinementChanged( job );
	}
	
	job->mode = MODE_NONE;
}

/****************************************************************************/
/*							Local function									*/
/****************************************************************************/

/*
*	Creates the cursor selection box
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	GtkWidget	*JobCursorCreate( JOB job )
{
GtkWidget	*fra;
GtkWidget	*box;
GtkWidget	*rad;
GSList		*grp;

	grp = NULL;
	box = GuiVbox( FALSE, 0 );
	GuiLabel( "Brush size", med_style, box, FALSE, FALSE );
	
	job->butt_cur1 = GuiRadio( "Small", &grp, JobCursorSig, job, 
							   box, FALSE, FALSE );

	job->butt_cur2 = GuiRadio( "Medium", &grp, JobCursorSig, job, 
							   box, FALSE, FALSE );

	job->butt_cur4 = GuiRadio( "Large", &grp, JobCursorSig, job, 
							   box, FALSE, FALSE );

	fra = GuiFrame( NULL, NULL, box, GTK_SHADOW_OUT );
	
	return( fra );
}

/*
*	Signal handler for the cursor size buttons
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	JobCursorSig( GtkWidget *butt, JOB job )
{
	if ( ! gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( butt ) ) ) return;
	
	if ( butt == job->butt_cur1 ) {
	
		gdk_window_set_cursor( job->canvas->window, cursor_sml );
		job->cont_size = 7;
	}
	
	if ( butt == job->butt_cur2 ) {
	
		gdk_window_set_cursor( job->canvas->window, cursor_med );
		job->cont_size = 15;
	}
	
	if ( butt == job->butt_cur4 ) {
	
		gdk_window_set_cursor( job->canvas->window, cursor_lge );
		job->cont_size = 31;
	}
}

/*
*	Signal handler for the clear all button
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	JobCursorClear( GtkWidget *butt, JOB job )
{
int		y, x;
RECT	box;

	for ( y = 0 ; y < job->ys ; y++ ) {
	
		for ( x = 0 ; x < job->xs ; x++ ) {
		
			job->mark[ y ][ x ] = ( job->mark[ y ][ x ] & 2 ) ? 2 : 0;
		}
	}
	
	box.tly = 0;
	box.tlx = 0;
	box.bry = job->ys - 1;
	box.brx = job->xs - 1;
	JobPartialRefresh( job, &box );
}

/*
*	Signal handler for the restore button
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	JobCursorRestore( GtkWidget *butt, JOB job )
{
int		y, x;
RECT	box;

	for ( y = 0 ; y < job->ys ; y++ ) {
	
		for ( x = 0 ; x < job->xs ; x++ ) {
		
			job->mark[ y ][ x ] = ( job->mark[ y ][ x ] & 2 ) ? 3 : 0;
		}
	}
	
	box.tly = 0;
	box.tlx = 0;
	box.bry = job->ys - 1;
	box.brx = job->xs - 1;
	JobPartialRefresh( job, &box );
}

/*
*	Signal handler for mouse events
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	int		JobMouse( GtkWidget *wid, GdkEventMotion *event, JOB job )
{
int				x, y;
GdkModifierType	s;
RECT			box;

	if ( event->is_hint ) {
	
		gdk_window_get_pointer( event->window, &x, &y, &s );
	}
	else {
	
		x = event->x;
		y = event->y;
		s = event->state;
	}
	
	// If the event coordinates are out of the image, forget it
	
	if ( y < 0 || y > job->ys || x < 0 || x > job-> xs ) {
	
		return( TRUE );
	}
	
	// If the mouse is not pressed, we're not interested
	// We do clear the last coordinates fields, however
	
	if ( ! ( s & ( GDK_BUTTON1_MASK | GDK_BUTTON3_MASK ) ) ) {
	
		job->cont_oldx = -1;
		job->cont_oldy = -1;
		return( TRUE );
	}
	
	if ( ( s & GDK_BUTTON1_MASK ) && ! ( s & GDK_BUTTON3_MASK ) ) {
	
		// Mark area between the last and current points
		
		JobMarkCont( job, y, x, 1 );
		
	}
		
	if ( ! ( s & GDK_BUTTON1_MASK ) && ( s & GDK_BUTTON3_MASK ) ) {
	
		// Mark area between the last and current points
		
		JobMarkCont( job, y, x, 0 );
		
	}
		
	return( TRUE );
}

/*
*	Sets the given containment area according to the current cursor
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	JobMarkCont( JOB job, int y, int x, int val )
{
int		dy, yo, ya, yc, ys;
int		dx, xo, xa, xc, xs;
int		st;
RECT	box;

	// We have to trace the cursor between its last point and
	// its oldest point. We use the Bresenham algorithm, although
	// not the classic implementation.
	
	if ( job->cont_oldx == -1 || job->cont_oldy == -1 ) {
	
		job->cont_oldy = y;
		job->cont_oldx = x;
	}
	
	ys = job->cont_oldy;
	xs = job->cont_oldx;
	dy = ys - y;
	dx = xs - x;
	st = MAX( ABS( dx ), ABS( dy ) );
	xc = st / 2;
	yc = st / 2;
		
	box.tly = job->ys;
	box.tlx = job->xs;
	box.bry = 0;
	box.brx = 0;
	
	job->cont_oldy = y;
	job->cont_oldx = x;

	do {
	
		for ( yo = -job->cont_size ; yo <= job->cont_size ; yo++ ) {
		
			ya = yo + y;
			if ( ya < 0 || ya >= job->ys ) continue;
			
			for ( xo = -job->cont_size ; xo <= job->cont_size ; xo++ ) {
		
				xa = xo + x;
				
				if ( xa < 0 || xa >= job->xs ) continue;
			 	if ( SQR( yo ) + SQR( xo ) > SQR( job->cont_size ) ) continue;
				
				job->mark[ ya ][ xa ] &= ~1;
				job->mark[ ya ][ xa ] |= val;
				
				if ( box.tly > ya ) box.tly = ya;
				if ( box.bry < ya ) box.bry = ya;
				if ( box.tlx > xa ) box.tlx = xa;
				if ( box.brx < xa ) box.brx = xa;
			}
		}
		
		yc += ABS( dy );
		xc += ABS( dx );
		
		if ( yc >= st ) {
		
			yc -= st;
			y  += SIG( dy );
		}
		
		if ( xc >= st ) {
		
			xc -= st;
			x  += SIG( dx );
		}
		
	} while ( y != ys || x != xs );
	
	JobPartialRefresh( job, &box );
}

/*
*	Updates the foreground for the given area and requests a
*	display update for the selected range
*/

static	void	JobPartialRefresh( JOB job, RECT *box )
{
int		y, x, i;

	for ( y = box->tly ; y <= box->bry ; y++ ) {
	
		i = ( job->xs * y + box->tlx ) * 3;
		
		for ( x = box->tlx ; x <= box->brx ; x++, i += 3 ) {
	
			if ( ( job->mark[ y ][ x ] & 1 ) && ( ( x ^ y ) & 1 ) ) {
			
				if ( ( ( x >> 1 ) ^ y ) & 2 ) {
				
					job->rawrgb[ i + 0 ] = 0xff;
					job->rawrgb[ i + 1 ] = 0xff;
					job->rawrgb[ i + 2 ] = 0xff;
				}
				else {
				
					job->rawrgb[ i + 0 ] = 0;
					job->rawrgb[ i + 1 ] = 0;
					job->rawrgb[ i + 2 ] = 0;
				}
			}
			else {
			
				job->rawrgb[ i + 0 ] = job->orig->im[ R ][ y ][ x ];
				job->rawrgb[ i + 1 ] = job->orig->im[ G ][ y ][ x ];
				job->rawrgb[ i + 2 ] = job->orig->im[ B ][ y ][ x ];
			}
		}
	}
	
	gtk_widget_queue_draw_area( job->canvas, 
								box->tlx, box->tly, 
								box->brx - box->tlx, box->bry - box->tly );
}
