/****************************************************************************/
/* Copyright 2004, © Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	Graph generation														*/
/*	~~~~~~~~~~~~~~~~														*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zoltán Kócsi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <glib.h>
#include <math.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>

#include <types.h>
#include <job.h>
#include <styles.h>
#include <objects.h>
#include <distance.h>
#include <gtkwrappers.h>
#include <memory.h>
#include <stats.h>
#include <options.h>
#include "jobdefs.h"
#include "canvas.h"
#include "buttons.h"
#include "process.h"
#include "jobgui.h"

/****************************************************************************/
/*							Local definitions								*/
/****************************************************************************/

#define	FSIZE	7
#define	SMUDGE	3
#define	GAMMA	0.6

/****************************************************************************/
/*							Prototypes										*/
/****************************************************************************/

static	void	JobDrawCurve( double *data, int **array, int *max,
							  double hiv, int xs, int ys, int smudge );
static	void	JobDrawDot( int **array, int x, int y, int xs, int ys, 
							int s, int *m );
static	void	JobGraphBlit( JOB job, int **array, int max,
							  int x0, int y0, int xs, int ys, int col );
static	double	JobGraphLimits( double *lo, double *hi );
static	void	JobGraphNumber( JOB job, char *num, int ral, int yp, int xp );

static	const	char	*nums[ 13 ][ 7 ] = {

	{
		"..#..",
		".#.#.",
		"#...#",
		"#...#",
		"#...#",
		".#.#.",
		"..#.."
	},
	
	{
		"..#..",
		".##..",
		"#.#..",
		"..#..",
		"..#..",
		"..#..",
		"..#.."
	},
	
	{
		".###.",
		"#...#",
		"....#",
		"...#.",
		"..#..",
		".#...",
		"#####"
	},
	
	{
		".###.",
		"#...#",
		"....#",
		"..##.",
		"....#",
		"#...#",
		".###."
	},
	
	{
		"...#.",
		"..##.",
		".#.#.",
		"#####",
		"...#.",
		"...#.",
		"...#."
	},
	
	{
		"#####",
		"#....",
		"####.",
		"....#",
		"....#",
		"#...#",
		".###."
	},
	
	{
		".###.",
		"#...#",
		"#....",
		"####.",
		"#...#",
		"#...#",
		".###."
	},
	
	{
		"#####",
		"....#",
		"...#.",
		"..#..",
		"..#..",
		"..#..",
		"..#.."
	},
	
	{
		".###.",
		"#...#",
		"#...#",
		".###.",
		"#...#",
		"#...#",
		".###."
	},
	
	{
		".###.",
		"#...#",
		"#...#",
		".####",
		"....#",
		"#...#",
		".###."
	},
	
	{
		".....",
		".....",
		".....",
		".....",
		"..##.",
		"..##.",
		"....."
	},
	
	{
		".....",
		".....",
		".....",
		"#####",
		".....",
		".....",
		"....."
	},
	
	{
		"#####",
		"#....",
		"#....",
		"###..",
		"#....",
		"#....",
		"#####"
	}
};

#if 0

static	const	char	*nums[ 13 ][ 12 ] = {

	{
		"..####..",
		".#....#.",
		"#......#",
		"#......#",
		"#......#",
		"#......#",
		"#......#",
		"#......#",
		"#......#",
		"#......#",
		".#....#.",
		"..####.."
	},
	{
		"....#...",
		"...##...",
		"..#.#...",
		"....#...",
		"....#...",
		"....#...",
		"....#...",
		"....#...",
		"....#...",
		"....#...",
		"....#...",
		"....#..."
	},
	{
		"..####..",
		".#....#.",
		"#......#",
		".......#",
		"......#.",
		".....#..",
		"....#...",
		"...#....",
		"..#.....",
		".#......",
		"#.......",
		"########"
	},
	{
		"..####..",
		".#....#.",
		"#......#",
		".......#",
		"......#.",
		"....##..",
		"....##..",
		"......#.",
		".......#",
		"#......#",
		".#....#.",
		"..####.."
	},
	{
		"......#.",
		".....##.",
		"....#.#.",
		"...#..#.",
		"..#...#.",
		".#....#.",
		"########",
		"......#.",
		"......#.",
		"......#.",
		"......#.",
		"......#."
	},
	{
		"########",
		"#.......",
		"#.......",
		"#.......",
		"#.####..",
		"#.....#.",
		".......#",
		".......#",
		".......#",
		"#......#",
		".#....#.",
		"..####.."
	},
	{
		"..####..",
		".#....#.",
		"#......#",
		"#.......",
		"#.......",
		"#.####..",
		"##....#.",
		"#......#",
		"#......#",
		"#......#",
		".#....#.",
		"..####.."
	},
	{
		"########",
		".......#",
		".......#",
		"......#.",
		".....#..",
		"....#...",
		"...#....",
		"..#.....",
		"..#.....",
		"..#.....",
		"..#.....",
		"..#....."
	},
	{
		"..####..",
		".#....#.",
		"#......#",
		"#......#",
		".#....#.",
		"..####..",
		".#....#.",
		"#......#",
		"#......#",
		"#......#",
		".#....#.",
		"..####.."
	},
	{
		".#####..",
		".#....#.",
		"#......#",
		"#......#",
		"#......#",
		".#....##",
		"..####.#",
		".......#",
		".......#",
		"#......#",
		".#....#.",
		"..####.."
	},
	{
		"........",
		"........",
		"........",
		"........",
		"........",
		"........",
		"........",
		"........",
		"........",
		"...##...",
		"...##...",
		"........"
	},
	{
		"........",
		"........",
		"........",
		"........",
		"........",
		"########",
		"........",
		"........",
		"........",
		"........",
		"........",
		"........"
	},
	{
		"########",
		"#.......",
		"#.......",
		"#.......",
		"#.......",
		"#####...",
		"#.......",
		"#.......",
		"#.......",
		"#.......",
		"#.......",
		"########"
	}
};
#endif

/****************************************************************************/
/*						Module public functions								*/
/****************************************************************************/

/*
*	Creates a graph
*	~~~~~~~~~~~~~~~
*	Receives the job, a dlink of DATA objects and the position of the
*	graph within the raw RGB area.
*	The first row of the graph is the actual data, the rest is the
*	statistical. The first row is painted by red, the stats is by
*	blue. Markers are black.
*
*	The scale factor just compensates for the pixel/micron stuff.
*
*	The raw data is the list of distances, in pixels.
*/
	
void	JobGraph( JOB job, DLINK *data, int x, int y, int xs, int ys, double scal )
{
double	min, max, sum, hiv;
double	tmp0, *tmp1, **tmp2;
double	filt[ 2*FSIZE+2 ], PI;
double	dx, dy, ct;
DATA	*p;
int		yc, xc, xf, xd;
int		row, nrows;
int		**avar, imax;
char	buff[ 20 ];
	
	// Shrink the size so that we can fit the numbers
	
	ys -= 15;
	xs -= 55;
	x  += 55;
		
	// Create the raised cosine filter kernel
	
	PI = atan( 1 ) * 4;
	
	for ( xd = -FSIZE ; xd <= +FSIZE ; xd++ ) {
	
		filt[ xd + FSIZE ] = 1 + cos( ( PI * xd ) / FSIZE );
	}
	
	// Go through all data points and find the min and the max.
	// The data points are actually the count bins, in pixels
	// xc is the 
	// We also count the number of data sets.
	// Negative numbers are added 1 (compensation for dist/overlap)
		
	min = 1E100;
	max = -1E100;
		
	for ( nrows=0, p = DlinkFirst( data ) ; p ; p = DlinkNext( p ), nrows++ ) {
	
		for ( xc = 0 ; xc < p->numd ; xc++ ) {
		
			if ( min > p->data[ xc ] ) min = p->data[ xc ];
			if ( max < p->data[ xc ] ) max = p->data[ xc ];
		}
	}
	
	// Make min and max some sensible values, so that we can create
	// a 10 line division
	
	min *= scal;
	max *= scal;
	dx = JobGraphLimits( &min, &max );
	
	// Allocate the temp data we need. tmp2 will be cleared.
	
	tmp1 = Malloc( xs * sizeof( double ) );
	tmp2 = MallocArray( nrows, xs, sizeof( double ) );
	hiv  = 0;
	
	// Process the raw data and create the distribution curves
	
	for ( row = 0, p = DlinkFirst( data ) ; p ; p = DlinkNext( p ), row++ ) {
		
		// Clear temp array and the sum
		
		for ( xc = 0 ; xc < xs ; xc++ ) tmp1[ xc ] = 0;
		sum = 0;
			
		// Mark the positions
		
		for ( xd = 0 ; xd < p->numd ; xd++ ) {
		
			if ( p->data[ xd ] < 0 ) {
			
				xc = ( ( ( p->data[ xd ] + 1 ) * scal - min ) / 
					   ( max - min ) ) * ( xs - 1 ) + 0.5;
			
			}
			else {
			
				xc = ( ( p->data[ xd ] * scal - min ) / 
					   ( max - min ) ) * ( xs - 1 ) + 0.5;
			}
			
			tmp1[ xc ] += 1.0;
		}
		
		// Do a raised cosine convolution filtering
		
		for ( xc = 0 ; xc < xs ; xc++ ) {
		
			for ( xf = -FSIZE ; xf <= +FSIZE ; xf++ ) {
			
				xd = xc + xf;
				
				if ( xd < 0 || xd >= xs ) continue;
				
				tmp0 = tmp1[ xd ] * filt[ xf + FSIZE ];
				sum += tmp0;
				tmp2[ row ][ xc ] += tmp0;
			}
		}
		
		// Normalise the result and find the max in the entire data set
		// The normalised result should be such that integrating from
		// min to max the result should be 1. That is, the average is
		// sum / xs, so sum/xs * (max-min) = 1.
		
		sum *= ( max - min ) / xs;
		
		for ( xc = 0 ; xc < xs ; xc++ ) {
		
			tmp2[ row ][ xc ] /= sum;
			if ( tmp2[ row ][ xc ] > hiv ) hiv = tmp2[ row ][ xc ];
		}
	}
	
	// We have all our data normalised and we know that the actual
	// values are between hiv and 0. Do the clever thing again.
	
	sum = 0;
	dy  = JobGraphLimits( &sum, &hiv );

	// Create the array for the averaging. It clears the array.
	
	avar = MallocArray( ys, xs, sizeof( int ) );
	imax = 0;
	
	// Process the background curves first
	
	for ( row = 1 ; row < nrows ; row++ ) {
	
		JobDrawCurve( tmp2[ row ], avar, &imax, hiv, xs, ys, TRUE );
	}
	
	// Make it into the image
	
	if ( imax ) JobGraphBlit( job, avar, imax, x, y, xs, ys, 0x0000FF );
	
	// Process the foreground curve
	
	Free( avar );
	avar = MallocArray( ys, xs, sizeof( int ) );
	imax = 0;
	JobDrawCurve( tmp2[ 0 ], avar, &imax, hiv, xs, ys, FALSE );
	JobGraphBlit( job, avar, imax, x, y, xs, ys, 0xFF0000 );

	// Draw the marker lines
	
	Free( avar );
	avar = MallocArray( ys, xs, sizeof( int ) );
	imax = 1;
	
	for ( ct = min ; ct <= max * 1.01 ; ct += dx ) {
	
		if ( ABS( ct ) < 0.01 * dx ) ct = 0;
		
		xc = ( ct - min ) / ( max - min ) * ( xs - 1 );
		
		for ( yc = 0 ; yc < ys ; yc++ ) avar[ yc ][ xc ] = 1;
		
		sprintf( buff, "%g", ct );
		JobGraphNumber( job, buff, FALSE, y + ys + 3, x + xc );
	}
	
	for ( ct = sum ; ct <= hiv * 1.01 ; ct += dy ) {
	
		yc = ( ct - sum ) / ( hiv - sum ) * ( ys - 1 );
		
		for ( xc = 0 ; xc < xs ; xc++ ) avar[ yc ][ xc ] = 1;
		
		sprintf( buff, "%5.2f", ct );
		JobGraphNumber( job, buff, TRUE, y + ys - yc, x - 5  );
	}
	
	JobGraphBlit( job, avar, imax, x, y, xs, ys, 0x000000 );
	Free( avar );
	Free( tmp2 );
	Free( tmp1 );	
}

/*
*	Creates a histogram
*	~~~~~~~~~~~~~~~~~~~
*	Receives the job, a dlink of DATA objects and the position of the
*	graph within the raw RGB area. Only the first row if the is used.
*	Rows are marked green. 
*	Markers are black.
*
*	The scale factor just compensates for the pixel/micron stuff.
*
*	The raw data is the list of distances, in pixels.
*/
	
void	JobHist( JOB job, DLINK *data, int x, int y, int xs, int ys, 
				 double div, double scal )
{
double	min, max, may;
double	tmp0, *tmp1, **tmp2;
double	dx, dy, ct;
DATA	*p;
int		bins, val;
int		*cnts;
int		yc, xc;
int		**avar, imax;
char	buff[ 20 ];
	
	// Shrink the size so that we can fit the numbers
	
	ys -= 15;
	xs -= 55;
	x  += 55;
		
	// Go through all data points and find the max. That will allow us
	// to calculate how many bins (colums) we will need.
		
	max = 0;
	p = DlinkFirst( data );

	for ( xc = 0 ; xc < p->numd ; xc++ ) {
		
		if ( max < p->data[ xc ] ) max = p->data[ xc ];
	}

	// Make min and max some sensible values, so that we can create
	// a 10 line division
	
	min  = 0;
	max *= scal;
	dx	 = JobGraphLimits( &min, &max );
	   
	// Calculate how many bins do we need. It is the max value divided
	// by the division factor, rounded up.
	
//	bins = max / div;
//	if ( max / div > bins ) bins++;
	
	bins = div;
	div  = max / bins;
	
	// Allocate the temp array.
	
	cnts = Malloc( bins * sizeof( int ) );
	
	// Sort the data into bins
	
	for ( xc = 0 ; xc < bins ; xc++ ) cnts[ xc ] = 0;
	
	for ( xc = 0 ; xc < p->numd ; xc++ ) 
		
		cnts[ (int)( p->data[ xc ] * scal / div ) ]++;
	
	// Find the highest bin and calculate the normalisation factor
	
	for ( may = 0, xc = 0 ; xc < bins ; xc++ ) 
		
		if ( may < cnts[ xc ] ) may = cnts[ xc ];

	min = 0;
	dy  = JobGraphLimits( &min, &may );
	
	// Now we allocate an array into which we draw the bins
	
	avar = MallocArray( ys, xs, sizeof( int ) );

	for ( xc = 0 ; xc < xs ; xc++ ) {
		
		val = cnts[ (int)(( bins * xc ) / xs) ] * (ys-1) / may;
		while ( val >= 0 ) avar[ val-- ][ xc ] = 1;
	}
		
	// Blit it into the image and release data
	
	JobGraphBlit( job, avar, 1, x, y, xs, ys, 0x00ff00 );
	Free( avar );
	Free( cnts );
	
	// Draw the marker lines
	
	avar = MallocArray( ys, xs, sizeof( int ) );
	imax = 1;
	
	for ( ct = min ; ct <= max * 1.01 ; ct += dx ) {
	
		if ( ABS( ct ) < 0.01 * dx ) ct = 0;
		
		xc = ( ct - min ) / ( max - min ) * ( xs - 1 );
		
		for ( yc = 0 ; yc < ys ; yc++ ) avar[ yc ][ xc ] = 1;
		
		sprintf( buff, "%g", ct );
		JobGraphNumber( job, buff, FALSE, y + ys + 3, x + xc );
	}
	
	for ( ct = min ; ct <= may * 1.01 ; ct += dy ) {
	
		yc = ( ct - min ) / ( may - min ) * ( ys - 1 );
		
		for ( xc = 0 ; xc < xs ; xc++ ) avar[ yc ][ xc ] = 1;
		
		sprintf( buff, "%5.2f", ct );
		JobGraphNumber( job, buff, TRUE, y + ys - yc, x - 5  );
	}
	
	JobGraphBlit( job, avar, imax, x, y, xs, ys, 0x010101 );
	Free( avar );
}

/****************************************************************************/
/*							Local functions									*/
/****************************************************************************/

/*
*	Draws a point into the accumulating array
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	JobDrawCurve( double *data, int **array, int *max,
							  double hiv, int xs, int ys, int smudge )
{
int		y0, y1, yd;
int		xc, y;

	for ( xc = 0 ; xc < xs - 1 ; xc++ ) {
		
		// Draw the line from one point to the other
			
		y0 = data[ xc + 0 ] / hiv * (ys - 1) + 0.5;
		y1 = data[ xc + 1 ] / hiv * (ys - 1) + 0.5;
		yd = y0 > y1 ? -1 : 1;
			
		if ( y0 != y1 )
			
			for ( y = y0 ; y != y1 ; y += yd ) 
				
				JobDrawDot( array, xc, y, xs, ys, smudge, max );
		else
			JobDrawDot( array, xc, y0, xs, ys, smudge, max );
	}
}

/*
*	Draws a dot into the array, with optional smudging
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	JobDrawDot( int **array, int x, int y, int xs, int ys, 
							int s, int *m )
{
int		dx, dy;
int		xx, yy;

	for ( dx = -SMUDGE ; dx <= SMUDGE ; dx++ ) {
	
		xx = x + dx;
		if ( xx < 0 || xx >= xs ) continue;
		
		for ( dy = -SMUDGE ; dy <= SMUDGE ; dy++ ) {
		
			yy = y + dy;
			if ( yy < 0 || yy >= ys ) continue;
			
			if ( ! s && ( dx || dy ) ) continue;
			
			array[ yy ][ xx ] += 2 * ( 1 + SMUDGE ) - ABS( dy ) - ABS( dx );
			
			if ( *m < array[ yy ][ xx ] ) *m = array[ yy ][ xx ];
		}
	}
}

/*
*	Calculates the limits and the marker delta
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	double	JobGraphLimits( double *lo, double *hi )
{
double	l, h;
double	d, t;
int		m;

	if ( *lo == *hi ) {
	
		if ( ! *lo && ! *hi ) {
		
			*lo = -1;
			*hi = +1;
		}
		else {
		
			*lo *= 0.5;
			*hi *= 1.5;
		}
	}
	
	l = *lo;
	h = *hi;
	d = h - l;
	d = exp( floor( log( d ) / log( 10.0 ) - 1 ) * log( 10.0 ) );
		
	for ( m = 1 ; ; ) {
		
		t = m * d;
		l = t * floor( *lo / t );
		h = t * ceil( *hi / t );

		if ( ( h - l ) / t > 6.5 ) {
		
			if ( m == 5 ) {
			
				m  = 0;
				d *= 10;
			}
			
			if ( ++m == 4 ) m = 5;
		}
		else
			break;
	}
	
	*lo = l;
	*hi = h;
	return( t );	
}

/*
*	Blits the actual image content into the raw RGB
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Only pixels that are white will be changed. The result
*	will have the given colour.
*/

static	void	JobGraphBlit( JOB job, int **array, int max,
							  int x0, int y0, int xs, int ys, int col )
{
int		y, x;
ubyte	*p;
ubyte	gct[ 255 ];

	for ( gct[ 0 ] = 255, x = 1 ; x < 256 ; x++ ) 
	
		gct[ x ] = 255 - (int)(255 * exp( log( x / 255.0 ) * GAMMA ) + 0.5);
	
	col ^= -1;
	
	for ( y = 0 ; y < ys ; y++ ) {
	
		p = job->rawrgb + 3 * ( ( y0 + ys - y - 1 ) * STAT_CANVAS_X + x0 );
		
		for ( x = 0 ; x < xs ; x++, p += 3 ) {
		
			if ( ! array[ y ][ x ] ) continue;
			
			p[ 0 ] = gct[ array[ y ][ x ] * ( ( col >> 16 ) & 255 ) / max ];
			p[ 1 ] = gct[ array[ y ][ x ] * ( ( col >>  8 ) & 255 ) / max ];
			p[ 2 ] = gct[ array[ y ][ x ] * ( ( col >>  0 ) & 255 ) / max ];
		}
	}
}

/*
*	Writes a number for the X axis
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	JobGraphNumber( JOB job, char *num, int ral, int yp, int xp )
{
int		i, l, y, x;
ubyte	*p, *d;

	l = strlen( num );
	
	xp = xp - ( ral ? l * 6 : ( l * 6 ) / 2 );
	yp = ral ? yp - 4 : yp;
	
	for ( y = 0 ; y < 7 ; y++ ) {
	
		for ( i = 0 ; i < l ; i++ ) {
		
			p = job->rawrgb + 3 * ( ( yp + y ) * STAT_CANVAS_X + xp + 6 * i );
			
			switch ( num[ i ] ) {
			
				case ' ':
				
					d = NULL;
					break;
						
				case '0'...'9': 
				
					d = (ubyte *) nums[ num[ i ] - '0' ][ y ];
					break;
					
				case 'E':
				case 'e':
				
					d = (ubyte *) nums[ 12 ][ y ];
					break;
					
				case '.':
					
					d = (ubyte *) nums[ 10 ][ y ];
					break;
					
				default:
					
					d = (ubyte *) nums[ 11 ][ y ];
					break;
			}
				
			for ( x = 0 ; d && x < 5 ; x++, p += 3 ) {
			
				if ( d[ x ] == '#' ) p[ 0 ] = p[ 1 ] = p[ 2 ] = 0;
			}
		}
	}					
}

