/*
*	Creates a histogram
*	~~~~~~~~~~~~~~~~~~~
*	Receives the job, a dlink of DATA objects and the position of the
*	graph within the raw RGB area. Only the first row if the is used.
*	Rows are marked green. 
*	Markers are black.
*
*	The scale factor just compensates for the pixel/micron stuff.
*
*	The raw data is the list of distances, in pixels.
*/
	
void	JobHist( JOB job, DLINK *data, int x, int y, int xs, int ys, 
				 double div, double scal )
{
double	min, max, may;
double	tmp0, *tmp1, **tmp2;
double	dx, dy, ct;
DATA	*p;
int		yc, xc, xf, xd;
int		row, nrows;
int		**avar, imax;
char	buff[ 20 ];
	
	// Shrink the size so that we can fit the numbers
	
	ys -= 15;
	xs -= 55;
	x  += 55;
		
	// Go through all data points and find the max. That will allow us
	// to calculate how many bins (colums) we will need.
		
	max = 0;
	p = DlinkFirst( data );

	for ( xc = 0 ; xv < p->numd ; xc++ ) {
		
		if ( max < p->data[ xc ] ) max = p->data[ xc ];
	}
	
	// Make min and max some sensible values, so that we can create
	// a 10 line division
	
	min  = 0;
	max *= scal;
	dx	 = JobGraphLimits( &min, &max );
	
	// Calculate how many bins do we need. It is the max value divided
	// by the division factor, rounded up.
	
	bins = max / div;
	if ( max / div > bins ) bins++;
	
	// Allocate the temp array.
	
	cnts = Malloc( bins * sizeof( int ) );
	
	// Sort the data into bins
	
	for ( xc = 0 ; xc < binx ; xc++ ) cnts[ xc ] = 0;
	
	for ( xc = 0 ; xc < p->numd ; xc++ ) 
		
		cnts[ (int)( p->data[ xc ] * scal / div ) ]++;
	
	// Find the highest bin and calculate the normalisation factor
	
	for ( may = 0, xc = 0 ; xc < binx ; xc++ ) 
		
		if ( may < cnts[ xc ] ) may = cnts[ xc ];
	
	min = 0;
	dy  = JobGraphLimits( &min, &may );
	
	// Now we allocate an array into which we draw the bins
	
	avar = MallocArray( ys, xs, sizeof( int ) );
	
	for ( xc = 0 ; xc < xs ; xc++ ) {
		
		val = cnts[ ( bins * xc ) / xs ] * ys / may;
		while ( val >= 0 ) avar[ val ][ xs ] = 1;
	}
		
	// Blit it into the image and release data
	
	JobGraphBlit( job, avar, 1, x, y, xs, ys, 0x00FF00 );
	Free( avar );
	Free( cnts );
	
	// Draw the marker lines
	
	avar = MallocArray( ys, xs, sizeof( int ) );
	imax = 1;
	
	for ( ct = min ; ct <= max * 1.01 ; ct += dx ) {
	
		if ( ABS( ct ) < 0.01 * dx ) ct = 0;
		
		xc = ( ct - min ) / ( max - min ) * ( xs - 1 );
		
		for ( yc = 0 ; yc < ys ; yc++ ) avar[ yc ][ xc ] = 1;
		
		sprintf( buff, "%g", ct );
		JobGraphNumber( job, buff, FALSE, y + ys + 3, x + xc );
	}
/*	
	for ( ct = sum ; ct <= hiv * 1.01 ; ct += dy ) {
	
		yc = ( ct - sum ) / ( hiv - sum ) * ( ys - 1 );
		
		for ( xc = 0 ; xc < xs ; xc++ ) avar[ yc ][ xc ] = 1;
		
		sprintf( buff, "%5.2f", ct );
		JobGraphNumber( job, buff, TRUE, y + ys - yc, x - 5  );
	}
*/	
	JobGraphBlit( job, avar, imax, x, y, xs, ys, 0x000000 );
	Free( avar );
}

