void	TrackDump( char * );
/****************************************************************************/
/* Copyright 2004, � Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	GUI image window functionality											*/
/*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~											*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zolt�n K�csi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <glib.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>

#include <types.h>
#include <image.h>
#include <memory.h>
#include <options.h>
#include <objects.h>
#include <gtkwrappers.h>
#include <job.h>
#include <progress.h>
#include <randomlist.h>

#include "jobdefs.h"
#include "canvas.h"
#include "buttons.h"
#include "process.h"
#include "jobgui.h"

/****************************************************************************/
/*							Local definitions								*/
/****************************************************************************/

/****************************************************************************/
/*							Prototypes										*/
/****************************************************************************/

static	void	JobGuiCreate( JOB job );
static	int		JobGuiDestroy( GtkWidget *win, GdkEvent *ev, JOB job );
static	void	JobClose( JOB job );

/****************************************************************************/
/*							Public functions								*/
/****************************************************************************/

/*
*	Creates a new job instance with the given filename
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	It receives a filename to load and a function that will be
*	called if the user wants to close the window.
*
*	Returns whether it managed to open and read in the file.
*/

JOB		JobCreate( const char *file, int (*event)(JOB,int,int) )
{
JOB		job;
IMAGE	*orig;
char	*p;

	if ( ! ( orig = ImageLoad( file ) ) ) return( FALSE );
	
	// Create a job descriptor
	
	job = Malloc( sizeof( struct s_job ) );
	
	// Initialise basic things
	
	job->ys		= orig->y;
	job->xs		= orig->x;
	job->orig	= orig;
	job->filt	= NULL;
	job->gamm	= NULL;
	job->fgnd	= NULL;
	job->bgnd	= &job->orig;
	job->mask	= 0;
	job->rawrgb	= NULL;
	job->line	= NULL;
	job->stat	= NULL;
	job->tris	= NULL;
	job->magnif = 1;
	job->actlen = 0;
	job->reflen = 0;
	job->conlen = 0;
	job->signal = event;
	job->window = NULL;
	job->mark	= MallocArray( job->ys, job->xs, sizeof( uword ) );

	job->opt_hash = FALSE;
	job->opt_objs = FALSE;
	job->opt_dist = FALSE;
	job->opt_ovlp = FALSE;
	job->opt_conf = FALSE;
	job->opt_tria = FALSE;

	DlinkInit( &job->ref );
	DlinkInit( &job->act );
	DlinkInit( &job->con );
	DlinkInit( &job->dist );
	DlinkInit( &job->ovlp );
	DlinkInit( &job->ltri );
	DlinkInit( &job->dsta );
	DlinkInit( &job->tsta );
				
	// Store a copy of the filename
	
	job->filename = Malloc( strlen( file ) + 1 );
	strcpy( job->filename, file );
	
	for ( job->tname = p = job->filename ; *p ; *p++ ) {
	
		if ( *p == '/' ) job->tname = p + 1;
	}
	
	// Load current options
	
	OptionValueGet( optPreMicrosc,	&job->opt.csep );
	OptionValueGet( optPreColAct,	&job->opt.acol );
	OptionValueGet( optPreColRef,	&job->opt.rcol );
	OptionValueGet( optPreFilter,	&job->opt.frad );
	OptionValueGet( optPreGamma,	&job->opt.gamm );
	OptionValueGet( optPreEqual,	&job->opt.eqal );
	
	OptionValueGet( optExtActAlg,	&job->opt.act_meth );
	OptionValueGet( optExtActSeed,	&job->opt.act_seed );
	OptionValueGet( optExtActAcpt,	&job->opt.act_acpt );
	OptionValueGet( optExtActSize,	&job->opt.act_mins );
	OptionValueGet( optExtActFill,	&job->opt.act_fill );
	OptionValueGet( optExtActExcl,	&job->opt.act_excl );
	OptionValueGet( optExtRefAlg,	&job->opt.ref_meth );
	OptionValueGet( optExtRefSeed,	&job->opt.ref_seed );
	OptionValueGet( optExtRefAcpt,	&job->opt.ref_acpt );
	OptionValueGet( optExtRefSize,	&job->opt.ref_mins );
	OptionValueGet( optExtRefFill,	&job->opt.ref_fill );
	OptionValueGet( optExtRefExcl,	&job->opt.ref_excl );

	OptionValueGet( optPairMicron, &job->opt.pair_mic );
	OptionValueGet( optPairPixels, &job->opt.pair_pix );
	OptionValueGet( optPairDistMax, &job->opt.pair_dis );
	OptionValueGet( optPairRefMul,  &job->opt.pair_ref );
	OptionValueGet( optPairActMul,  &job->opt.pair_act );
	
	OptionValueGet( optStatSnum,	&job->opt.stat_num );
	OptionValueGet( optStatRund,	&job->opt.stat_act );
	OptionValueGet( optStatRunc,	&job->opt.stat_ref );
	OptionValueGet( optStatSelf,	&job->opt.dis_meth );
	OptionValueGet( optStatHist,	&job->opt.stat_hic );

	job->new = job->opt;
	
	// Create the GUI for the job
	
	JobGuiCreate( job );
	
	// Give the job back to the administrative process
	
	return( job );
}

/*
*	Receives commands for the job
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	JobControl( JOB job, int command, int param )
{
	switch ( command ) {
	
		case JOB_CMD_ACTIVE_ENABLE:
		
			// Changes the sensitive/insensitive state of the active
			// button
			
			JobButtonActiveEnable( job, param );
			break;
			
		case JOB_CMD_ACTIVE:
		
			// Changes the state of the active button
			
			JobButtonActiveState( job, param );
			(*job->signal)( job, JOB_STAT_ACTIVE, param );
			break;
			
		case JOB_CMD_PROCESS:
		
			(*job->signal)( job, JOB_STAT_STARTED, 0 );
			
			ProgressText( job->tname );
			
			gtk_widget_set_sensitive( job->butt_save, FALSE );
			gtk_widget_set_sensitive( job->butt_clse, FALSE );
			gtk_widget_set_sensitive( job->butt_mode, FALSE );
			gtk_widget_set_sensitive( job->butt_data, FALSE );
						
			if ( job->mode == MODE_PROC || job->mode == MODE_STAT ) {
			
				if ( param & 16 )
					JobStats( job );
				else if ( param &  8 )
					JobDistance( job );
				else if ( param &  4 ) 
					JobExtract( job );
				else if ( param &  2 ) 
					JobEqualise( job );
				else if ( param &  1 ) 
					JobSeparate( job );
			
				if ( job->mode != MODE_STAT ) {
				
					JobCanvasObjectMap( job );
					JobCanvasForeground( job );
					JobCanvasRefresh( job );
				}
				else {
				
					JobStatRefresh( job );
				}
				
			}
			
			gtk_widget_set_sensitive( job->butt_save, TRUE );
			gtk_widget_set_sensitive( job->butt_clse, TRUE );
			gtk_widget_set_sensitive( job->butt_mode, TRUE );
			gtk_widget_set_sensitive( job->butt_data, TRUE );
			ProgressText( "" );
			(*job->signal)( job, JOB_STAT_FINISHED, 0 );
			break;
			
		case JOB_CMD_ABORT:
		
			ProgressAbort();
			break;
		
		case JOB_CMD_CLOSE:
		
			// Closes the job
			
			JobClose( job );
			break;
			
		case JOB_CMD_OPTION_LOAD:
		
			// Instruct the job to get all option parameters from the
			// global option source
			
			OptionValueGet( optPreMicrosc,	&job->new.csep );
			OptionValueGet( optPreColAct,	&job->new.acol );
			OptionValueGet( optPreColRef,	&job->new.rcol );
			OptionValueGet( optPreFilter,	&job->new.frad );
			OptionValueGet( optPreGamma,	&job->new.gamm );
			OptionValueGet( optPreEqual,	&job->new.eqal );
	
			OptionValueGet( optExtActAlg,	&job->new.act_meth );
			OptionValueGet( optExtActSeed,	&job->new.act_seed );
			OptionValueGet( optExtActAcpt,	&job->new.act_acpt );
			OptionValueGet( optExtActSize,	&job->new.act_mins );
			OptionValueGet( optExtActFill,	&job->new.act_fill );
			OptionValueGet( optExtActExcl,	&job->new.act_excl );
			OptionValueGet( optExtRefAlg,	&job->new.ref_meth );
			OptionValueGet( optExtRefSeed,	&job->new.ref_seed );
			OptionValueGet( optExtRefAcpt,	&job->new.ref_acpt );
			OptionValueGet( optExtRefSize,	&job->new.ref_mins );
			OptionValueGet( optExtRefFill,	&job->new.ref_fill );
			OptionValueGet( optExtRefExcl,	&job->new.ref_excl );

			OptionValueGet( optPairMicron,  &job->new.pair_mic );
			OptionValueGet( optPairPixels,  &job->new.pair_pix );
			OptionValueGet( optPairDistMax, &job->new.pair_dis );
			OptionValueGet( optPairRefMul,  &job->new.pair_ref );
			OptionValueGet( optPairActMul,  &job->new.pair_act );

			OptionValueGet( optStatSnum,	&job->new.stat_num );
			OptionValueGet( optStatRund,	&job->new.stat_act );
			OptionValueGet( optStatRunc,	&job->new.stat_ref );
			OptionValueGet( optStatSelf,	&job->new.dis_meth );
			OptionValueGet( optStatHist,	&job->new.stat_hic );
			break;
			
		case JOB_CMD_OPTION_SAVE:
		
			// Instruct the job to save its current options in the
			// global option source
			
			OptionValueSet( optPreMicrosc,	&job->opt.csep );
			OptionValueSet( optPreColAct,	&job->opt.acol );
			OptionValueSet( optPreColRef,	&job->opt.rcol );
			OptionValueSet( optPreFilter,	&job->opt.frad );
			OptionValueSet( optPreGamma,	&job->opt.gamm );
			OptionValueSet( optPreEqual,	&job->opt.eqal );
			
			OptionValueSet( optExtActAlg,	&job->opt.act_meth );
			OptionValueSet( optExtActSeed,	&job->opt.act_seed );
			OptionValueSet( optExtActAcpt,	&job->opt.act_acpt );
			OptionValueSet( optExtActSize,	&job->opt.act_mins );
			OptionValueSet( optExtActFill,	&job->opt.act_fill );
			OptionValueSet( optExtActExcl,	&job->opt.act_excl );
			OptionValueSet( optExtRefAlg,	&job->opt.ref_meth );
			OptionValueSet( optExtRefSeed,	&job->opt.ref_seed );
			OptionValueSet( optExtRefAcpt,	&job->opt.ref_acpt );
			OptionValueSet( optExtRefSize,	&job->opt.ref_mins );
			OptionValueSet( optExtRefFill,	&job->opt.ref_fill );
			OptionValueSet( optExtRefExcl,	&job->opt.ref_excl );
		
			OptionValueSet( optPairMicron,	&job->opt.pair_mic );
			OptionValueSet( optPairPixels,	&job->opt.pair_pix );
			OptionValueSet( optPairDistMax, &job->opt.pair_dis );
			OptionValueSet( optPairRefMul,  &job->opt.pair_ref );
			OptionValueSet( optPairActMul,  &job->opt.pair_act );
			
			OptionValueSet( optStatSnum,	&job->opt.stat_num );
			OptionValueSet( optStatRund,	&job->opt.stat_act );
			OptionValueSet( optStatRunc,	&job->opt.stat_ref );
			OptionValueSet( optStatSelf,	&job->opt.dis_meth );
			OptionValueSet( optStatHist,	&job->opt.stat_hic );

			break;
					
		case JOB_CMD_COLOURS:
		
			if ( job->mode == MODE_PROC ) {
			
				JobCanvasObjectMap( job );
				JobCanvasForeground( job );
				JobCanvasRefresh( job );
			}
			
			break;
	}
}

/****************************************************************************/
/*							Local functions									*/
/****************************************************************************/

/*
*	Creates the GUI for the job
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	It uses native GTK objects, not our own due to the
*	highly graphical nature of the activities
*/

static	void	JobGuiCreate( JOB job )
{
GtkWidget	*win;
GtkWidget	*box;
int			xds, yds;
int			min, max;

	// Create top level window
	
	win = gtk_window_new( GTK_WINDOW_TOPLEVEL );
	gtk_window_set_title( GTK_WINDOW( win ), job->filename );
	gtk_window_set_policy( GTK_WINDOW( win ), TRUE, TRUE, FALSE );
	gtk_window_set_position( GTK_WINDOW( win ), GTK_WIN_POS_MOUSE );
	gtk_signal_connect( GTK_OBJECT( win ), "delete-event", 
						GTK_SIGNAL_FUNC( JobGuiDestroy ), job );
	
	// Calculate the initial window size
	
	xds = job->xs + SCROLLBAR_SIZE + BUTTONS_HSIZE;
	min = VIEWPORT_MIN_X + SCROLLBAR_SIZE + BUTTONS_HSIZE;
	max = ( gdk_screen_width() * 2 ) / 3;
	
	if ( job->xs < min ) xds = min;
	if ( job->xs > max ) xds = max;
	
	yds = job->ys + SCROLLBAR_SIZE;
	min = BUTTONS_VSIZE;
	max = ( gdk_screen_height() * 2 ) / 3;
	
	if ( job->ys < min ) yds = min;
	if ( job->ys > max ) yds = max;
	
	job->vpy	= yds;
	job->vpx	= xds;
	
	// Create a packing box, put the box into the into the window
	
	box = GuiHbox( FALSE, 0 );
	
	// Start filling the box with various GUI elements	
	// Create the canvas and the scrollbars, put into the box.
	
	GuiBoxAdd( JobCanvasCreate( job ),		box, TRUE, TRUE );
	GuiBoxAdd( JobInterfaceCreate( job ),	box, FALSE, FALSE );
	
	gtk_container_add( GTK_CONTAINER( win ), box );
	gtk_widget_set_usize( win, job->vpx, job->vpy );
	gtk_widget_show( win );
	job->window = win;
}

/*
*	This function is called when the job's window gets a close request
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	int		JobGuiDestroy( GtkWidget *win, GdkEvent *ev, JOB job )
{
	if ( job->signal ) 
	
		return( ! (*job->signal)( job, JOB_REQ_CLOSE, 0 ) );
	else
		return( FALSE );
}

/*
*	Closes a job, releasing everything associated with it
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	JobClose( JOB job )
{
	// Delete the signal function
	
	job->signal = NULL;
	
	// Delete everything used by processing
	
	JobRelease( job );
	
	// Free all the image memory we allocated
	
	if ( job->orig ) ImageDestroy( job->orig );
	if ( job->mark ) Free( job->mark );
		
	// Release the canvas
	
	JobCanvasDestroy( job );
	
	// Destroy the GUI, if we have any
	
	if ( job->window ) gtk_widget_destroy( job->window );
	
	// Free the filename
	
	if ( job->filename ) Free( job->filename );
	
	// Release ourselves
	
	Free( job );
}
