/****************************************************************************/
/* Copyright 2004, � Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	Individual image processing												*/
/*	~~~~~~~~~~~~~~~~~~~~~~~~~~~												*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zolt�n K�csi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#ifndef __JOBDEFS_H__
#define __JOBDEFS_H__

#include <glib.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>

#include <types.h>
#include <image.h>
#include <job.h>
#include <dlinklist.h>
#include <stats.h>

#define	VIEWPORT_MIN_X		300
#define	VIEWPORT_MIN_Y		200

#define	MARK_ACT_INT		1
#define	MARK_REF_INT		2
#define	MARK_CON_INT		4
#define	MARK_ACT_OUT		8
#define	MARK_REF_OUT		16
#define	MARK_CON_OUT		32
#define	MARK_DISTANCE		64
#define	MARK_OVERLAP		128
#define	MARK_TRIANGLE		256

#define	MODE_NONE			0
#define	MODE_PROC			1
#define	MODE_CONT			2
#define	MODE_STAT			3
#define	MODE_CSEL			4

typedef	struct {

	// Preprocessing options
	
	int		csep;						/* Colour separation method			*/
	int		rcol;						/* Reference colour					*/
	int		acol;						/* Active colour					*/
	double	frad;						/* Filter radius					*/
	int		eqal;						/* Equalisation is needed			*/
	double	gamm;						/* Gamma correction value			*/
	
	// Extraction options
	
	int		act_meth;					/* Active object search method		*/
	int		act_seed;					/* Active object search seed		*/
	int		act_acpt;					/* Active object min. accepted		*/
	int		act_mins;					/* Active object min. size			*/
	int		act_fill;					/* Active object fill holes			*/
	int		act_excl;					/* Active object exclude extreme	*/
	
	int		ref_meth;					/* Reference object search method	*/
	int		ref_seed;					/* Reference object search seed		*/
	int		ref_acpt;					/* Reference object min. accepted	*/
	int		ref_mins;					/* Reference object min. size		*/
	int		ref_fill;					/* Reference object fill holes		*/
	int		ref_excl;					/* Reference object exclude extreme	*/
	
	// Object pairing options
	
	int		pair_pix;					/* Ratio pixel value				*/
	int		pair_mic;					/* Ratio micron value				*/
	int		pair_dis;					/* Max distance between objects		*/
	int		pair_act;					/* Max pairs for active objects		*/
	int		pair_ref;					/* Max pairs for reference objects	*/
	
	// Statistical options
	
	int		stat_num;					/* Number of tests to run			*/
	int		stat_act;					/* Do active object distribution	*/
	int		stat_ref;					/* Do active object to reference	*/
	int		dis_meth;					/* Distance method					*/
	int		stat_hic;					/* Histogram columns				*/
	
} JOBOPT;	

struct s_job {
	
	// Administrative things
	
	DLINK	link;						/* Links jobs. MUST BE FIRST FIELD!	*/
	int		(*signal)(JOB,int,int);		/* Callback on events				*/
	
	// Filesystem related things
	
	char	*filename;					/* Name of the image file			*/
	char	*tname;						/* Temporary name while saving		*/
	
	// Options
	
	JOBOPT	opt;						/* Current options					*/
	JOBOPT	new;						/* New options						*/
	
	// Images and image display data
	
	int		ys, xs;						/* Image size						*/
	int		cys, cxs;					/* Canvas raw size (RGB buffer)		*/
	int		vpy, vpx;					/* Viewport size					*/
	IMAGE	*orig;						/* The original image				*/
	IMAGE	*filt;						/* Filtered image					*/
	IMAGE	*gamm;						/* Equalised and gamma corrected im	*/
	
	// Representation of the canvas
	
	int		magnif;						/* Magnification factor				*/
	ubyte	*rawrgb;					/* Image display memory				*/
	ubyte	*line;						/* Magnification display line		*/
	uword	**mark;						/* Mask used for foreground			*/
	IMAGE	**bgnd;						/* Background image pointer			*/
	IMAGE	*fgnd;						/* Foreground image					*/
	uword	mask;						/* Foreground mask					*/
	
	// Admin things
	
	int		mode;						/* Current operation mode			*/
	
	// Processing related data
	
	DLINK	ref;						/* Reference features				*/
	DLINK	act;						/* Active features					*/
	DLINK	con;						/* Confinement area					*/
	int		actlen;						/* Length of the active queue		*/
	int		reflen;						/* Length of the reference queue	*/
	int		conlen;						/* Length of confinement queue		*/
	DLINK	dist;						/* Object pair distances			*/
	DLINK	ovlp;						/* Object overlaps					*/
	STAT	*stat;						/* Statistical data					*/
	STAT	*tris;						/* Statistical data					*/
	int		ntri;						/* Number of active triangles		*/
	DLINK	ltri;						/* List of active triangle segments	*/
	DLINK	dsta;						/* Distance statistics data			*/
	DLINK	tsta;						/* Segment statistics data			*/
				
	// Display options
	
	int		opt_hash;					/* Objects shaded when shown		*/
	int		opt_objs;					/* Show objects over the image		*/
	int		opt_dist;					/* Show distances					*/
	int		opt_ovlp;					/* Show overlaps					*/
	int		opt_conf;					/* Show confinement area			*/
	int		opt_tria;					/* Show triangles					*/
	
	// Containment related things
	
	int		cont_size;					/* Size of the containment cursor	*/
	int		cont_oldy;					/* Old Y mouse coordinate			*/
	int		cont_oldx;					/* Old X mouse coordinate			*/
	int		mouse_cid;					/* Mouse event connection ID		*/
	
	// Colour selection things
	
	int		csel[ 3 ];					/* Current colour					*/
	int		ccnt;						/* Averaging count					*/
	
	// User selection, shows what the user would like to
	// select, if it was available to her
	
	int			pref_supp;				/* Supresses updating the prefs		*/
	GtkWidget	**pref_magn;			/* Magnification					*/
	GtkWidget	**pref_bgnd;			/* Background image					*/
	GtkWidget	**pref_fgnd;			/* Foreground image					*/
	
	// Major GTK widgets
	
	GtkWidget	*window;				/* Top level window					*/
	GtkWidget	*canvas;				/* Image canvas						*/
	
	// Various buttons
	
	GtkWidget	*butt_blnk;				/* Blank background selected		*/
	GtkWidget	*butt_orig;				/* Original image selected			*/
	GtkWidget	*butt_filt;				/* Filtered image selected			*/
	GtkWidget	*butt_gamm;				/* Level corrected image selected	*/
	GtkWidget	*butt_stat;				/* Statistics selected				*/
	
	GtkWidget	*butt_hash;				/* Objects shaded when shown		*/
	GtkWidget	*butt_objs;				/* Show objects over the image		*/
	GtkWidget	*butt_dist;				/* Show distances					*/
	GtkWidget	*butt_ovlp;				/* Show overlaps					*/
	GtkWidget	*butt_conf;				/* Show confinement area			*/
	GtkWidget	*butt_tria;				/* Show triangles					*/
	
	GtkWidget	*butt_clse;				/* Close button						*/
	GtkWidget	*butt_save;				/* Save button						*/
	GtkWidget	*butt_actv;				/* Active job toggle button			*/
	GtkWidget	*butt_data;				/* Save data button					*/
	
	GtkWidget	*butt_mag1;				/* Button for 1x zoom				*/
	GtkWidget	*butt_mag2;				/* Button for 2x zoom				*/
	GtkWidget	*butt_mag4;				/* Button for 4x zoom				*/
	
	GtkWidget	*butt_cur1;				/* Button for small cursor			*/
	GtkWidget	*butt_cur2;				/* Button for medium cursor			*/
	GtkWidget	*butt_cur4;				/* Button for large cursor			*/
	
	GtkWidget	*csel_cbox;				/* Colour box						*/
	
	GtkWidget	*butt_mode;				/* Mode select menu button			*/
	GtkWidget	*mode_list;				/* Object for mode select			*/
	GtkWidget	*mode_menu;				/* Object for mode select			*/
	GSList		*mode_grup;				/* Object for mode select			*/
	GtkWidget	*mode_cont;				/* Mode select, containment mode	*/
	GtkWidget	*mode_proc;				/* Mode select, processing mode		*/
	GtkWidget	*mode_stat;				/* Mode select, statistics mode		*/
	GtkWidget	*mode_csel;				/* Mode select, colour pick mode	*/
	GtkWidget	*pbox_cont;				/* Containment mode option box		*/
	GtkWidget	*pbox_proc;				/* Processing mode option box		*/
	GtkWidget	*pbox_stat;				/* Statistics mode option box		*/
	GtkWidget	*pbox_csel;				/* Statistics mode option box		*/
};

#endif

