/****************************************************************************/
/* Copyright 2004, © Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	Individual image processing internal GUI functions						*/
/*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~						*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zoltán Kócsi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#ifndef __JOBGUI_H__
#define __JOBGUI_H__

#include <glib.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>

#include <types.h>
#include <image.h>
#include <job.h>
#include <dlinklist.h>

/****************************************************************************/
/*							Useful macros									*/
/****************************************************************************/

#define	SCROLLBAR_SIZE	22
#define	BUTTONS_VSIZE	490
#define	BUTTONS_HSIZE	120

#define	STAT_CANVAS_X		600
#define	STAT_CANVAS_Y		1300

typedef	struct {

	DLINK	link;
	int		numd;
	double	data[];
	
} DATA;

/*
*	A macro to test the status of an option button
*/

#define	IS_ACTIVE( but ) \
	gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(job->butt_ ## but ) )
	
#define	SET_ACTIVE( but, sta ) \
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(job->butt_ ## but ), sta )

/****************************************************************************/
/*						Function prototypes									*/
/****************************************************************************/

/*
*	Creates the box that contains the processing operation items
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*JobProcPanel( JOB job, GtkWidget *box, int wide, int fill );

/*
*	Called when the processing mode is entered
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	JobProcEnter( JOB job );

/*
*	Called when the processing mode is left
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	JobProcLeave( JOB job );

/*
*	Creates the box that contains the containment operation items
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*JobContPanel( JOB job, GtkWidget *box, int wide, int fill );

/*
*	This function is called when the containment mode is entered
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	JobContEnter( JOB job );

/*
*	This function is called when the containment mode is left
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	JobContLeave( JOB job );

/*
*	Creates the box that contains the statistics operation items
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*JobStatPanel( JOB job, GtkWidget *box, int wide, int fill );

/*
*	This function is called when the statistics mode is entered
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	JobStatEnter( JOB job );

/*
*	This function is called when the statistics mode is left
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	JobStatLeave( JOB job );

/*
*	Refreshes the statistical canvas
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	JobStatRefresh( JOB job );

/*
*	Creates the box that contains the colour select operation items
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*JobCselPanel( JOB job, GtkWidget *box, int wide, int fill );

/*
*	This function is called when the colour select mode is entered
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	JobCselEnter( JOB job );

/*
*	This function is called when the colour select mode is left
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	JobCselLeave( JOB job );

/*
*	Creates a graph
*	~~~~~~~~~~~~~~~
*	Receives the job, a dlink of DATA objects and the position of the
*	graph within the raw RGB area.
*	The first row of the graph is the actual data, the rest is the
*	statistical. The first row is painted by red, the stats is by
*	blue. Markers are black.
*/
	
void	JobGraph( JOB job, DLINK *data, int x, int y, int xs, int ys, double scal );


/*
*	Creates a histogram
*	~~~~~~~~~~~~~~~~~~~
*	Receives the job, a dlink of DATA objects and the position of the
*	graph within the raw RGB area. Only the first row if the is used.
*	Rows are marked green. 
*	Markers are black.
*
*	The scale factor just compensates for the pixel/micron stuff.
*
*	The raw data is the list of distances, in pixels.
*/
	
void	JobHist( JOB job, DLINK *data, int x, int y, int xs, int ys, 
				 double div, double scal );

#endif
