/****************************************************************************/
/* Copyright 2004,  Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	GUI image processing functionality										*/
/*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~										*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zoltn Kcsi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <glib.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>

#include <types.h>
#include <job.h>
#include <styles.h>
#include <objects.h>
#include <distance.h>
#include <gtkwrappers.h>
#include "jobdefs.h"
#include "canvas.h"
#include "buttons.h"
#include "process.h"
#include "jobgui.h"

/****************************************************************************/
/*							Prototypes										*/
/****************************************************************************/

static	GtkWidget	*JobDispselCreate( JOB job );
static	GtkWidget	*JobDispoptCreate( JOB job );
static	GtkWidget	*JobActionsCreate( JOB job );
static	GtkWidget	*JobMagnifyCreate( JOB job );

static	void	JobDispoptSig( GtkWidget *butt, JOB job );
static	void	JobMagnifySig( GtkWidget *butt, JOB job );
static	void	JobDispselSig( GtkWidget *butt, JOB job );

/****************************************************************************/
/*						Module public functions								*/
/****************************************************************************/

/*
*	Creates the box that contains the processing operation items
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*JobProcPanel( JOB job, GtkWidget *box, int wide, int fill )
{
GtkWidget	*panel;

	// Create a packing box for the panels
	
	panel = GuiVbox( FALSE, 5 );
	GuiBoxAdd( JobDispselCreate( job ), panel, FALSE, FALSE );
	GuiBoxAdd( JobDispoptCreate( job ), panel, FALSE, FALSE );
	GuiBoxAdd( JobMagnifyCreate( job ), panel, FALSE, FALSE );
	GuiBoxAdd( panel, box, wide, fill );
	gtk_widget_hide( panel );
	return( panel );
}

/*
*	Called when the processing mode is entered
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	JobProcEnter( JOB job )
{
	if ( job->mode == MODE_PROC ) return;
	gtk_widget_show( job->pbox_proc );
	SET_ACTIVE( orig, TRUE );	
	SET_ACTIVE( mag1, TRUE );
	JobCanvasObjectMap( job );
	JobCanvasForeground( job );
	JobCanvasRefresh( job );

	job->mode = MODE_PROC;
}

/*
*	Called when the processing mode is left
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	JobProcLeave( JOB job )
{
	if ( job->mode != MODE_PROC ) return;
	SET_ACTIVE( orig, TRUE );	
	SET_ACTIVE( mag1, TRUE );
	gtk_widget_hide( job->pbox_proc );
	job->mode = MODE_NONE;
}

/****************************************************************************/
/*							Local functions									*/
/****************************************************************************/

/*
*	Creates the selection of items to display
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Returns a vertical packing box with everything in it
*/

static	GtkWidget	*JobDispselCreate( JOB job )
{
GtkWidget	*fra;
GtkWidget	*box;
GSList		*grp;

	grp = NULL;
	box = GuiVbox( FALSE, 0 );
	GuiLabel( "Background", med_style, box, FALSE, FALSE );
	
	job->butt_blnk = GuiRadio( "Blank", &grp, JobDispselSig, job, 
							   box, FALSE, FALSE );
	
	job->butt_orig = GuiRadio( "Original", &grp, JobDispselSig, job, 
							   box, FALSE, FALSE );
	
	job->butt_filt = GuiRadio( "Filtered", &grp, JobDispselSig, job, 
							   box, FALSE, FALSE );
	
	job->butt_gamm = GuiRadio( "Equalised", &grp, JobDispselSig, job, 
							   box, FALSE, FALSE );
	
	fra = GuiFrame( NULL, NULL, box, GTK_SHADOW_OUT );
	
	SET_ACTIVE( orig, TRUE );
	return( fra );
}

/*
*	Creates the magnification box
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Returns a vertical packing box with everything in it
*/

static	GtkWidget	*JobMagnifyCreate( JOB job )
{
GtkWidget	*fra;
GtkWidget	*box;
GtkWidget	*rad;
GSList		*grp;

	grp = NULL;
	box = GuiVbox( FALSE, 0 );
	GuiLabel( "Zoom", med_style, box, FALSE, FALSE );
	
	job->butt_mag1 = GuiRadio( "x1", &grp, JobMagnifySig, job, 
							   box, FALSE, FALSE );

	job->butt_mag2 = GuiRadio( "x2", &grp, JobMagnifySig, job, 
							   box, FALSE, FALSE );

	job->butt_mag4 = GuiRadio( "x4", &grp, JobMagnifySig, job, 
							   box, FALSE, FALSE );

	fra = GuiFrame( NULL, NULL, box, GTK_SHADOW_OUT );
	
	return( fra );
}

/*
*	Creates the display options buttons
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Returns a vertical packing box with everything in it
*/

static	GtkWidget	*JobDispoptCreate( JOB job )
{
GtkWidget	*box;
GtkWidget	*fra;

	box = GuiVbox( FALSE, 5 );
	GuiLabel( "Foreground", med_style, box, FALSE, FALSE );
	
	job->butt_objs = GuiToggle( "Objects", JobDispoptSig, job, 
								box, FALSE, FALSE );
	
	job->butt_dist = GuiToggle( "Distances", JobDispoptSig, job, 
								box, FALSE, FALSE );
		
	job->butt_ovlp = GuiToggle( "Overlaps", JobDispoptSig, job, 
								box, FALSE, FALSE );
		
	job->butt_conf = GuiToggle( "Confinement", JobDispoptSig, job, 
								box, FALSE, FALSE );
		
	job->butt_tria = GuiToggle( "Segments", JobDispoptSig, job, 
								box, FALSE, FALSE );
	
	job->butt_hash = GuiToggle( "Hatch objects", JobDispoptSig, job, 
								box, FALSE, FALSE );
				
	fra = GuiFrame( NULL, NULL, box, GTK_SHADOW_OUT );
	return( fra );
}

/*
*	Signal handler for background selection buttons
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	JobDispselSig( GtkWidget *butt, JOB job )
{
	if ( ! gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( butt ) ) ) return;
	
	if ( butt == job->butt_blnk ) {
	
		if ( ! job->pref_supp ) job->pref_bgnd = &job->butt_blnk;
		job->bgnd = NULL;
	}
	
	if ( butt == job->butt_orig ) {
	
		if ( ! job->pref_supp ) job->pref_bgnd = &job->butt_orig;
		job->bgnd = &job->orig;
	}
	
	if ( butt == job->butt_filt ) {
	
		if ( ! job->pref_supp ) job->pref_bgnd = &job->butt_filt;
		job->bgnd = &job->filt;
	}
	
	if ( butt == job->butt_gamm ) {
	
		if ( ! job->pref_supp ) job->pref_bgnd = &job->butt_gamm;
		job->bgnd = &job->gamm;
	}
	
	JobCanvasRefresh( job );	
}

/*
*	Signal handler for the zoom buttons
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	JobMagnifySig( GtkWidget *butt, JOB job )
{
int		zoom;

	if ( ! gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( butt ) ) ) return;
	
	zoom = job->magnif;
	
	if ( butt == job->butt_mag1 ) zoom = 1;
	if ( butt == job->butt_mag2 ) zoom = 2;
	if ( butt == job->butt_mag4 ) zoom = 4;
	
	JobCanvasMagnify( job, zoom );
}

/*
*	Signal handler for display option buttons
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	JobDispoptSig( GtkWidget *butt, JOB job )
{
	if ( butt == job->butt_hash ) job->opt_hash = IS_ACTIVE( hash );
	if ( butt == job->butt_objs ) job->opt_objs = IS_ACTIVE( objs );
	if ( butt == job->butt_dist ) job->opt_dist = IS_ACTIVE( dist );
	if ( butt == job->butt_ovlp ) job->opt_ovlp = IS_ACTIVE( ovlp );
	if ( butt == job->butt_conf ) job->opt_conf = IS_ACTIVE( conf );
	if ( butt == job->butt_tria ) job->opt_tria = IS_ACTIVE( tria );
	
	job->mask = 0;
	
	if ( job->opt_objs ) job->mask |= MARK_ACT_OUT | MARK_REF_OUT;
	if ( job->opt_conf ) job->mask |= MARK_CON_OUT;
	if ( job->opt_dist ) job->mask |= MARK_DISTANCE;
	if ( job->opt_ovlp ) job->mask |= MARK_OVERLAP;
	if ( job->opt_tria ) job->mask |= MARK_TRIANGLE;
	
	if ( job->opt_hash ) {
	
		if ( job->opt_objs ) job->mask |= MARK_ACT_INT | MARK_REF_INT;
		if ( job->opt_conf ) job->mask |= MARK_CON_INT;
	}
	
	JobCanvasForeground( job );
	JobCanvasRefresh( job );
}
