/****************************************************************************/
/* Copyright 2004, � Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	GUI image statistics functionality										*/
/*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~										*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zolt�n K�csi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <glib.h>
#include <math.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>

#include <types.h>
#include <job.h>
#include <styles.h>
#include <objects.h>
#include <distance.h>
#include <gtkwrappers.h>
#include <memory.h>
#include <stats.h>
#include <options.h>
#include "jobdefs.h"
#include "canvas.h"
#include "buttons.h"
#include "process.h"
#include "jobgui.h"

/****************************************************************************/
/*							Local definitions								*/
/****************************************************************************/

#define	SQR(x)		((x)*(x))

/****************************************************************************/
/*							Prototypes										*/
/****************************************************************************/

static	void	CubicSplineParams( double coef[ 4 ], double y[ 4 ] );
static	double	CubicSplinePoly( double coef[ 4 ], double x );
static	void	JobDrawTexts( JOB job );
static	void	StatString( int bold, GdkPixmap *pm, GdkGC *gc,
							int row, int col, char *string );
static	void	StatTexts( JOB job, GdkPixmap *pm, GdkGC *gc );
static	void	JobStatSizes( JOB job );
static	void	JobStatCurves( JOB job );

/****************************************************************************/
/*						Module public functions								*/
/****************************************************************************/

/*
*	Creates the box that contains the statistics operation items
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*JobStatPanel( JOB job, GtkWidget *box, int wide, int fill )
{
GtkWidget	*panel;

	// There are no items...

	panel = GuiVbox( FALSE, 5 );
	GuiBoxAdd( panel, box, wide, fill );
	job->pbox_stat = panel;
	gtk_widget_hide( panel );
	return( panel );
}

/*
*	This function is called when the statistics mode is entered
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	JobStatEnter( JOB job )
{
	if ( job->mode == MODE_STAT ) return;
	
	// Update current histogram column option
	
	OptionValueGet( optStatHist, &job->new.stat_hic );
	
	// Show interface
	
	gtk_widget_show( job->pbox_stat );
	gtk_widget_hide( job->canvas );
	
	// Resize the canvas and create a new RGB buffer for it
	
	gtk_drawing_area_size( GTK_DRAWING_AREA( job->canvas ), 
						   STAT_CANVAS_X, STAT_CANVAS_Y  );
	
	Free( job->rawrgb );
	job->rawrgb = Malloc( STAT_CANVAS_X * STAT_CANVAS_Y * 3 );
	memset( job->rawrgb, 0xff, STAT_CANVAS_X * STAT_CANVAS_Y * 3 );
	
	job->magnif = 1;
	job->cxs = STAT_CANVAS_X;
	job->cys = STAT_CANVAS_Y;
	
	// Create the statistical image and display it
	
	JobDrawTexts( job );
	JobStatSizes( job );
	JobStatCurves( job );
	gtk_widget_show( job->canvas );
	job->mode = MODE_STAT;
}

/*
*	This function is called when the statistics mode is left
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	JobStatLeave( JOB job )
{
	if ( job->mode != MODE_STAT ) return;
	
	// Hide interface
	
	gtk_widget_hide( job->pbox_stat );
	
	// Restore canvas size and RGB buffer
	
	gtk_drawing_area_size( GTK_DRAWING_AREA( job->canvas ), job->xs, job->ys );
	Free( job->rawrgb );
	job->rawrgb = Malloc( job->xs * job->ys * 3 );
	job->cxs  = job->xs;
	job->cys  = job->ys;
	job->mode = MODE_NONE;
}

/*
*	Refreshes the statistical canvas
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	JobStatRefresh( JOB job )
{
	if ( job->mode != MODE_STAT ) return;
	memset( job->rawrgb, 0xff, STAT_CANVAS_X * STAT_CANVAS_Y * 3 );
	JobDrawTexts( job );
	JobStatSizes( job );
	JobStatCurves( job );
	gtk_widget_hide( job->canvas );
	gtk_widget_show( job->canvas );
}

/****************************************************************************/
/*							Statistics operation							*/
/****************************************************************************/

/*
*	Gets a text, renders it and copies into the rawrgb at the given location
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	This is a horror function. I could not find anything in GTK/GDK that
*	would render text into a memory buffer. Therefore, the text is rendered
*	into a drawable, that is then grabbed into an image which, in turn, is
*	then read pixel by pixel and the result is copied into the raw RGB buffer.
*	I told you it is sheer horror.
*/

static	void	JobDrawTexts( JOB job )
{
GdkPixmap	*pixmap;
GdkImage	*image;
GdkGC		*gc;
int			x, y, rgb;
ubyte		*rgbp;

	pixmap = gdk_pixmap_new( job->canvas->window, 
							 STAT_CANVAS_X, STAT_CANVAS_Y, 24 );
	gc = gdk_gc_new( pixmap );	
	
	gdk_draw_rgb_image( pixmap, gc,
						0, 0, STAT_CANVAS_X, STAT_CANVAS_Y,
						GDK_RGB_DITHER_MAX, job->rawrgb, STAT_CANVAS_X * 3 );
						
	// Now we create the text
	
	
	StatTexts( job, pixmap, gc );	
	
	// Put the drawable into our raw RGB buffer
	
	image = gdk_image_get( pixmap, 0, 0, STAT_CANVAS_X, STAT_CANVAS_Y );
	
	for ( rgbp = job->rawrgb, y = 0 ; y < STAT_CANVAS_Y ; y++ ) {
	
		for ( x = 0 ; x < STAT_CANVAS_X ; x++, rgbp += 3 ) {
		
			rgb		  = gdk_image_get_pixel( image, x, y );
			rgbp[ 0 ] = ( rgb >> 16 ) & 255;
			rgbp[ 1 ] = ( rgb >>  8 ) & 255;
			rgbp[ 2 ] = ( rgb >>  0 ) & 255;
		}
	}
	
	// Release temporary things
	
	gdk_image_destroy( image );
	gdk_gc_unref( gc );
	gdk_pixmap_unref( pixmap );
	
}

/*
*	Writes a string into the given row and column
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	The '\t' in the string acts as a data separator
*/

static	void	StatString( int bold, GdkPixmap *pm, GdkGC *gc,
							int row, int col, char *string )
{
char	*p;
int		x, y;
GdkFont	*f;

	y = row * 15 + 20;
	x = 20 + ( col ? STAT_CANVAS_X / 2 : 0 );
	f = bold ? big_style->font : sml_style->font;
	
	for ( p = string ; *p ; p++ ) {
	
		if ( *p == '\t' ) {
		
			*p++ = '\0';
			break;
		}
	}
	
	gdk_draw_string( pm, f, gc, x, y, string );
	gdk_draw_string( pm, f, gc, x + 110, y, p );
}

#define	STRBUFF	100

/*
*	Creates the texts
*	~~~~~~~~~~~~~~~~~
*/

static	void	StatTexts( JOB job, GdkPixmap *pm, GdkGC *gc )
{
char	buff[ STRBUFF+1 ];
char	*p1, *p2;
OBJECT	o;
int		n, m;
int		line;
void	*p;
double	a = 0, t, v = 0;
	
	// Basic things
		
	for ( p2 = p1 = job->filename ; *p1 ; p1++ ) {
	
		if ( *p1 == '/' ) p2 = p1 + 1;
	}
	
	memset( buff, STRBUFF+1, 0 );
	snprintf( buff, STRBUFF, "File name:\t%s", p2 );
	StatString( TRUE, pm, gc,  0,  0, buff );
		
	// Preprocessing options
	
	StatString( TRUE, pm, gc, 2, 0, "Preprocessing" );
	
	sprintf( buff, "Ref. colour:\tR=%4.2f G=%4.2f B=%4.2f", 
			 ( ( job->opt.rcol >> 16 ) & 255 ) / 255.0,
			 ( ( job->opt.rcol >>  8 ) & 255 ) / 255.0,
			 ( ( job->opt.rcol >>  0 ) & 255 ) / 255.0 );
	StatString( FALSE, pm, gc,  3,  0, buff );
		
	sprintf( buff, "Act. colour:\tR=%4.2f G=%4.2f B=%4.2f", 
			 ( ( job->opt.acol >> 16 ) & 255 ) / 255.0,
			 ( ( job->opt.acol >>  8 ) & 255 ) / 255.0,
			 ( ( job->opt.acol >>  0 ) & 255 ) / 255.0 );
	StatString( FALSE, pm, gc,  3,  1, buff );
	
	if ( job->opt.frad )
	
		sprintf( buff, "Filter radius:\t%.1f pixels", job->opt.frad );
	else
		sprintf( buff, "Filter radius:\tno filter" );
	StatString( FALSE, pm, gc,  4,  0, buff );
	
	if ( job->opt.gamm )
	
		sprintf( buff, "Gamma:\t%4.2f", job->opt.gamm );
	else
		sprintf( buff, "Gamma:\tnone" );
	StatString( FALSE, pm, gc,  4,  1, buff );
	
	sprintf( buff, "Microscope:\t%s", 
			 job->opt.csep == OPT_COLOUR_SUBTRACT ? "light" : "fluorescent" );
	StatString( FALSE, pm, gc,  5,  0, buff );
		
	sprintf( buff, "Equalise:\t%s", job->opt.eqal ? "yes" : "no" );
	StatString( FALSE, pm, gc,  5,  1, buff );
	
	// Extraction options
	
	StatString( TRUE, pm, gc, 7, 0, "Active objects" );
	
	sprintf( buff, "Method:\t%s objects", 
					job->opt.act_meth == OPT_EXT_BLOB ? "small" : "large" );
	StatString( FALSE, pm, gc,  8,  0, buff );
	
	sprintf( buff, "Seed limit:\t%4.2f", job->opt.act_seed / 255.0 );
	StatString( FALSE, pm, gc,  9,  0, buff );
	
	sprintf( buff, "Accept limit:\t%4.2f", job->opt.act_acpt / 255.0 );
	StatString( FALSE, pm, gc, 10,  0, buff );
	
	sprintf( buff, "Min. size:\t%d pixels", job->opt.act_mins );
	StatString( FALSE, pm, gc, 11,  0, buff );
	
	sprintf( buff, "Fill holes:\t%s", 
				   job->opt.act_fill ? "yes" : "no"  );
	StatString( FALSE, pm, gc, 12,  0, buff );
	
	
	StatString( TRUE, pm, gc, 7, 1, "Reference objects" );
	
	sprintf( buff, "Method:\t%s objects", 
					job->opt.ref_meth == OPT_EXT_BLOB ? "small" : "large" );
	StatString( FALSE, pm, gc,  8,  1, buff );
	
	sprintf( buff, "Seed limit:\t%4.2f", job->opt.ref_seed / 255.0 );
	StatString( FALSE, pm, gc,  9,  1, buff );
	
	sprintf( buff, "Accept limit:\t%4.2f", job->opt.ref_acpt / 255.0 );
	StatString( FALSE, pm, gc, 10,  1, buff );
	
	sprintf( buff, "Min. size:\t%d pixels", job->opt.ref_mins );
	StatString( FALSE, pm, gc, 11,  1, buff );
	
	sprintf( buff, "Fill holes:\t%s", 
				   job->opt.ref_fill ? "yes" : "no"  );
	StatString( FALSE, pm, gc, 12,  1, buff );
	
	// Pairing options
		
	StatString( TRUE, pm, gc, 14, 0, "Distances" );
	
	sprintf( buff, "Ref. pairs:\t%s", 
					job->opt.pair_ref ? "multiple" : "single" );
	StatString( FALSE, pm, gc, 15,  0, buff );
	
	sprintf( buff, "Act. pairs:\t%s", 
					job->opt.pair_act ? "multiple" : "single" );
	StatString( FALSE, pm, gc, 16,  0, buff );
	
	sprintf( buff, "Max. distance:\t%.2f microns", (double) job->opt.pair_dis * job->opt.pair_mic / job->opt.pair_pix );
	StatString( FALSE, pm, gc, 17,  0, buff );
	
	// Restrictions
	
	StatString( TRUE, pm, gc, 14, 1, "Constraints" );
	
	sprintf( buff, "Image size:\t%d x %d", job->xs, job->ys );
	StatString( FALSE, pm, gc, 15,  1, buff );
	
	for ( n = m = 0, o = DlinkFirst( &job->con ) ; o ; o = DlinkNext( o ) ) {
	
		n += 1;
		m += ObjectArea( o );
	}
	
	sprintf( buff, "Banned zones:\t%d", n );
	StatString( FALSE, pm, gc, 16,  1, buff );
	
	sprintf( buff, "Excluded area:\t%d%%", 100 * m / job->xs / job->ys );
	StatString( FALSE, pm, gc, 17,  1, buff );
	
	// Draw distribution curves, if available
	
	if ( DlinkFirst( &job->act ) && DlinkFirst( &job->ref ) ) {
	
		if ( job->new.stat_hic == 0 ) {
			
			StatString( TRUE, pm, gc, 19,  0, 
						"Reference object size density function" );
			StatString( TRUE, pm, gc, 19,  1, 
						"Active object size density function" );
			StatString( FALSE, pm, gc, 21,  0, "X axis: equivalent object diameter in microns" );
			StatString( FALSE, pm, gc, 21,  1, "X axis: equivalent object diameter in microns" );
			
			sprintf( buff, "Number of objects:\t%d", job->actlen );
			StatString( FALSE, pm, gc, 20,  1, buff );
			
			sprintf( buff, "Number of objects:\t%d", job->reflen );
			StatString( FALSE, pm, gc, 20,  0, buff );
			
			for ( m = 0, p = DlinkFirst( &job->act ) ; p ; p = DlinkNext( p ) ) {
			
				a += sqrt( ObjectArea( p ) ) * job->opt.pair_mic / job->opt.pair_pix * 4 / 3.1415;
			}
			
			a /= job->actlen;
			
			for ( m = 0, p = DlinkFirst( &job->act ) ; p ; p = DlinkNext( p ) ) {
			
				t = sqrt( ObjectArea( p ) ) * job->opt.pair_mic / job->opt.pair_pix * 4 / 3.1415;
				v += ( t - a ) * ( t - a );
			}
			
			v /= job->actlen > 1 ? ( job->actlen - 1 ) : job->actlen;
			
			sprintf( buff, "Mean diameter:\t%g micron", a );
			StatString( FALSE, pm, gc, 22,  1, buff );
			sprintf( buff, "Diameter variance:\t%g", v );
			StatString( FALSE, pm, gc, 23,  1, buff );
			
			for ( m = 0, p = DlinkFirst( &job->ref ) ; p ; p = DlinkNext( p ) ) {
			
				a += sqrt( ObjectArea( p ) ) * job->opt.pair_mic / job->opt.pair_pix * 4 / 3.1415;
			}
			
			a /= job->reflen;
			
			for ( m = 0, p = DlinkFirst( &job->ref ) ; p ; p = DlinkNext( p ) ) {
			
				t = sqrt( ObjectArea( p ) ) * job->opt.pair_mic / job->opt.pair_pix * 4 / 3.1415;
				v += ( t - a ) * ( t - a );
			}
			
			v /= job->reflen > 1 ? ( job->reflen - 1 ) : job->reflen;
			
			sprintf( buff, "Mean diameter:\t%g micron", a );
			StatString( FALSE, pm, gc, 22,  0, buff );
			sprintf( buff, "Diameter variance:\t%g", v );
			StatString( FALSE, pm, gc, 23,  0, buff );
			
			line = 19 + 19;
		}
		else {
			StatString( TRUE, pm, gc, 19,  0, 
						"Reference object size histogram" );
			StatString( TRUE, pm, gc, 19,  1, 
						"Active object size histogram" );
			StatString( FALSE, pm, gc, 21,  0, "X axis: object area in square microns" );
			StatString( FALSE, pm, gc, 21,  1, "X axis: object area in square microns" );
			
			sprintf( buff, "Number of objects:\t%d", job->actlen );
			StatString( FALSE, pm, gc, 20,  1, buff );
			
			sprintf( buff, "Number of objects:\t%d", job->reflen );
			StatString( FALSE, pm, gc, 20,  0, buff );
			
			for ( m = 0, p = DlinkFirst( &job->act ) ; p ; p = DlinkNext( p ) ) {
			
				a += ObjectArea( p ) * SQR( (double) job->opt.pair_mic / job->opt.pair_pix );
			}
			
			a /= job->actlen;
			
			for ( m = 0, p = DlinkFirst( &job->act ) ; p ; p = DlinkNext( p ) ) {
			
				t = ObjectArea( p ) * SQR( (double) job->opt.pair_mic / job->opt.pair_pix );
				v += ( t - a ) * ( t - a );
			}
			
			v /= job->actlen > 1 ? ( job->actlen - 1 ) : job->actlen;
			
			sprintf( buff, "Mean area:\t%g square micron", a );
			StatString( FALSE, pm, gc, 22,  1, buff );
			sprintf( buff, "Area std.dev.:\t%g", sqrt( v ) );
			StatString( FALSE, pm, gc, 23,  1, buff );
			
			for ( m = 0, p = DlinkFirst( &job->ref ) ; p ; p = DlinkNext( p ) ) {
			
				a += ObjectArea( p ) * SQR( (double) job->opt.pair_mic / job->opt.pair_pix );
			}
			
			a /= job->reflen;
			
			for ( m = 0, p = DlinkFirst( &job->ref ) ; p ; p = DlinkNext( p ) ) {
			
				t = ObjectArea( p ) * SQR( (double) job->opt.pair_mic / job->opt.pair_pix );
				v += ( t - a ) * ( t - a );
			}
			
			v /= job->reflen > 1 ? ( job->reflen - 1 ) : job->reflen;
			
			sprintf( buff, "Mean area:\t%g square micron", a );
			StatString( FALSE, pm, gc, 22,  0, buff );
			sprintf( buff, "Area std.dev.:\t%g", sqrt( v ) );
			StatString( FALSE, pm, gc, 23,  0, buff );
			
			line = 19 + 19;
		}
	}
	else {
	
		line = 19;
	}
	
	// Active vs. reference results
	
	StatString( TRUE, pm, gc, line, 0, "Proximity statistics" );
	
	if ( ! job->stat ) {
	
		StatString( FALSE, pm, gc, line+1, 0, "Analysis has not been run" );
		line += 3;
	}
	else if ( job->actlen < 10 || job->reflen < 10 ) {
	
		StatString( FALSE, pm, gc, line+1, 0, "Not enougn objects for analysis" );
		line += 3;
	}
	else {
	
		sprintf( buff, "Num. of tests:\t%d", job->opt.stat_num );
		StatString( FALSE, pm, gc, line+1,  0, buff );
	
		if ( job->stat->pro < 0.05 ) {
	
			sprintf( buff, "Conclusion:\tObjects are related" );
			StatString( FALSE, pm, gc, line+2,  0, buff );
		
			if ( job->stat->pro < 0.001 )
		
				sprintf( buff, "Confidence:\t< 0.001" );
			else
				sprintf( buff, "Confidence:\t%5.3f", job->stat->pro );
		
			StatString( FALSE, pm, gc, line+3,  0, buff );
		}
		else if ( job->stat->pro < 0.2 ) {
	
			sprintf( buff, "Conclusion:\tObjects are possibly related" );
			StatString( FALSE, pm, gc, line+2,  0, buff );
			sprintf( buff, "Confidence:\t%5.2f (unconclusive)", job->stat->pro );
			StatString( FALSE, pm, gc, line+3,  0, buff );
		}
		else {
	
			sprintf( buff, "Conclusion:\tNo statistical evidence for relation between objects" );
			StatString( FALSE, pm, gc, line+2,  0, buff );
		}
		
		n = 0;
		
		for ( p = DlinkFirst( &job->dist ) ; p ; p = DlinkNext( p ), n++ );
		for ( p = DlinkFirst( &job->ovlp ) ; p ; p = DlinkNext( p ), n++ );
		
		StatString( TRUE, pm, gc, line+5, 0, "Distance/overlap density function" );
		sprintf( buff, "Pairs:\t%d", n );
		StatString( FALSE, pm, gc, line+6, 0, buff );
		StatString( FALSE, pm, gc, line+7, 0, "X axis is distance (positive) / equivalent overlap radius (negative) in microns." );
		line += 7 + 14;
	}
	
	// Active distribution results
	
	StatString( TRUE, pm, gc, line, 0, "Active object distribution statistics" );
	
	if ( ! job->tris ) {
	
		StatString( FALSE, pm, gc, line+1, 0, "Analysis has not been run" );
		line += 3;
	}
	else if ( job->actlen < 10 ) {
	
		StatString( FALSE, pm, gc, line+1, 0, "Not enougn objects for analysis" );
		line += 3;
	}
	else {
	
		sprintf( buff, "Num. of tests:\t%d", job->opt.stat_num );
		StatString( FALSE, pm, gc, line+1,  0, buff );
	
		if ( job->tris->pro < 0.05 ) {
	
			sprintf( buff, "Conclusion:\tDistribution is not random" );
			StatString( FALSE, pm, gc, line+2,  0, buff );
		
			if ( job->tris->pro < 0.001 )
		
				sprintf( buff, "Confidence:\t< 0.001" );
			else
				sprintf( buff, "Confidence:\t%5.3f", job->tris->pro );
		
			StatString( FALSE, pm, gc, line+3,  0, buff );
		}
		else if ( job->tris->pro < 0.2 ) {
	
			sprintf( buff, "Conclusion:\tDistribution might be non-random" );
			StatString( FALSE, pm, gc, line+2,  0, buff );
			sprintf( buff, "Confidence:\t%5.2f (unconclusive)", job->tris->pro );
			StatString( FALSE, pm, gc, line+3,  0, buff );
		}
		else {
	
			sprintf( buff, "Conclusion:\tNo statistical evidence for non-random distribution" );
			StatString( FALSE, pm, gc, line+2,  0, buff );
		}
		
		StatString( TRUE, pm, gc, line+5, 0, "Active object distance density function" );
		sprintf( buff, "Segments:\t%d", job->ntri );
		StatString( FALSE, pm, gc, line+6, 0, buff );
		StatString( FALSE, pm, gc, line+7, 0, "X axis is segment length in microns" );
		
		line += 7 + 13;
	}
}

/*
*	Creates the graphs for the object sizes
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	JobStatSizes( JOB job )
{
DLINK	data;
DATA	*d;
OBJECT	p;
int		n;
double	rat;
	
	if ( ! DlinkFirst( &job->act ) || ! DlinkFirst( &job->ref ) ) return;

	if ( job->new.stat_hic == 0 ) {
		
		rat = (double) job->opt.pair_mic / job->opt.pair_pix * 4 / 3.1415;
		
		DlinkInit( &data );
		for ( n=0, p = DlinkFirst( &job->act ) ; p ; p = DlinkNext( p ), n++ );
		d = Malloc( sizeof( DATA ) + sizeof( double ) * n );
		d->numd = n;
		for ( n=0, p = DlinkFirst( &job->act ) ; p ; p = DlinkNext( p ), n++ )
		
			d->data[ n ] = sqrt( (double) ObjectArea( p ) ) * rat;
		
		DlinkAddHead( &data, d );
		JobGraph( job, &data, 
				  STAT_CANVAS_X / 2 + 20, 380, 
				  STAT_CANVAS_X / 2 - 40, 150, 1.0 );
		Free( d );
	
		DlinkInit( &data );
		for ( n = 0, p = DlinkFirst( &job->ref ) ; p ; p = DlinkNext( p ), n++ );
		d = Malloc( sizeof( DATA ) + sizeof( double ) * n );
		d->numd = n;
		for ( n = 0, p = DlinkFirst( &job->ref ) ; p ; p = DlinkNext( p ), n++ )
		
			d->data[ n ] = sqrt( (double) ObjectArea( p ) ) * rat;
		
		DlinkAddHead( &data, d );
		JobGraph( job, &data, 20, 380, STAT_CANVAS_X / 2 - 40, 150, 1.0 );
		Free( d );
	}
	else {
	
		rat = (double) job->opt.pair_mic / job->opt.pair_pix;
		rat *= rat;
		
		DlinkInit( &data );
		for ( n=0, p = DlinkFirst( &job->act ) ; p ; p = DlinkNext( p ), n++ );
		d = Malloc( sizeof( DATA ) + sizeof( double ) * n );
		d->numd = n;
		for ( n=0, p = DlinkFirst( &job->act ) ; p ; p = DlinkNext( p ), n++ )
		
			d->data[ n ] = ObjectArea( p ) * rat;
		
		DlinkAddHead( &data, d );
		JobHist( job, &data, 
				  STAT_CANVAS_X / 2 + 20, 380, 
				  STAT_CANVAS_X / 2 - 40, 150, job->new.stat_hic, 1.0 );
		Free( d );
	
		DlinkInit( &data );
		for ( n = 0, p = DlinkFirst( &job->ref ) ; p ; p = DlinkNext( p ), n++ );
		d = Malloc( sizeof( DATA ) + sizeof( double ) * n );
		d->numd = n;
		for ( n = 0, p = DlinkFirst( &job->ref ) ; p ; p = DlinkNext( p ), n++ )
		
			d->data[ n ] = ObjectArea( p ) * rat;
		
		DlinkAddHead( &data, d );
		JobHist( job, &data, 20, 380, STAT_CANVAS_X / 2 - 40, 150, job->new.stat_hic, 1.0 );
		Free( d );

	}
}

static	void	JobStatCurves( JOB job )
{
int		y;
double	s;

	s = (double) job->opt.pair_mic / job->opt.pair_pix;
	
	if ( job->actlen <= 10 ) return;
	
	y = ( job->reflen <= 10 || ! DlinkFirst( &job->dsta ) ) ? 760 : 1050;
	
	if ( DlinkFirst( &job->tsta ) ) {
	
		JobGraph( job, &job->tsta, 20, y, STAT_CANVAS_X - 60, 150, s );
	}
	
	if ( job->reflen <= 10 ) return;
	
	if ( DlinkFirst( &job->dsta ) ) {
	
		JobGraph( job, &job->dsta, 20, 720, STAT_CANVAS_X - 60, 150, s );
	}
}
