/****************************************************************************/
/* Copyright 2004,  Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	Distance calculation related functionality								*/
/*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~								*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zoltn Kcsi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#ifndef __DISTANCE_H__
#define __DISTANCE_H__

#include <types.h>
#include <image.h>
#include <dlinklist.h>
#include <objects.h>

/****************************************************************************/
/*						Type definitions									*/
/****************************************************************************/

/*
*	A distance descriptor
*	~~~~~~~~~~~~~~~~~~~~~
*/

typedef	struct s_dist {

	DLINK	link;				/* List linker								*/
	RECT	pair;				/* Positions of the matching pixels			*/
	int		dist;				/* Distance between the pixels				*/
	
} DIST;

typedef struct s_ovlp {

	DLINK	link;				/* List linker								*/
	DLINK	pixs;				/* List of pixels							*/
	int		nump;				/* Number of pixels							*/
	
} OVLP;

/****************************************************************************/
/*						Interface functions									*/
/****************************************************************************/

/*
*	Maps the shortest distance between an object and a list
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters:
*		list		A list of objects to compare with
*		obj			A single object to compare against list members
*		mind		An acceptable minimum solution
*		maxd		An upper limit for the solution distance
*
*	Return value:
*		A pointer to a newly allocated DIST object or NULL
*
*	Stops mapping if the minimum criteria is satisfied.
*	Returns a DIST pointer if it found a pair less than
*	the maximum apart or NULL if not.
*/

DIST	*DistMapObjList( DLINK *list, OBJECT obj, int mind, int maxd );

/*
*	Maps the distance between two lists
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters:
*		dis		An empty list onto which the result DIST pairs will be put
*		ovl		An empty list onto which the result OVLP pairs will be put
*		ref		List of reference objects
*		act		List of active objects
*		maxdis	Maximum acceptable distance between objects
*		actmul	Flag indicating that active objects can have multiple pairs
*		refmul	Flag indicating that reference objects can have multiple pairs
*		pixlst	Flag indicating is the OVLP pixel list is needed or not
*
*	Return value:
*		TRUE is succeeded ABORT if not
*/

int		DistMapLists( DLINK *dis, DLINK *ovl, DLINK *ref, DLINK *act,
					  int maxdis, int refmul, int actmul, int pixlst );

#endif
