/****************************************************************************/
/* Copyright 1998, (c) Bendor Research Pty. Ltd.	All rights reserved.	*/
/*																			*/
/* Permission is granted to use this software for any purpose as long as	*/
/* this copyright notice remains intact in all sources.						*/
/* Bendor Research Pty. Ltd. does not guarantee the quality of this			*/
/* software and does not assume any liability whatsoever for any damages	*/
/* arising out of the use of this software.									*/
/*																			*/
/****************************************************************************/
/*																			*/
/*	This file contains a general-purpose doubly linked list library.		*/
/*																			*/
/*	History:																*/
/*																			*/
/*	10/11/98	Zoltan Kocsi	Original revision							*/
/*																			*/
/****************************************************************************/

#ifndef __LIB_DLINK_H__
#define __LIB_DLINK_H__

#ifndef	NULL
	#define	NULL	((void *) 0)
#endif

typedef	struct {

	void	*head;
	void	*tail;
	
} DLINK;

/*
*	Static initialiser for a DLINK
*/

#define	DLINK_EMPTY		((DLINK) { NULL, NULL })

/*
*	Initialises a doubly-linked list header
*/

static	inline	void	DlinkInit( DLINK *list )
{
	list->head = NULL;
	list->tail = NULL;
}

/*
*	Inserts an element to the head of the list
*/

void	DlinkAddHead( DLINK *list, void *element );

/*
*	Inserts an element to the tail of the list
*/

void	DlinkAddTail( DLINK *list, void *element );

/*
*	Inserts an element after the given one
*/

void	DlinkAddAfter( DLINK *list, void *element, void *after );

/*
*	Inserts an element before the given one
*/

void	DlinkAddBefore( DLINK *list, void *element, void *before );

/*
*	Removes the first element of the list
*/

void	*DlinkRemoveFirst( DLINK *list );

/*
*	Removes the last element of the list
*/

void	*DlinkRemoveLast( DLINK *list );

/*
*	Removes the given element from the list
*/

void	DlinkRemove( DLINK *list, void *element );

/*
*	Returns a pointer to the first element of the list
*/

static	inline	void	*DlinkFirst( DLINK *list )
{
	return( list->head );
}

/*
*	Returns a pointer to the last element of the list
*/

static	inline	void	*DlinkLast( DLINK *list )
{
	return( list->tail );
}

/*
*	Returns a pointer to the next element on the list
*/

static	inline	void	*DlinkNext( void *element )
{
	return( ((DLINK *) element)->head );
}

/*
*	Returns a pointer to the previous element of the list
*/

static	inline	void	*DlinkPrev( void *element )
{
	return( ((DLINK *) element)->tail );
}

/*
*	Appends the second list to the first one. The second list
*	becomes empty. Returns the pointer to the resulting list.
*/

DLINK	*DlinkCat( DLINK *list1, DLINK *list2 );

/*
*	This function splits the first list at the given element and puts the
*	list starting with the given element to the second list.
*/

void	DlinkSplit( DLINK *list1, DLINK *list2, void *element );

/*
*	Returns the length of a list
*/

static	inline	int	DlinkLength( DLINK *list )
{
int		length;
void	*p;

	length = 0;
	for ( p = DlinkFirst( list ) ; p ; p = DlinkNext( p ) ) length++;
	return( length );
}

#endif
