/****************************************************************************/
/* Copyright 2004,  Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	Wrapper functions around the most frequently used GTK functions			*/
/*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~		*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zoltn Kcsi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#ifndef __GTKWRAPPERS_H__
#define __GTKWRAPPERS_H__

#include <glib.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>

#define	CPLX_XSIZE	174
#define	CPLX_YSIZE	41

/*
*	Creates a horizontal packing box
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*GuiHbox( int homo, int border );

/*
*	Creates a vertical packing box
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*GuiVbox( int homo, int border );

/*
*	Adds an item to a packing box
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	GuiBoxAdd( GtkWidget *obj, GtkWidget *box, int wide, int fill );

/*
*	Creates a horizontal separator line and adds it to a box
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	GuiHline( GtkWidget *box );

/*
*	Creates a vertical separator line and adds it to a box
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	GuiVline( GtkWidget *box );

/*
*	Creates a frame and puts something in it
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*GuiFrame( char *txt, GtkStyle *style, 
					   GtkWidget *content, int shadow );;

/*
*	Creates a toggle button and puts it in a box
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*GuiToggle( char *txt, void *handler, void *param,
						GtkWidget *box, int wide, int fill );

/*
*	Creates a press button and puts it in a box
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*GuiButton( char *txt, void *handler, void *param,
						GtkWidget *box, int wide, int fill );

/*
*	Creates a label with the given style and buts it in a box
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*GuiLabel( char *txt, GtkStyle *style,
					   GtkWidget *box, int wide, int fill );

/*
*	Creates a radio button and places it in a box
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*GuiRadio( char *txt, GSList **grp, void *handler, void *param,
					   GtkWidget *box, int wide, int fill );

/*
*	Creates a menu and adds it to a box
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	The menu is surrounded by an etched out frame
*/

GtkWidget	*GuiMenu( char *txt, GtkWidget **menu, GtkWidget **list,
					  GtkWidget *box, int wide, int fill );

/*
*	Adds a menu item to menu
*	~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*GuiMenuItem( char *txt, GtkWidget *list, GSList **group,
						  void *handler, void *param );

/*
*	Creates a slider and puts it in a box
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*GuiSlider( char *txt, GtkObject **adj, GtkWidget **lab,
						void *handler, void *param,
					    GtkWidget *box, int wide, int fill );

/*
*	Creates a spin box and puts it in a box
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*GuiSpinbox( char *txt, int min, int max, 
						 GtkWidget **spin, GtkObject **adj,
						void *handler, void *param,
					    GtkWidget *box, int wide, int fill );

#endif
