/****************************************************************************/
/* Copyright 2004,  Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	Image handling															*/
/*	~~~~~~~~~~~~~~															*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zoltn Kcsi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#ifndef __IMAGE_H__
#define __IMAGE_H__

#include "types.h"

#define	R				0			/* Red layer							*/
#define	G				1			/* Green layer							*/
#define	B				2			/* Blue layer							*/
#define	MAX_LAYER		3			/* Number of colour layers				*/

/*
*	Structure for an image
*	~~~~~~~~~~~~~~~~~~~~~~
*/

typedef	struct	{

	int		x;						/* Width of the image					*/
	int		y;						/* Height of the image					*/
	int		l;						/* Number of layers						*/
	ubyte	**im[ MAX_LAYER ];		/* Actual pixels						*/

} IMAGE;

/****************************************************************************/
/*						Public functions									*/
/****************************************************************************/

/*
*	Creates an image of the given size
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

IMAGE	*ImageCreate( int y, int x, int nlayer );

/*
*	Clears a layer of an image
*	~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	ImageClear( IMAGE *im, int layer, ubyte val );

/*
*	Destroys an image
*	~~~~~~~~~~~~~~~~~
*/

void	ImageDestroy( IMAGE *image );

/*
*	Duplicates an image
*	~~~~~~~~~~~~~~~~~~~
*/

IMAGE	*ImageDup( IMAGE *src );

/*
*	Copies a layer to an image
*	~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	ImageCopyLayer( IMAGE *new, int newl, IMAGE *old, int oldl );

/*
*	Loads an image from disk
*	~~~~~~~~~~~~~~~~~~~~~~~~
*/

IMAGE	*ImageLoad( const char *name );

/*
*	Writes an RGB buffer to the disk
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Returns a success code
*/

int		TiffWrite( ubyte *buffer, int y, int x, const char *name );

#endif
