/****************************************************************************/
/* Copyright 2004,  Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	Interface for the single image ('job') processing module				*/
/*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~				*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zoltn Kcsi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#ifndef __JOB_H__
#define	__JOB_H__

/****************************************************************************/
/****************************************************************************/

/*
*	Job control commands
*	~~~~~~~~~~~~~~~~~~~~
*/

#define	JOB_CMD_NOOP			0	/* Does nothing							*/
#define	JOB_CMD_ACTIVE_ENABLE	1	/* Controls the sensitivity of active	*/
#define	JOB_CMD_ACTIVE			2	/* Controls the state of active			*/
#define	JOB_CMD_PROCESS			3	/* Instructs the job to process data	*/
#define	JOB_CMD_OPTION_LOAD		4	/* Get options from the option dialog	*/
#define	JOB_CMD_OPTION_SAVE		5	/* Put options to the option dialog		*/
#define	JOB_CMD_CLOSE			6	/* Tells the job to close down			*/
#define	JOB_CMD_COLOURS			7	/* Forces a colour update				*/
#define	JOB_CMD_ABORT			8	/* Request for abortion of operation	*/

/*
*	Requests and events
*	~~~~~~~~~~~~~~~~~~~
*/

#define	JOB_STAT_IDLE		0		/* Job does nothing						*/
#define	JOB_STAT_STARTED	1		/* Job started processing				*/
#define	JOB_STAT_FINISHED	2		/* Job finished processing				*/
#define	JOB_STAT_ABORTED	3		/* Processing was aborted				*/
#define	JOB_STAT_ACTIVE		4		/* Job's active state changed			*/
#define	JOB_REQ_CLOSE		5		/* User wants to close the job			*/
#define	JOB_REQ_ACTIVE		6		/* Tells if user wants to activate job	*/

/****************************************************************************/
/****************************************************************************/

/*
*	The opaque data structure that describes a job.
*	The structure starts with a DLINK field so that it can be placed
*	onto a doubly linked list. All other fields in the structure are 
*	private and should not be touched.
*/

typedef	struct s_job	*JOB;

/*
*	Creates a new job for the given file
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters
*
*		file:	The name of the file that should be opened.
*
*		event:	A call-back function that will be called when the
*				job has anything to report. If the job requests
*				something, this function should return TRUE or FALSE
*				indicating if it grants the request of denies it.
*
*	Return value
*
*		A reference to the newly created job or NULL, if the creation
*		is not possible
*
*/

JOB		JobCreate( const char *name, int (*event)(JOB,int,int) );

/*
*	Controls a job
*	~~~~~~~~~~~~~~
*
*	Parameters
*
*		job		The reference (obtained by JobCreate()) to the job
*		command	The command that the job has to execute
*		param	Parameter for the command
*
*	Return value
*		-
*/

void	JobControl( JOB job, int command, int param );

#endif
