/****************************************************************************/
/* Copyright 2004, � Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	Feature processing related things										*/
/*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~										*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zolt�n K�csi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#ifndef __OBJECTS_H__
#define __OBJECTS_H__

#include <types.h>
#include <image.h>
#include <dlinklist.h>

/****************************************************************************/
/*						Type definitions									*/
/****************************************************************************/

/*
*	Structure describing a rectangle
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

typedef	struct {

	short	tlx, tly;				/* Top left coordinates					*/
	short	brx, bry;				/* Bottom right coordinates				*/

} RECT;

/*
*	Opaque types
*	~~~~~~~~~~~~
*/

typedef	struct s_pixel	*PIXEL;		/* Describes a single pixel				*/
typedef struct s_object	*OBJECT;	/* An object found on the image			*/
typedef struct s_tria	*TRIA;		/* Delaunay triangle					*/

/****************************************************************************/
/*						Interface functions									*/
/****************************************************************************/

/*
*	Destroy an object
*	~~~~~~~~~~~~~~~~~
*
*	Parameters
*		obj		Reference to the object
*
*	Return value
*		-
*/

void	ObjectDestroy( OBJECT obj );

/*
*	Destroy an object list
*	~~~~~~~~~~~~~~~~~~~~~~
*
*	Parameters
*		list	Pointer to a DLINK containing the objects
*
*	Return value
*		-
*/

void	ObjectDestroyList( DLINK *list );

/*
*	Get the first pixel of an object
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
*	Parameters
*
*		obj		Reference to the object
*		border	TRUE if border pixels are needed, FALSE if area ones
*		y		Pointer to an int where the pixel's Y coordinate should be put
*		x		Pointer to an int where the pixel's X coordinate should be put
*
*	Return value
*
*		Reference to the pixel or NULL if there are no pixels
*/

PIXEL	ObjectFirstPixel( OBJECT obj, int border, int *y, int *x );

/*
*	Get the next pixel of an object
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
*	Parameters
*
*		obj		Reference to the object
*		pix		Reference to the previous pixel
*		y		Pointer to an int where the pixel's Y coordinate should be put
*		x		Pointer to an int where the pixel's X coordinate should be put
*
*	Return value
*
*		Reference to the pixel or NULL if the previous was the last pixel
*/

PIXEL	ObjectNextPixel( OBJECT obj, PIXEL pix, int *y, int *x );

/*
*	Get the coordinates of a pixel
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
*	Parameters
*
*		pix		Reference to the pixel
*		y		Pointer to an int where the pixel's Y coordinate should be put
*		x		Pointer to an int where the pixel's X coordinate should be put
*
*	Return value
*		-
*/

void	ObjectPixelCoords( PIXEL pix, int *y, int *x );

/*
*	Returns the area of an object
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
*	Parameters
*
*		obj		Reference to the object
*
*	Return value
*		The area in pixels
*/

int		ObjectArea( OBJECT obj );

/*
*	Returns the weighted centre of an object
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	ObjectCentre( OBJECT obj, double *x, double *y );

#endif
