/****************************************************************************/
/* Copyright 2004, � Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	Types of various program options										*/
/*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~										*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zolt�n K�csi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#ifndef __OPTIONS_H__
#define	__OPTIONS_H__

#include <glib.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>

#include <types.h>

/****************************************************************************/
/*						Definitions											*/
/****************************************************************************/

#define	OPT_EXT_BLOB			0
#define	OPT_EXT_TWIG			1

#define	OPT_COLOUR_SUBTRACT		0
#define	OPT_COLOUR_ADDITIVE		1

#define	OPT_SELF_TREE			0
#define	OPT_SELF_TRIA			1
#define	OPT_SELF_NEIG			2
#define	OPT_SELF_RATS			3

/*
*	Option identifiers
*/

typedef	enum {

	optNone = 0,					/* Nonexistent option					*/
	
	// Preprocessing options
	
	optPreMicrosc,					/* Colour separation selection			*/
	optPreColRef,					/* Reference object base colour			*/
	optPreColAct,					/* Active object base colour			*/
	optPreFilter,					/* Filter radius						*/
	optPreEqual,					/* Equalisation is needed				*/
	optPreGamma,					/* Gamma correction value				*/
	
	// Reference object extraction options
	
	optExtRefAlg,					/* Extraction method					*/
	optExtRefSeed,					/* Minimum seed value					*/
	optExtRefAcpt,					/* Minimum acceptance value				*/
	optExtRefSize,					/* Minimum object size					*/
	optExtRefFill,					/* Fill object							*/
	optExtRefExcl,					/* Exclude boundary objects				*/
	
	// Active object extraction options
	
	optExtActAlg,					/* Extraction method					*/
	optExtActSeed,					/* Minimum seed value					*/
	optExtActAcpt,					/* Minimum acceptance value				*/
	optExtActSize,					/* Minimum object size					*/
	optExtActFill,					/* Fill object							*/
	optExtActExcl,					/* Exclude boundary objects				*/
	
	// Pairing options
	
	optPairMicron,					/* Scaling factor micron component		*/
	optPairPixels,					/* Scaling factor pixel component		*/ 
	optPairDistMax,					/* Max. distance between pairs			*/
	optPairRefMul,					/* Max. actives paired to reference		*/
	optPairActMul,					/* Max. refernces paired to an active	*/
	
	// Statistical options
	
	optStatSnum,					/* Number of random samples				*/
	optStatSelf,					/* Active object distance method		*/
	optStatRunc,					/* Run colocalisation test				*/
	optStatRund,					/* Run distribution test				*/
	optStatHist,					/* Number of historgram columns			*/
	
	// Display colour options
	
	optColRefObj,					/* Colour for reference objects			*/
	optColActObj,					/* Colour for reference objects			*/
	optColDistance,					/* Colour for distance vectors			*/
	optColConfine,					/* Colour for confinement area			*/
		
	optLast
	
} typOpt;

/****************************************************************************/
/*							Prototypes										*/
/****************************************************************************/

/*
*	Builds the option GUI
*	~~~~~~~~~~~~~~~~~~~~~
*	Parameters:
*		-
*
*	Returns:
*
*		A GTK widget representing the outermost option widget
*/

GtkWidget	*OptionsGuiCreate( void );

/*
*	Locks/enables the options
*	~~~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters:
*		sens		A boolean showing if the options should be sensitive
*
*	Return value:
*		-
*/

void	OptionsSensitive( int sens );

/*
*	Cleans up all GUI items
*	~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters:
*		-
*
*	Returns:
*		-
*
*	Destroys all internal references to GTK widgets because they
*	can no longer be accessed.
*/

void	OptionsCleanupGui( void );

/*
*	Returns the value of the given option
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters
*	
*		option:		Selects which option to retrieve
*
*		data:		Pointer to the option value (type depends on option)
*
*	Return value
*		-
*/

void	OptionValueGet( typOpt option, void *data );

/*
*	Loads sensible default options
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters: -
*	Returns:	-
*/

void	OptionsLoadDefaults( void );

/*
*	Sets the value of the given option
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters
*	
*		option:		Selects which option to set
*
*		data:		Pointer to the option value (type depends on option)
*
*	Return value
*		-
*/

void	OptionValueSet( typOpt option, void *data );

/*
*	Loads options from the command line
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters
*		argc:	Max. number of arguments to scan
*		argv:	Argument vector
*
*	Returns:
*		The index of the element which is not a valid option, or argc
*		if it scanned all available arguments
*/

int		OptionsLoadCommandline( int argc, char **argv );

/*
*	Loads options from a file
*	~~~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters:
*		filename:	The name of the file the options hysould be written to
*
*	Returns:
*		-
*/

void	OptionsLoadFile( char *filename, void *unused );

/*
*	Save options to the given file
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters:
*		filename:	The name of the file the options hysould be written to
*
*	Returns:
*		TRUE if succeeded, FALSE if it can't open the file
*/

int		OptionsSaveFile( char *filename );

#endif
