/****************************************************************************/
/* Copyright 2004,  Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	Statistical analysis functionality										*/
/*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~										*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zoltn Kcsi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#ifndef __STATISTICS_H__
#define __STATISTICS_H__

#include <types.h>
#include <dlinklist.h>

#define	STAT_POINTS			20
#define	MEAN				0
#define	SMIN				1
#define	SMAX				2
#define	ORIG				3

typedef	struct	{

	int		min;							/* Maximum overlap				*/
	int		max;							/* Maximum distance				*/
	double	pro;							/* Probability of error			*/
	double	pnts[ 4 ][ STAT_POINTS ];		/* Sample points				*/
	
} STAT;

typedef	struct	s_sinfo		*SINFO;			/* Temporary data handle		*/

/*
*	Prepares the statistical analysis
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters:
*		dist		A doubly-linked list of DIST objects that describe
*					the distances of the sample data
*		ovlp		A doubly-linked list of OVLP objects that describe
*					the overlaps of the sample data
*		maxd		The maximum distance on the list
*		tests		The number of tests that will be performed
*
*	Return value:
*		A statistical data structure pointer
*/

SINFO	StatPrepare( DLINK *dist, DLINK *ovlp );

/*
*	One run of the new data
*	~~~~~~~~~~~~~~~~~~~~~~~
*	Adds a new set of random data to the statistical pool
*	
*	Parameters:
*		sdesc		The statistical data obtained by StatPrepare()
*		dist		A doubly linked list of the random distances
*		ovlp		A doubly-linked list of the random overlaps
*
*	Return value:
*		-
*
*	The dist and ovlp lists are retained by the routine and the relevant
*	DLINK objects will be empty.
*/

void	StatRound( SINFO sdesc, DLINK *dist, DLINK *ovlp );

/*
*	Releases the info structure
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters:
*		info		The statistical data obtained by StatPrepare()
*
*	Return value:
*		-
*/

void	StatDestroy( SINFO info );

/*
*	Calculates statistical results from the gathered data
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters:
*		info		The statistical data obtained by StatPrepare()
*
*	Return value:
*		A Malloc-ed structure containing the statistical results
*		or NULL, if the statistics could not be calculated.
*
*	The dist and ovlp lists are retained by the routine and the relevant
*	DLINK objects will be empty.
*/

STAT	*StatFinal( SINFO info );

#endif
