/****************************************************************************/
/* Copyright 2004, � Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	Accessing option descriptors and values									*/
/*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~									*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zolt�n K�csi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <glib.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>

#include <types.h>
#include <options.h>
#include <fatal.h>
#include <gui.h>
#include "option_types.h"

/****************************************************************************/
/*							Prototypes										*/
/****************************************************************************/

static	void	OptionSet( char *cli, char *val );

/****************************************************************************/
/*							Prototypes										*/
/****************************************************************************/

static	int		sensitive = TRUE;

/****************************************************************************/
/*							Local data										*/
/****************************************************************************/

static	OPTION	optdes[] = {

{ otiVbox,	NULL,						NULL,	optNone,		NULL	  },
{ otiHbox,	NULL,						NULL,	optNone,		NULL	  },
{ otiVbox,	NULL,						NULL,	optNone,		NULL	  },
{ otiLabl,	"Preprocessing",			NULL,	optNone,		NULL	  },
{ otiList,	"Microscope",				"mict",	optPreMicrosc,	&colour	  },
{ otiCsel,	"Reference colour",			"rcol",	optPreColRef,	&col_ref  },
{ otiCsel,	"Active colour",			"acol",	optPreColAct,	&col_act  },
{ otiSlid,	"Filter kernel radius",		"filt",	optPreFilter,	&filter	  },
{ otiSlid,	"Gamma correction",			"gamm",	optPreGamma,	&gammac	  },
{ otiFlag,	"Equalise intensity",		"eqal", optPreEqual,	NULL	  },
{ otiEbox,	NULL,						NULL,	optNone,		NULL	  },
{ otiSepl,	NULL,						NULL,	optNone,		NULL	  },
{ otiVbox,	NULL,						NULL,	optNone,		NULL	  },
{ otiLabl,	"Reference extraction",		NULL,	optNone,		NULL	  },
{ otiList,	"Algorithm",				"real",	optExtRefAlg,	&ref_extr },
{ otiSlid,	"Minimum seed value",		"resd",	optExtRefSeed,	&ref_seed },
{ otiSlid,	"Minimum acceptance",		"reac",	optExtRefAcpt,	&ref_acpt },
{ otiSlid,	"Minimum object size",		"rems",	optExtRefSize,	&ref_size },
{ otiFlag,	"Fill holes in objects",	"refh",	optExtRefFill,	NULL	  },
{ otiFlag,	"Exclude border objects",	"reeb",	optExtRefExcl,	NULL	  },
{ otiEbox,	NULL,						NULL,	optNone,		NULL	  },
{ otiSepl,	NULL,						NULL,	optNone,		NULL	  },
{ otiVbox,	NULL,						NULL,	optNone,		NULL	  },
{ otiLabl,	"Active extraction",		NULL,	optNone,		NULL	  },
{ otiList,	"Algorithm",				"aeal",	optExtActAlg,	&act_extr },
{ otiSlid,	"Minimum seed value",		"aesd",	optExtActSeed,	&act_seed },
{ otiSlid,	"Minimum acceptance",		"aeac",	optExtActAcpt,	&act_acpt },
{ otiSlid,	"Minimum object size",		"aems",	optExtActSize,	&act_size },
{ otiFlag,	"Fill holes in objects",	"aefh",	optExtActFill,	NULL	  },
{ otiFlag,	"Exclude border objects",	"aeeb",	optExtActExcl,	NULL	  },
{ otiEbox,	NULL,						NULL,	optNone,		NULL	  },
{ otiEbox,	NULL,						NULL,	optNone,		NULL	  },
{ otiSepl,	NULL,						NULL,	optNone,		NULL	  },
{ otiHbox,	NULL,						NULL,	optNone,		NULL	  },
{ otiVbox,	NULL,						NULL,	optNone,		NULL	  },
{ otiLabl,	"Object pairing",			NULL,	optNone,		NULL	  },
{ otiSpin,	"Micron",					"micr",	optPairMicron,	&micron	  },
{ otiSpin,	"Pixels",					"pixs",	optPairPixels,	&pixels   },
{ otiSlid,	"Maximum distance",			"pdmx",	optPairDistMax,	&distmax  },
{ otiFlag,	"Many pairs for references","rpmp",	optPairRefMul,	NULL	  },
{ otiFlag,	"Many pairs for actives",	"apmp",	optPairActMul,	NULL	  },
{ otiEbox,	NULL,						NULL,	optNone,		NULL	  },
{ otiSepl,	NULL,						NULL,	optNone,		NULL	  },
{ otiVbox,	NULL,						NULL,	optNone,		NULL	  },
{ otiLabl,	"Statistics",				NULL,	optNone,		NULL	  },
{ otiSlid,	"Number of samples",		"snum",	optStatSnum,	&statsnum },
{ otiList,	"Active distances",			"sadi",	optStatSelf,	&stat_dis },
{ otiFlag,	"Run colocalisation test",	"scol",	optStatRunc,	NULL	  },
{ otiFlag,	"Run distribution test",	"sdis",	optStatRund,	NULL	  },

{ otiSlid,	"Histogram columns",		"hist",	optStatHist,	&stathist },

{ otiEbox,	NULL,						NULL,	optNone,		NULL	  },
{ otiSepl,	NULL,						NULL,	optNone,		NULL	  },
{ otiVbox,	NULL,						NULL,	optNone,		NULL	  },
{ otiLabl,	"Display colours",			NULL,	optNone,		NULL	  },
{ otiCset,	"Reference objects",		" roc",	optColRefObj,	&ref_col  },
{ otiCset,	"Active objects",			" aoc",	optColActObj,	&act_col  },
{ otiCset,	"Distance segments",		" dsc",	optColDistance,	&dis_col  },
{ otiCset,	"Confinement area",			" cac",	optColConfine,	&con_col  },
{ otiEbox,	NULL,						NULL,	optNone,		NULL	  },
{ otiEbox,	NULL,						NULL,	optNone,		NULL	  },
{ otiEbox,	NULL,						NULL,	optNone,		NULL	  },
{ otiLast,	NULL,						NULL,	optNone,		NULL	  }
};

static	char	optvals[ optLast ][ OPT_STRLEN + 1 ];

/****************************************************************************/
/*						Module interface functions							*/
/****************************************************************************/

/*
*	Builds the option GUI
*	~~~~~~~~~~~~~~~~~~~~~
*	Parameters:
*		-
*
*	Returns:
*
*		A GTK widget representing the outermost option widget
*/

GtkWidget	*OptionsGuiCreate( void )
{
GtkWidget	*opts;

	opts = OptionsBuildGui( optdes );
	OptionsSensitive( sensitive );
	return( opts );
}

/*
*	Locks/enables the options
*	~~~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters:
*		sens		A boolean showing if the options should be sensitive
*
*	Return value:
*		-
*/

void	OptionsSensitive( int sens )
{
OPTION		*p;
GtkWidget	*wid;
       
	// Store the state
	
	sensitive = sens;
	
	// We just have to run through and clean all widget references
	
	for ( p = optdes ; p->type != otiLast ; p++ ) {
	
		switch ( p->type ) {
		
			case otiFlag:
			
				wid = p->desc;
				break;
		
			case otiSlid:
			
				wid = ((SLIDER *) p->desc)->label;
				if ( wid ) gtk_widget_set_sensitive( wid, sens );
				wid = ((SLIDER *) p->desc)->slide;
				break;
				
			case otiList:
			
				wid = ((MENU *) p->desc)->menu;
				break;
			
			case otiCsel:
			
				wid = ((CSEL *) p->desc)->button;
				break;
				
			default:
				
				wid = NULL;
				break;
		}
		
		if ( wid ) gtk_widget_set_sensitive( wid, sens );
	}
}


/*
*	Cleans up all GUI items
*	~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters:
*		-
*
*	Returns:
*		-
*
*	Destroys all internal references to GTK widgets because they
*	can no longer be accessed.
*/

void	OptionsCleanupGui( void )
{
OPTION	*p;
ITEM	*i;

	// We just have to run through and clean all widget references
	
	for ( p = optdes ; p->type != otiLast ; p++ ) {
	
		switch ( p->type ) {
		
			case otiFlag:
			
				p->desc = NULL;
				break;
		
			case otiSlid:
			
				((SLIDER *) p->desc)->slide = NULL;
				((SLIDER *) p->desc)->label = NULL;
				((SLIDER *) p->desc)->adjus = NULL;
				break;
				
			case otiList:
			
				((MENU *) p->desc)->menu = NULL;				
				((MENU *) p->desc)->list = NULL;
				
				for ( i = ((MENU *) p->desc)->items ; i->cid ; i++ ) {
				
					i->wid = NULL;
				}
				
				break;
			
			case otiCsel:
			case otiCset:
			
				((CSEL *) p->desc)->canvas = NULL;
				((CSEL *) p->desc)->dialog = NULL;
				break;
				
			default:
				break;
		}
	}
}

/*
*	Returns the value of the given option
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters
*	
*		option:		Selects which option to retrieve
*
*		data:		Pointer to the option value (type depends on option)
*
*	Return value
*		-
*/

void	OptionValueGet( typOpt option, void *data )
{
OPTION	*p;
MENU	*m;
ITEM	*i;
SLIDER	*s;
CSEL	*c;

	if ( ! ( p = OptionFindId( option ) ) ) return;
	
	switch ( p->type ) {
	
		case otiFlag:
		
			*((int *) data) = ! strcmp( OptionValue( p->optid ), "yes" );
			break;
			
		case otiList:
		
			m = p->desc;
			
			for ( i = m->items ; i->cid ; i++ ) {
			
				if ( ! strcmp( i->cid->cli, OptionValue( p->optid ) ) ) {
				
					*((int *) data) = i->cid->val;
					return;
				}
			}
				
			FATAL( "Option %d has invalid value \"%s\"", 
				   option, OptionValue( p->optid ) );
			break;
		
		case otiSlid:
		
			s = p->desc;
			(*s->str_val)( p, OptionValue( p->optid ), data );
			break;
			
		case otiSpin:
		
			sscanf( OptionValue( p->optid ), "%d", data );
			break;
			
		case otiCsel:
		case otiCset:
		
			c = p->desc;
			sscanf( OptionValue( p->optid ), "%x", data );
			if ( ! ( *((int *) data) &= 0xffffff ) ) *((int *) data) = 1;
			break;
			
		default:
			FATAL( "Option %d has illegal type %d", option, p->type );
			break;
	}
}	

/*
*	Sets the value of the given option
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters
*	
*		option:		Selects which option to set
*
*		data:		Pointer to the option value (type depends on option)
*
*	Return value
*		-
*/

void	OptionValueSet( typOpt option, void *data )
{
OPTION	*p;
MENU	*m;
ITEM	*i;
SLIDER	*s;
SPINNER	*r;
CSEL	*c;
int		n;

	if ( ! ( p = OptionFindId( option ) ) ) {
	
		return;
	}
	
	switch ( p->type ) {
	
		case otiFlag:
		
			if ( p->desc ) {
			
				gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( p->desc ),
											  *((int *) data) );
			}
			
			strcpy( OptionValue( p->optid ), *((int*)data) ? "yes" : "no" );
			break;
		
		case otiList:

			m = p->desc;
			
			for ( n = 0, i = m->items ; i->cid ; i++, n++ ) {
			
				if ( *((int *) data) == i->cid->val ) {
				
					if ( m->menu ) {
					
						gtk_option_menu_set_history(GTK_OPTION_MENU(m->menu),n);
					}
					
					strcpy( OptionValue( p->optid ), i->cid->cli );
					return;
				}
			}
				
			FATAL( "Setting illegal choice %d for option %d",
				   *((int*)data), option );
			break;
			
		case otiSpin:
		
			r = p->desc;
			
			if ( r->adjus ) {
			
				gtk_adjustment_set_value( GTK_ADJUSTMENT( r->adjus ), 
									  	  *((int *)data) ); 
			}
			
			sprintf( OptionValue( p->optid ), "%d", *((int *)data) ); 
			break;
						
		case otiSlid:
		
			s = p->desc;
			(*s->val_str)( p, OptionValue( p->optid ), data );
			
			if ( s->adjus ) {
			
				gtk_adjustment_set_value( GTK_ADJUSTMENT( s->adjus ), 
									  (*s->str_pos)(p, OptionValue(p->optid)) );
			}
			
			break;
			
		case otiCsel:
		case otiCset:
		
			c = p->desc;
			n = *((int *) data) & 0xffffff;
			sprintf( OptionValue( option ), "%06x", n );			
			c->colour = n;
			
			GuiColourChange();
			
			if ( c->canvas ) {
			
				gtk_widget_hide( c->canvas );
				gtk_widget_show( c->canvas );
			}
			
			break;
			
		default:
			
			FATAL( "Option %d has illegal type %d", option, p->type );
			break;
	}
}

/*
*	Loads sensible default options
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters: -
*	Returns:	-
*/

void	OptionsLoadDefaults( void )
{
	// Preprocessing
	
	OptionSet( "mict",		"lm"		);
	OptionSet( "rcol",		"cc763d"	);
	OptionSet( "acol",		"cc527a"	);
	OptionSet( "filt",		"6.5"		);
	OptionSet( "eqal",		"yes"		);
	OptionSet( "gamm",		"2.00"		);	// To force a refresh
	OptionSet( "gamm",		"1.30"		);
	
	// Reference object extraction
	
	OptionSet( "real",		"twig"		);
	OptionSet( "resd",		"170"		);
	OptionSet( "reac",		"70"		);
	OptionSet( "rems",		"50"		);
	OptionSet( "refh",		"no"		);
	
	// Active object extraction
	
	OptionSet( "aeal",		"twig"		);
	OptionSet( "aesd",		"90"		);
	OptionSet( "aeac",		"70"		);
	OptionSet( "aems",		"50"		);
	OptionSet( "aefh",		"yes"		);
	
	// Distance options
	
	OptionSet( "micr",		"100"		);
	OptionSet( "pixs",		"100"		);
	OptionSet( "pdmx",		"100"		);
	OptionSet( "apmp",		"no"		);
	OptionSet( "rpmp",		"yes"		);
	
	// Default colours
	
	OptionSet( " roc",		"a56b44"	);
	OptionSet( " aoc",		"388eb2"	);
	OptionSet( " dsc",		"ffff00"	);
	OptionSet( " cac",		"0000ff"	);
	
	// Statistics
	
	OptionSet( "snum",		"250"		);
	OptionSet( "sadi",		"tree"		);
	OptionSet( "scol",		"yes"		);
	OptionSet( "sdis",		"yes"		);
}

/*
*	Loads options from the command line
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters
*		argc:	Max. number of arguments to scan
*		argv:	Argument vector
*
*	Returns:
*		The index of the element which is not a valid option, or argc
*		if it scanned all available arguments
*/

int		OptionsLoadCommandline( int argc, char **argv )
{
int		i;
char	*par;

	for ( i = 0 ; i < argc ; i++ ) {
	
		if ( argv[ i ][ 0 ] != '-' ) return( i );
		if ( ! ( par = strchr( argv[ i ] + 1, '=' ) ) ) return( i );
		if ( ! par[ 1 ] ) return( i );
		*par = '\0';
		OptionSet( argv[ i ] + 1, par + 1 );
	}
	
	return( i );
}

/*
*	Loads options from a file
*	~~~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters:
*		filename:	The name of the file the options sould be written to
*
*	Returns:
*		-
*/

void	OptionsLoadFile( char *filename, void *unused )
{
FILE	*file;
char	name[ 11 ], value[ 11 ];

	if ( ! ( file = fopen( filename, "r" ) ) ) return;
	
	while ( fscanf( file, " \"%10[^\"]\" %10s", name, value ) == 2 ) {
	
		OptionSet( name, value );
	}
	
	fclose( file );
}

/*
*	Save options to the given file
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters:
*		filename:	The name of the file the options sould be written to
*
*	Returns:
*		TRUE if succeeded, FALSE if it can't open the file
*/

int		OptionsSaveFile( char *filename )
{
FILE	*file;
OPTION	*p;

	if ( ! ( file = fopen( filename, "w" ) ) ) return( FALSE );
	
	for ( p = optdes ; p->type != otiLast ; p++ ) {
	
		if ( p->optid != optNone ) {
		
			fprintf( file, "\"%s\" %s\n", p->cli, OptionValue( p->optid ) );
		}
	}
	
	fclose( file );
	return( TRUE );
}

/****************************************************************************/
/*					Public functions used within the module					*/
/****************************************************************************/

/*
*	Finds an option descriptor by the option's ID
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

OPTION	*OptionFindId( typOpt option )
{
OPTION	*p;

	for ( p = optdes ; p->optid != option && p->optid != optLast ; p++ );
	
	if ( p->optid == optLast ) {
	
		FATAL( "Request for nonexistent option %d", option );
		return( NULL );
	}
	
	return( p );
}

/*
*	Returns the address of the option's value string by its ID
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

char	*OptionValue( typOpt option )
{
	return( optvals[ option ] );
}

/****************************************************************************/
/*						Local functions										*/
/****************************************************************************/

/*
*	Sets an option value from the string representation
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	OptionSet( char *cli, char *val )
{
int		n;
OPTION	*p;
MENU	*m;
ITEM	*i;
SLIDER	*s;
CSEL	*c;
double	d;

	for ( p = optdes ; p->type != otiLast ; p++ ) {
	
		if ( ! p->cli || strcmp( cli, p->cli ) ) continue;
		
		switch ( p->type ) {
		
			case otiFlag:
			
				n = ! strcmp( val, "yes" );
				OptionValueSet( p->optid, &n );
				break;
				
			case otiList:
			
				m = p->desc;
			
				for ( i = m->items ; i->cid ; i++ ) {
			
					if ( ! strcmp( i->cid->cli, val ) ) {
				
						OptionValueSet( p->optid, &i->cid->val );
						break;
					}
				}
				
				if ( ! i->cid ) return;
				break;
			
			case otiSpin:
			
				sscanf( val, "%d", &n );
				OptionValueSet( p->optid, &n );
				break;
			
			case otiSlid:
			
				s = p->desc;
				(*s->pos_str)( p, OptionValue( p->optid ),
							   (*s->str_pos)( p, val ) );
				if ( s->adjus ) {
				
					gtk_adjustment_set_value( GTK_ADJUSTMENT( s->adjus ), 
									  (*s->str_pos)(p, OptionValue(p->optid)) );
				}
				
				break;
		
			case otiCsel:
			case otiCset:
			
				c = p->desc;
				sscanf( val, "%x", &c->colour );
				OptionValueSet( p->optid, &c->colour );

			default:
				return;
		}
		
		break;
	}
}

