/****************************************************************************/
/* Copyright 2004,  Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	Distance mapping options												*/
/*	~~~~~~~~~~~~~~~~~~~~~~~~												*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zoltn Kcsi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <math.h>

#include <types.h>
#include <options.h>
#include <fatal.h>
#include "option_types.h"

/****************************************************************************/
/*							Local definitions								*/
/****************************************************************************/

#define	DMAX_MIN	10
#define	DMAX_MAX	200
#define	DMAX_RES	10

#define	TIME_MIN	0
#define	TIME_MAX	5
#define	TIME_RES	0.1

/****************************************************************************/
/*								Prototypes									*/
/****************************************************************************/

static	void	DmaxSV( OPTION *opt, char *str, void *val );
static	void	DmaxVS( OPTION *opt, char *str, void *val );
static	double	DmaxSP( OPTION *opt, char *str );
static	void	DmaxPS( OPTION *opt, char *str, double val );
static	void	SlideChange( OPTION *obj, int newwal );

/****************************************************************************/
/*					Data describing the option items						*/
/****************************************************************************/

	SLIDER	distmax = DEF_SLID( DmaxSV, DmaxVS, DmaxSP, DmaxPS, NULL );
	SPINNER	micron	= DEF_SPIN( 1, 500, SlideChange );
	SPINNER	pixels  = DEF_SPIN( 1, 500, SlideChange );

/****************************************************************************/
/*					Dmax kernel diameter slider functions					*/
/****************************************************************************/

/*
*	Converts a string to a user value
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	DmaxSV( OPTION *opt, char *str, void *val )
{
double	dval;
int		pixs;
int		mics;
	
	OptionValueGet( optPairMicron, &mics );
	OptionValueGet( optPairPixels, &pixs );
	
	sscanf( str, "%lf", &dval );
	
	*((int *) val) = dval / mics * pixs;
}

/*
*	Converts a user value to a string
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	DmaxVS( OPTION *opt, char *str, void *val )
{
int		ival;
double	dval;
int		pixs;
int		mics;
	
	OptionValueGet( optPairMicron, &mics );
	OptionValueGet( optPairPixels, &pixs );
	
	ival = *((int *) val);
	if ( ival < DMAX_MIN ) ival = DMAX_MIN;
	if ( ival > DMAX_MAX ) ival = DMAX_MAX;
	
	dval = (double) ival * mics / pixs;
	sprintf( str, "%7.2f", dval );
}

/*
*	Converts a string into a position value
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	double	DmaxSP( OPTION *opt, char *str )
{
int		val;
double	dva;

	DmaxSV( opt, str, &val );
	if ( val < DMAX_MIN ) val = DMAX_MIN;
	if ( val > DMAX_MAX ) val = DMAX_MAX;
	return( (double) ( val - DMAX_MIN ) / ( DMAX_MAX - DMAX_MIN ) );
}

/*
*	Converts a position value to a string
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	DmaxPS( OPTION *opt, char *str, double val )
{
int		ival;

	val *= DMAX_MAX - DMAX_MIN;
	ival = (int)( val / DMAX_RES + 0.5 ) * DMAX_RES + DMAX_MIN;
	DmaxVS( opt, str, &ival );
}
		
/****************************************************************************/
/*					Object time slider										*/
/****************************************************************************/

static	void	SlideChange( OPTION *obj, int newwal )
{
int		val;
int		tm1 = 0, tm2 = 100;

	OptionValueSet( obj->optid, &newwal );
	
//	OptionValueGet( optExtActSize, &val );
//	OptionValueSet( optExtActSize, &val );
	
//	OptionValueGet( optExtRefSize, &val );
//	OptionValueSet( optExtRefSize, &val );

	OptionValueGet( optPairDistMax, &val );
	OptionValueSet( optPairDistMax, &tm1 );
	OptionValueSet( optPairDistMax, &tm2 );
	OptionValueSet( optPairDistMax, &val );
}
