/****************************************************************************/
/* Copyright 2004,  Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	Extracting options														*/
/*	~~~~~~~~~~~~~~~~~~														*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zoltn Kcsi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#include <stdio.h>
#include <math.h>

#include <types.h>
#include <options.h>
#include <fatal.h>
#include "option_types.h"

/****************************************************************************/
/*							Local definitions								*/
/****************************************************************************/

#define	SEED_MIN	50
#define	SEED_MAX	250
#define	SEED_RES	5

#define	SIZE_MIN	25
#define	SIZE_MAX	250
#define	SIZE_RES	5

/****************************************************************************/
/*								Prototypes									*/
/****************************************************************************/

static	void	SeedSV( OPTION *opt, char *str, void *val );
static	void	SeedVS( OPTION *opt, char *str, void *val );
static	double	SeedSP( OPTION *opt, char *str );
static	void	SeedPS( OPTION *opt, char *str, double val );
static	void	SeedUser( OPTION *opt );

static	void	SizeSV( OPTION *opt, char *str, void *val );
static	void	SizeVS( OPTION *opt, char *str, void *val );
static	double	SizeSP( OPTION *opt, char *str );
static	void	SizePS( OPTION *opt, char *str, double val );

/****************************************************************************/
/*					Data describing the option items						*/
/****************************************************************************/

static	CHOICE	exb		= { "Small objects",	"blob",	OPT_EXT_BLOB };
static	CHOICE	ext		= { "Large objects",	"twig",	OPT_EXT_TWIG };
static	ITEM	aext[]  = { DEF_ITEM( exb ), DEF_ITEM( ext ), DEF_LAST };
static	ITEM	rext[]  = { DEF_ITEM( exb ), DEF_ITEM( ext ), DEF_LAST };

	MENU	act_extr = DEF_MENU( aext );
	SLIDER	act_seed = DEF_SLID( SeedSV, SeedVS, SeedSP, SeedPS, SeedUser );
	SLIDER	act_acpt = DEF_SLID( SeedSV, SeedVS, SeedSP, SeedPS, SeedUser );
	SLIDER	act_size = DEF_SLID( SizeSV, SizeVS, SizeSP, SizePS, NULL );
		
	MENU	ref_extr = DEF_MENU( rext );
	SLIDER	ref_seed = DEF_SLID( SeedSV, SeedVS, SeedSP, SeedPS, SeedUser );
	SLIDER	ref_acpt = DEF_SLID( SeedSV, SeedVS, SeedSP, SeedPS, SeedUser );
	SLIDER	ref_size = DEF_SLID( SizeSV, SizeVS, SizeSP, SizePS, NULL );
		
/****************************************************************************/
/*				Seed and acceptance value slider functions					*/
/****************************************************************************/


/*
*	Converts a string to a user value
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	SeedSV( OPTION *opt, char *str, void *val )
{
double	d;

	sscanf( str, "%lf", &d );
	*((int *) val) = d * 255.0 + 0.5;
}

/*
*	Converts a user value to a string
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	SeedVS( OPTION *opt, char *str, void *val )
{
int		d;
	
	d = *((int *) val);
	if ( d < SEED_MIN ) d = SEED_MIN;
	if ( d > SEED_MAX ) d = SEED_MAX;
	sprintf( str, "%4.2lf", d / 255.0 );
}

/*
*	Converts a string into a position value
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	double	SeedSP( OPTION *opt, char *str )
{
int		val;

	SeedSV( opt, str, &val );
	if ( val < SEED_MIN ) val = SEED_MIN;
	if ( val > SEED_MAX ) val = SEED_MAX;
	return( (double) ( val - SEED_MIN ) / ( SEED_MAX - SEED_MIN ) );
}

/*
*	Converts a position value to a string
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	SeedPS( OPTION *opt, char *str, double val )
{
int 	ival;

	val  *= SEED_MAX - SEED_MIN;
	ival  = (int)( val / SEED_RES + 0.5 ) * SEED_RES + SEED_MIN;
	SeedVS( opt, str, &ival );
}

/*
*	Additional action associated with the slider
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	SeedUser( OPTION *opt )
{
int		this, that;

	switch ( opt->optid ) {
	
		case optExtActSeed:
		
			OptionValueGet( optExtActSeed, &this );
			OptionValueGet( optExtActAcpt, &that );
			if ( this < that ) OptionValueSet( optExtActAcpt, &this );
			break;
			
		case optExtActAcpt:
		
			OptionValueGet( optExtActSeed, &that );
			OptionValueGet( optExtActAcpt, &this );
			if ( that < this ) OptionValueSet( optExtActSeed, &this );
			break;
			
		case optExtRefSeed:
		
			OptionValueGet( optExtRefSeed, &this );
			OptionValueGet( optExtRefAcpt, &that );
			if ( this < that ) OptionValueSet( optExtRefAcpt, &this );
			break;
			
		case optExtRefAcpt:
		
			OptionValueGet( optExtRefSeed, &that );
			OptionValueGet( optExtRefAcpt, &this );
			if ( that < this ) OptionValueSet( optExtRefSeed, &this );
			break;
			
		default:
			break;
	}
}			
		
/****************************************************************************/
/*					Object size slider										*/
/****************************************************************************/

/*
*	Converts a string to a user value
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	SizeSV( OPTION *opt, char *str, void *val )
{
	sscanf( str, "%d", val );
}

/*
*	Converts a user value to a string
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	SizeVS( OPTION *opt, char *str, void *val )
{
int		d;
	
	d = *((int *) val);
	if ( d < SIZE_MIN ) d = SIZE_MIN;
	if ( d > SIZE_MAX ) d = SIZE_MAX;
	sprintf( str, "% 3d", d );
}

/*
*	Converts a string into a position value
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	double	SizeSP( OPTION *opt, char *str )
{
int		val;

	SizeSV( opt, str, &val );
	if ( val < SIZE_MIN ) val = SIZE_MIN;
	if ( val > SIZE_MAX ) val = SIZE_MAX;
	return( (double) ( val - SIZE_MIN ) / ( SIZE_MAX - SIZE_MIN ) );
}

/*
*	Converts a position value to a string
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	SizePS( OPTION *opt, char *str, double val )
{
	val *= SIZE_MAX - SIZE_MIN;
	val  = (int)( val / SIZE_RES + 0.5 ) * SIZE_RES + SIZE_MIN;
	sprintf( str, "% 3d", (int) val );
}

