/****************************************************************************/
/* Copyright 2004,  Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	Accessing option descriptors and values									*/
/*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~									*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zoltn Kcsi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <glib.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>

#include <types.h>
#include <options.h>
#include <fatal.h>
#include <types.h>
#include <styles.h>
#include <gui.h>
#include <gtkwrappers.h>
#include "option_types.h"

/****************************************************************************/
/*							Prototypes										*/
/****************************************************************************/

static	GtkWidget	*OptionsBuildBox( OPTION **ptr );
static	void		OptionToggleCreate( OPTION *ptr, GtkWidget *box, int wid );
static	void		OptionMenuCreate( OPTION *ptr, GtkWidget *box, int wid );
static	void		OptionSliderCreate( OPTION *ptr, GtkWidget *box, int wid );
static	void		OptionsPopulateBox( OPTION **ptr, GtkWidget *box, int hor );
static	void		OptionToggle( GtkWidget *butt, OPTION *p );
static	void		OptionMenu( GtkWidget *item, OPTION *p );
static	void		OptionSlider( GtkAdjustment *adj, OPTION *p );
static	void		OptionSpinner( GtkAdjustment *adj, OPTION *p );

static	void	OptionColourCreate( OPTION *ptr, GtkWidget *box, int wid );
static	void	OptionExpose( GtkWidget *wid, GdkEventExpose *ev, OPTION *op );
static	void	OptionColour( GtkWidget *butt, OPTION *p );
static	void	OptionDestroy( GtkWidget *dia );
static	void	OptionNewcolour( GtkWidget *dia, OPTION *opt );
static	void	OptionSpinnerCreate( OPTION *ptr, GtkWidget *box, int wid );

/****************************************************************************/
/*							Interface functions								*/
/****************************************************************************/

/*
*	Builds the GUI from the array pointed by p
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Returns the GTK pointer to the toplevel widget.
*/

GtkWidget	*OptionsBuildGui( OPTION *p )
{
GtkWidget	*box;

	box = GuiVbox( FALSE, 0 );
	GuiLabel( "Options", big_style, box, FALSE, TRUE );
	while ( p->type != otiVbox && p->type != otiHbox ) p++;
	GuiBoxAdd( OptionsBuildBox( &p ), box, FALSE, TRUE );
	return( GuiFrame( NULL, NULL, box, GTK_SHADOW_OUT ) );
}

/****************************************************************************/
/*		Functions to create the GUI representation of the options			*/
/****************************************************************************/

/*
*	Creates a packing box
*	~~~~~~~~~~~~~~~~~~~~~
*/

static	GtkWidget	*OptionsBuildBox( OPTION **ptr )
{
GtkWidget	*box;
int			hor;

	if ( (*ptr)++->type == otiHbox ) {
		
		box = GuiHbox( FALSE, 5 );
		hor = TRUE;
	}
	else {
	
		box = GuiVbox( FALSE, 5 );
		hor = FALSE;
	}
	
	OptionsPopulateBox( ptr, box, hor );
	return( box );
}

/*
*	Populates a packing box
*	~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	OptionsPopulateBox( OPTION **ptr, GtkWidget *box, int hor )
{
	for ( ;; (*ptr)++ ) {
		
		switch ( (*ptr)->type ) {
	
			case otiHbox:
			case otiVbox:
		
				GuiBoxAdd( OptionsBuildBox( ptr ), box, hor, TRUE );
				(*ptr)--;
				continue;
		
			case otiEbox:
		
				(*ptr)++;
				return;
		
			case otiSepl:
		
				if ( hor ) 
				
					GuiVline( box );
				else
					GuiHline( box );
				
				break;
			
			case otiLabl:
		
				GuiLabel( (*ptr)->gui, med_style, box, hor, TRUE );
				break;
			
			case otiHead:
		
				GuiLabel( (*ptr)->gui, big_style, box, hor, TRUE );
				break;
			
			case otiFlag:
		
				OptionToggleCreate( *ptr, box, hor );
				break;
			
			case otiSlid:
				
				OptionSliderCreate( *ptr, box, hor );
				break;
				
			case otiSpin:
				
				OptionSpinnerCreate( *ptr, box, hor );
				break;
				
			case otiList:
			
				OptionMenuCreate( *ptr, box, hor );
				break;
				
			case otiCsel:
			case otiCset:
			
				OptionColourCreate( *ptr, box, hor );
				break;
				
			default:
				break;
		}
	}
}

/*
*	Creates a toggle button
*	~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	OptionToggleCreate( OPTION *ptr, GtkWidget *box, int wid )
{
int		val;

	ptr->desc = GuiToggle( ptr->gui, OptionToggle, ptr, box, wid, TRUE );
	OptionValueGet( ptr->optid, &val );
	OptionValueSet( ptr->optid, &val );
}

/*
*	Creates menu
*	~~~~~~~~~~~~
*	A menu contains a frame, that is labelled by the name of the menu
*	and contains a drop-down menu button.
*	The button is populated with the menu options.
*/

static	void	OptionMenuCreate( OPTION *ptr, GtkWidget *box, int wid )
{
MENU		*mp;
ITEM		*ip;
int			val;

	mp		  = ptr->desc;
	mp->group = NULL;
	mp->frame = GuiMenu( ptr->gui, &mp->menu, &mp->list, box, wid, TRUE );
	
	for ( ip = mp->items ; ip->cid ; ip++ ) {
			
		ip->wid = GuiMenuItem( ip->cid->gui, mp->list, &mp->group, 
							   OptionMenu, ptr );
	}
	
	OptionValueGet( ptr->optid, &val );
	OptionValueSet( ptr->optid, &val );
}

/*
*	Creates a slider
*	~~~~~~~~~~~~~~~~
*	A slider has a frame, named as the option's GUI name indicates.
*	Within the frame there is a horizontal packing box, that contains
*	a GTK slider and a frame. The slider's value is not displayed by GTK, 
*	but by a label that is put into the frame beside the slider.
*	The conversion between the slider position and the string for the label
*	is done by user functions.
*/

static	void	OptionSliderCreate( OPTION *ptr, GtkWidget *box, int wid )
{
SLIDER		*slid;
double		pos;
	
	slid  = (SLIDER *) ptr->desc;
	slid->slide = GuiSlider( ptr->gui, &slid->adjus, &slid->label,
							 OptionSlider, ptr, box, FALSE, FALSE );
	pos = (*slid->str_pos)( ptr, OptionValue( ptr->optid ) );
	gtk_adjustment_set_value( GTK_ADJUSTMENT( slid->adjus ), pos );
	
	// We also have to force an adjustment because if the adjustment was
	// 0.5 the adjustment did not call the evnet handler...
	
	OptionSlider( (GtkAdjustment *) slid->adjus, ptr );
}

/*
*	Creates a slider
*	~~~~~~~~~~~~~~~~
*	A slider has a frame, named as the option's GUI name indicates.
*	Within the frame there is a horizontal packing box, that contains
*	a GTK slider and a frame. The slider's value is not displayed by GTK, 
*	but by a label that is put into the frame beside the slider.
*	The conversion between the slider position and the string for the label
*	is done by user functions.
*/

static	void	OptionSpinnerCreate( OPTION *ptr, GtkWidget *box, int wid )
{
SPINNER		*spin;
int			pos;
	
	spin  = (SPINNER *) ptr->desc;
	GuiSpinbox( ptr->gui, spin->min, spin->max, 
				&spin->spbox, &spin->adjus,
				OptionSpinner, ptr, box, FALSE, FALSE );
	
	sscanf( OptionValue( ptr->optid ), "%d", &pos );
	gtk_adjustment_set_value( GTK_ADJUSTMENT( spin->adjus ), pos );
	
	// We also have to force an adjustment because if the adjustment was
	// 0.5 the adjustment did not call the evnet handler...
	
	OptionSpinner( (GtkAdjustment *) spin->adjus, ptr );
}

/*
*	Creates a colour selection option item
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	It is a fixed size frame that shows the
*	colour and a button to change it, All in a frame
*/

static	void	OptionColourCreate( OPTION *ptr, GtkWidget *box, int wid )
{
GtkWidget		*drw;
GtkWidget		*hor;
GtkWidget		*fra;

	// Create the drawing area and set up a signal handler
	
	drw = gtk_drawing_area_new();
	gtk_widget_show( drw );
	gtk_signal_connect( GTK_OBJECT( drw ), "expose_event",
						GTK_SIGNAL_FUNC( OptionExpose ), ptr );
	
	// Create a frame and put the drawable into it
	
	fra = GuiFrame( NULL, NULL, drw, GTK_SHADOW_IN );
	
	// Create a horizontal box that will hold the frame and the
	// select button
	
	hor = GuiHbox( TRUE, 5 );
	
	// Put the frame in the box
	
	GuiBoxAdd( fra, hor, TRUE, TRUE );
	
	// Create the button that will fire up the colour selection
	
	((CSEL *) ptr->desc)->button = 
			GuiButton( "Change", OptionColour, ptr, hor, TRUE, TRUE );
	
	// Create a frame surrounding the whole lot
	
	fra = GuiFrame( ptr->gui, NULL, hor, GTK_SHADOW_ETCHED_OUT );
	gtk_widget_set_usize( fra, CPLX_XSIZE, CPLX_YSIZE );
	
	// Store the drawable in the colour selection descriptor
	
	((CSEL *) ptr->desc)->canvas = drw;
	
	// Place the enclosing frame into the box provided
	
	GuiBoxAdd( fra, box, wid, TRUE );
}

/****************************************************************************/
/*							Signal handlers									*/
/****************************************************************************/

/*
*	A check button changed state
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	NOTE: GTK sends the parameters in reverse order!
*/

static	void	OptionToggle( GtkWidget *butt, OPTION *p )
{
	if ( ! gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( butt ) ) )
	
		strcpy( OptionValue( p->optid ), "no" );
	else
		strcpy( OptionValue( p->optid ), "yes" );
}

/*
*	A menu option was toggled
*	~~~~~~~~~~~~~~~~~~~~~~~~~
*	NOTE: GTK sends the parameters in reverse order!
*/

static	void	OptionMenu( GtkWidget *item, OPTION *p )
{
MENU	*m;
ITEM	*i;

	if ( ! GTK_CHECK_MENU_ITEM( item )->active ) return;

	m = p->desc;
	
	for ( i = m->items ; i->cid ; i++ ) {
	
		if ( i->wid == item ) {
		
			strcpy( OptionValue( p->optid ), i->cid->cli );
			return;
		}
	}
	
	FATAL( "Invalid menu option widget ID for option %s", p->gui );
}

/*
*	Slider state changed
*	~~~~~~~~~~~~~~~~~~~~
*/

static	void	OptionSlider( GtkAdjustment *adj, OPTION *p )
{
char	buff[ OPT_STRLEN+1 ];
SLIDER	*s;

	s = p->desc;
	
	(*s->pos_str)( p, buff, adj->value );
	gtk_label_set_text( GTK_LABEL( s->label ), buff );
	strcpy( OptionValue( p->optid ), buff );
	if ( s->useract ) (*s->useract)( p );
}

/*
*	Expose event handler for colour selections
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	OptionExpose( GtkWidget *wid, GdkEventExpose *ev, OPTION *op )
{
int		col;
char	rgb[ 3 ];
int		i;

	// Obtain the colour
	
	col = ((CSEL *) op->desc)->colour;
	
	rgb[ 0 ] = ( col >> 16 ) & 255;
	rgb[ 1 ] = ( col >>  8 ) & 255;
	rgb[ 2 ] = ( col >>  0 ) & 255;
	
	// Paint the box column by column. It is slow but the 
	// box is small so it is not really a problem.
	
	for ( i = ev->area.x ; i < ev->area.x + ev->area.width ; i++ ) {
	
		gdk_draw_rgb_image( wid->window, wid->style->fg_gc[ GTK_STATE_NORMAL ],
							i, ev->area.y, 1, ev->area.height,
							GDK_RGB_DITHER_MAX, rgb, 0 );
	}					
}

/*
*	Colour change button click event handler
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	OptionColour( GtkWidget *butt, OPTION *op )
{
GtkWidget	*dia;
gdouble		rgba[ 4 ];
int			col;

	// Get current colour
	
	sscanf( OptionValue( op->optid ), "%x", &col );
	
	rgba[ 0 ] = ( ( col >> 16 ) & 255 ) / 255.0;
	rgba[ 1 ] = ( ( col >>  8 ) & 255 ) / 255.0;
	rgba[ 2 ] = ( ( col >>  0 ) & 255 ) / 255.0;
	rgba[ 3 ] = 0;
	
	// We have to fire up a colour selection dialog box
	
	dia = gtk_color_selection_dialog_new( op->gui );

	// Since we will make it grab the focus, we have to set up handlers
	// for the event it gets closed
	
	gtk_signal_connect( 
			GTK_OBJECT( dia ), 
			"destroy", GTK_SIGNAL_FUNC( OptionDestroy ), NULL );
	
	gtk_signal_connect_object( 
			GTK_OBJECT(	GTK_COLOR_SELECTION_DIALOG( dia )->cancel_button ), 
			"clicked", GTK_SIGNAL_FUNC( OptionDestroy ), (gpointer) dia );
	
	// We don't offer help so we destroy the help button
	
	gtk_widget_destroy( GTK_COLOR_SELECTION_DIALOG( dia )->help_button );
	
	// Set up the event handler for the OK button. 
	// Unfortunately the OK handler needs both the option descriptor and
	// the dialog box's colour selection widget's reference, so we use
	// the option's 'desc' field to temporarily hold the reference
	
	((CSEL *) op->desc)->dialog = dia;
	
	gtk_signal_connect( 
			GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(dia)->ok_button), 
			"clicked", GTK_SIGNAL_FUNC( OptionNewcolour ), op );
	
	// Set the default colour to the current one
	
	gtk_color_selection_set_color( 
			GTK_COLOR_SELECTION( GTK_COLOR_SELECTION_DIALOG( dia )->colorsel ),
			rgba );
	
	// Grab the focus and let it go
	
	gtk_widget_show( dia );
	gtk_grab_add( dia );
}

/*
*	Event handler for destruction of a colour selection dialog
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	OptionDestroy( GtkWidget *dia )
{
	gtk_grab_remove( dia );
	gtk_widget_destroy( dia );
}

/*
*	Event handler for the user accepting a new colour
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	OptionNewcolour( GtkWidget *but, OPTION *opt )
{
gdouble		rgba[ 4 ];
int			col;
GtkWidget	*dia;
GtkWidget	*drw;

	dia = ((CSEL *) opt->desc)->dialog;
	drw = ((CSEL *) opt->desc)->canvas;
	
	// Get the current colours
	
	gtk_color_selection_get_color( 
			GTK_COLOR_SELECTION( GTK_COLOR_SELECTION_DIALOG( dia )->colorsel ),
			rgba );
	
	// Make it an int and store in the option value
	
	col = 0;
	col |= (int)( rgba[ 0 ] * 255 + .5 ) << 16;
	col |= (int)( rgba[ 1 ] * 255 + .5 ) <<  8;
	col |= (int)( rgba[ 2 ] * 255 + .5 ) <<  0;

	((CSEL *) opt->desc)->colour = col;
	sprintf( OptionValue( opt->optid ), "%06x", col );
	
	// If we are not a job parameter but a general colour, inform
	// system about change
	
	if ( opt->type == otiCset ) GuiColourChange();
	
	// Hide and show the drawable, so that we force an expose event
	
	gtk_widget_hide( drw );
	gtk_widget_show( drw );
	
	// Destroy the dialog box
	
	gtk_widget_destroy( dia );
}

/*
*	Spinbox state changed
*	~~~~~~~~~~~~~~~~~~~~~
*/

static	void	OptionSpinner( GtkAdjustment *adj, OPTION *p )
{
char	buff[ OPT_STRLEN+1 ];
SPINNER	*s;

	s = p->desc;
	(*s->func)( p, (int) adj->value );
}
