/****************************************************************************/
/* Copyright 2004, © Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	Distance mapping options												*/
/*	~~~~~~~~~~~~~~~~~~~~~~~~												*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zoltán Kócsi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <math.h>

#include <types.h>
#include <options.h>
#include <fatal.h>
#include "option_types.h"

/****************************************************************************/
/*							Local definitions								*/
/****************************************************************************/

#define	TESTS_MIN	100
#define	TESTS_MAX	1000
#define	TESTS_RES	25

#define	HISTS_MIN	0
#define	HISTS_MAX	25
#define	HISTS_RES	1

/****************************************************************************/
/*								Prototypes									*/
/****************************************************************************/

static	void	TestsSV( OPTION *opt, char *str, void *val );
static	void	TestsVS( OPTION *opt, char *str, void *val );
static	double	TestsSP( OPTION *opt, char *str );
static	void	TestsPS( OPTION *opt, char *str, double val );
static	void	HistsVS( OPTION *opt, char *str, void *val );
static	double	HistsSP( OPTION *opt, char *str );
static	void	HistsPS( OPTION *opt, char *str, double val );

/****************************************************************************/
/*					Data describing the option items						*/
/****************************************************************************/

static	CHOICE	tri		= { "Triangles",	 	 "tria",	OPT_SELF_TRIA	};
static	CHOICE	spa		= { "Spanning tree", 	 "tree",	OPT_SELF_TREE	};
static	CHOICE	clo		= { "Closest neighbour", "clos",	OPT_SELF_NEIG	};
static	CHOICE	rat		= { "Every segment",	 "rats",	OPT_SELF_RATS	};

static	ITEM	self[]  = { 
	
	DEF_ITEM( tri ), 
	DEF_ITEM( spa ), 
	DEF_ITEM( clo ), 
	DEF_ITEM( rat ), 
	DEF_LAST	
};

	MENU	stat_dis = DEF_MENU( self );
	SLIDER	statsnum = DEF_SLID( TestsSV, TestsVS, TestsSP, TestsPS, NULL );
	SLIDER	stathist = DEF_SLID( TestsSV, HistsVS, HistsSP, HistsPS, NULL );

/****************************************************************************/
/*			Number of statisctical samples slider functions					*/
/****************************************************************************/

/*
*	Converts a string to a user value
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	TestsSV( OPTION *opt, char *str, void *val )
{
	sscanf( str, "%d", val );
}

/*
*	Converts a user value to a string
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	TestsVS( OPTION *opt, char *str, void *val )
{
int		d;
	
	d = *((int *) val);
	if ( d < TESTS_MIN ) d = TESTS_MIN;
	if ( d > TESTS_MAX ) d = TESTS_MAX;
	sprintf( str, "% 3d", d );
}

/*
*	Converts a string into a position value
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	double	TestsSP( OPTION *opt, char *str )
{
int		val;

	TestsSV( opt, str, &val );
	if ( val < TESTS_MIN ) val = TESTS_MIN;
	if ( val > TESTS_MAX ) val = TESTS_MAX;
	return( (double) ( val - TESTS_MIN ) / ( TESTS_MAX - TESTS_MIN ) );
}

/*
*	Converts a position value to a string
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	TestsPS( OPTION *opt, char *str, double val )
{
	val *= TESTS_MAX - TESTS_MIN;
	val  = (int)( val / TESTS_RES + 0.5 ) * TESTS_RES + TESTS_MIN;
	sprintf( str, "% 3d", (int) val );
}

/*
*	Converts a user value to a string
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	HistsVS( OPTION *opt, char *str, void *val )
{
int		d;
	
	d = *((int *) val);
	if ( d < HISTS_MIN ) d = HISTS_MIN;
	if ( d > HISTS_MAX ) d = HISTS_MAX;
	sprintf( str, "% 3d", d );
}

/*
*	Converts a string into a position value
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	double	HistsSP( OPTION *opt, char *str )
{
int		val;

	TestsSV( opt, str, &val );
	if ( val < HISTS_MIN ) val = HISTS_MIN;
	if ( val > HISTS_MAX ) val = HISTS_MAX;
	return( (double) ( val - HISTS_MIN ) / ( HISTS_MAX - HISTS_MIN ) );
}

/*
*	Converts a position value to a string
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	HistsPS( OPTION *opt, char *str, double val )
{
	val *= HISTS_MAX - HISTS_MIN;
	val  = (int)( val / HISTS_RES + 0.5 ) * HISTS_RES + HISTS_MIN;
	sprintf( str, "% 3d", (int) val );
}
