/****************************************************************************/
/* Copyright 2004, � Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	Access to objects														*/
/*	~~~~~~~~~~~~~~~~~														*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zolt�n K�csi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#include <types.h>
#include <dlinklist.h>
#include <objects.h>
#include <distance.h>
#include <memory.h>
#include <image.h>
#include <progress.h>
#include <options.h>
#include "process_intern.h"

/*
*	Destroy an object
*	~~~~~~~~~~~~~~~~~
*
*	Parameters
*		obj		Reference to the object
*
*	Return value
*		-
*/

void	ObjectDestroy( OBJECT obj )
{
PIXEL	p;

	DtreeDestroy( obj->atree );
	DtreeDestroy( obj->ptree );
	while ( ( p = DlinkRemoveFirst( &obj->pixels ) ) ) Free( p );
	while ( ( p = DlinkRemoveFirst( &obj->border ) ) ) Free( p );
	Free( obj );
}

/*
*	Destroy an object list
*	~~~~~~~~~~~~~~~~~~~~~~
*
*	Parameters
*		list	Pointer to a DLINK containing the objects
*
*	Return value
*		-
*/

void	ObjectDestroyList( DLINK *list )
{
OBJECT	obj;

	while ( ( obj = DlinkRemoveFirst( list ) ) ) ObjectDestroy( obj );
}

/*
*	Get the first pixel of an object
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
*	Parameters
*
*		obj		Reference to the object
*		border	TRUE if border pixels are needed, FALSE if area ones
*		y		Pointer to an int where the pixel's Y coordinate should be put
*		x		Pointer to an int where the pixel's X coordinate should be put
*
*	Return value
*
*		Reference to the pixel or NULL if there are no pixels
*/

PIXEL	ObjectFirstPixel( OBJECT obj, int border, int *y, int *x )
{
PIXEL	p;

	if ( ( p = DlinkFirst( border ? &obj->border : &obj->pixels ) ) ) {
	
		*y = p->y + obj->bbox.tly;
		*x = p->x + obj->bbox.tlx;
	}
	
	return( p );
}

/*
*	Get the next pixel of an object
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
*	Parameters
*
*		pix		Reference to the previous pixel
*		y		Pointer to an int where the pixel's Y coordinate should be put
*		x		Pointer to an int where the pixel's X coordinate should be put
*
*	Return value
*
*		Reference to the pixel or NULL if the previous was the last pixel
*/

PIXEL	ObjectNextPixel( OBJECT obj, PIXEL pix, int *y, int *x )
{
	if ( ( pix = DlinkNext( pix ) ) ) {
	
		*y = pix->y + obj->bbox.tly;
		*x = pix->x + obj->bbox.tlx;
	}
	
	return( pix );
}

/*
*	Get the coordinates of a pixel
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
*	Parameters
*
*		pix		Reference to the pixel
*		y		Pointer to an int where the pixel's Y coordinate should be put
*		x		Pointer to an int where the pixel's X coordinate should be put
*
*	Return value
*		-
*/

void	ObjectPixelCoords( PIXEL pix, int *y, int *x )
{
	*y = pix->y;
	*x = pix->x;
}

/*
*	Returns the area of an object
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
*	Parameters
*
*		obj		Reference to the object
*
*	Return value
*		The area in pixels
*/

int		ObjectArea( OBJECT obj )
{
	return( obj->area );
}

/*
*	Returns the weighted centre of an object
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	ObjectCentre( OBJECT obj, double *x, double *y )
{
	*x = obj->xc;
	*y = obj->yc;
}

