/****************************************************************************/
/* Copyright 2004,  Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	Calculating distances													*/
/*	~~~~~~~~~~~~~~~~~~~~~													*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zoltn Kcsi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#include <stdio.h>
#include <math.h>

#include <types.h>
#include <dlinklist.h>
#include <objects.h>
#include <memory.h>
#include "process_intern.h"
#include "distfunc.h"

/****************************************************************************/
/*						Local types and constants							*/
/****************************************************************************/

typedef	struct	{

	DLINK	link;						/* List element linker				*/
	int		min;						/* Distance minimum					*/
	SDIV	*act;						/* Spatial node, active object		*/
	SDIV	*ref;						/* Spatial node, reference object	*/
	
} QITEM;								/* Distance calculation queue item	*/

/****************************************************************************/
/*						Prototypes											*/
/****************************************************************************/

static	int		DistMinResolve( DLINK *old, RECT *offs, int maxd );
static	int		DistMinPrepare( DLINK *old, RECT *offs, int maxd );

static	void	DistOvlPrepare( DLINK *old, RECT *offs );
static	int		DistOvlResolve( DLINK *old, RECT *offs, DLINK *pix );
							  
/****************************************************************************/
/*						Local data											*/
/****************************************************************************/


/*
*	Precalculated tables for single pixel resolution distance calculations
*/

static const int	mod_single_dx[]  = {  0, -1,  0, -1, +1,  0, +1,  0, 
										  0, -1,  0, -1, +1,  0, +1,  0 };
									  
static const int	mod_single_dy[]  = {  0,  0, -1, -1,  0,  0, -1, -1,
										 +1, +1,  0,  0, +1, +1,  0,  0 };
								 
static const int	mod_single_aax[] = {  0,  0,  0,  0, +1, +1, +1, +1,
										  0,  0,  0,  0, +1, +1, +1, +1 };

static const int	mod_single_aay[] = {  0,  0,  0,  0,  0,  0,  0,  0,
										 +1, +1, +1, +1, +1, +1, +1, +1 };

static const int	mod_single_arx[] = {  0, +1,  0, +1,  0, +1,  0, +1,
										  0, +1,  0, +1,  0, +1,  0, +1 };

static const int	mod_single_ary[] = {  0,  0, +1, +1,  0,  0, +1, +1,
										  0,  0, +1, +1,  0,  0, +1, +1 };
								   
static const int	index_single[] = {
	
	3600,   -1,   -1,   -1,   -1, 1350,   -1,   -1, 3150,
	  -1, 1575,   -1,  450,    0,  225, 2250, 1800, 2025,
	  -1, 3375,   -1, 1125,  675,  900, 2925, 2475, 2700
};

static const ubyte	resol_single[] = {

	 0,  4,  0,  8,  0,  8,  0, 12,  0,  4,  0,  8,  0,  8,  0, 
	 1,  5,  1,  9,  1,  9,  1, 13,  1,  5,  1,  9,  1,  9,  1, 
	 1,  5,  1,  9,  1,  9,  1, 13,  1,  5,  1,  9,  1,  9,  1, 
	 2,  6,  2, 10,  2, 10,  2, 14,  2,  6,  2, 10,  2, 10,  2, 
	 2,  6,  2, 10,  2, 10,  2, 14,  2,  6,  2, 10,  2, 10,  2, 
	 1,  5,  1,  9,  1,  9,  1, 13,  1,  5,  1,  9,  1,  9,  1, 
	 1,  5,  1,  9,  1,  9,  1, 13,  1,  5,  1,  9,  1,  9,  1, 
	 3,  7,  3, 11,  3, 11,  3, 15,  3,  7,  3, 11,  3, 11,  3, 
	 3,  7,  3, 11,  3, 11,  3, 15,  3,  7,  3, 11,  3, 11,  3, 
	 3,  7,  3, 11,  3, 11,  3, 15,  3,  7,  3, 11,  3, 11,  3, 
	 3,  7,  3, 11,  3, 11,  3, 15,  3,  7,  3, 11,  3, 11,  3, 
	 3,  7,  3, 11,  3, 11,  3, 15,  3,  7,  3, 11,  3, 11,  3, 
	 3,  7,  3, 11,  3, 11,  3, 15,  3,  7,  3, 11,  3, 11,  3, 
	 3,  7,  3, 11,  3, 11,  3, 15,  3,  7,  3, 11,  3, 11,  3, 
	 3,  7,  3, 11,  3, 11,  3, 15,  3,  7,  3, 11,  3, 11,  3, 
	
	 0,  4,  0,  8,  0,  4,  0, 12,  0,  4,  0,  8,  0,  4,  0, 
	 1,  5,  1,  9,  1,  5,  1, 13,  1,  5,  1,  9,  1,  5,  1, 
	 1,  5,  1,  9,  1,  5,  1, 13,  1,  5,  1,  9,  1,  5,  1, 
	 2,  6,  2, 10,  2,  6,  2, 14,  2,  6,  2, 10,  2,  6,  2, 
	 2,  6,  2, 10,  2,  6,  2, 14,  2,  6,  2, 10,  2,  6,  2, 
	 2,  6,  2, 10,  2,  6,  2, 14,  2,  6,  2, 10,  2,  6,  2, 
	 2,  6,  2, 10,  2,  6,  2, 14,  2,  6,  2, 10,  2,  6,  2, 
	 3,  7,  3, 11,  3,  7,  3, 15,  3,  7,  3, 11,  3,  7,  3, 
	 3,  7,  3, 11,  3,  7,  3, 15,  3,  7,  3, 11,  3,  7,  3, 
	 3,  7,  3, 11,  3,  7,  3, 15,  3,  7,  3, 11,  3,  7,  3, 
	 3,  7,  3, 11,  3,  7,  3, 15,  3,  7,  3, 11,  3,  7,  3, 
	 3,  7,  3, 11,  3,  7,  3, 15,  3,  7,  3, 11,  3,  7,  3, 
	 3,  7,  3, 11,  3,  7,  3, 15,  3,  7,  3, 11,  3,  7,  3, 
	 3,  7,  3, 11,  3,  7,  3, 15,  3,  7,  3, 11,  3,  7,  3, 
	 3,  7,  3, 11,  3,  7,  3, 15,  3,  7,  3, 11,  3,  7,  3, 
	
	 0,  4,  0,  8,  0,  4,  0, 12,  0,  4,  0,  8,  0,  4,  0, 
	 1,  5,  1,  9,  1,  5,  1, 13,  1,  5,  1,  9,  1,  5,  1, 
	 1,  5,  1,  9,  1,  5,  1, 13,  1,  5,  1,  9,  1,  5,  1, 
	 2,  6,  2, 10,  2, 10,  2, 14,  2,  6,  2, 10,  2, 10,  2, 
	 2,  6,  2, 10,  2, 10,  2, 14,  2,  6,  2, 10,  2, 10,  2, 
	 1,  5,  1, 10,  1,  5,  1, 13,  1,  5,  1, 10,  1,  5,  1, 
	 1,  5,  1, 10,  1,  5,  1, 13,  1,  5,  1, 10,  1,  5,  1, 
	 3,  7,  3, 11,  3,  7,  3, 15,  3,  7,  3, 11,  3,  7,  3, 
	 3,  7,  3, 11,  3,  7,  3, 15,  3,  7,  3, 11,  3,  7,  3, 
	 3,  7,  3, 11,  3,  7,  3, 15,  3,  7,  3, 11,  3,  7,  3, 
	 3,  7,  3, 11,  3,  7,  3, 15,  3,  7,  3, 11,  3,  7,  3, 
	 3,  7,  3, 11,  3,  7,  3, 15,  3,  7,  3, 11,  3,  7,  3, 
	 3,  7,  3, 11,  3,  7,  3, 15,  3,  7,  3, 11,  3,  7,  3, 
	 3,  7,  3, 11,  3,  7,  3, 15,  3,  7,  3, 11,  3,  7,  3, 
	 3,  7,  3, 11,  3,  7,  3, 15,  3,  7,  3, 11,  3,  7,  3, 
	
	 0,  4,  4,  8,  0,  4,  4, 12, 12,  4,  4, 12, 12,  4,  4, 
	 1,  5,  5,  9,  1,  5,  5, 13, 13,  5,  5, 13, 13,  5,  5, 
	 0,  4,  4,  8,  0,  4,  4, 12, 12,  4,  4, 12, 12,  4,  4, 
	 2,  6,  6, 10,  2,  6,  6, 14, 14,  6,  6, 14, 14,  6,  6, 
	 2,  6,  6, 10,  2,  6,  6, 14, 14,  6,  6, 14, 14,  6,  6, 
	 2,  6,  6, 10,  2,  6,  6, 14, 14,  6,  6, 14, 14,  6,  6, 
	 2,  6,  6, 10,  2,  6,  6, 14, 14,  6,  6, 14, 14,  6,  6, 
	 3,  7,  7, 11,  3,  7,  7, 15, 15,  7,  7, 15, 15,  7,  7, 
	 0,  4,  4,  8,  0,  4,  4, 12, 12,  4,  4, 12, 12,  4,  4, 
	 3,  7,  7, 11,  3,  7,  7, 15, 15,  7,  7, 15, 15,  7,  7, 
	 0,  4,  4,  8,  0,  4,  4, 12, 12,  4,  4, 12, 12,  4,  4, 
	 2,  6,  6, 10,  2,  6,  6, 14, 14,  6,  6, 14, 14,  6,  6, 
	 2,  6,  6, 10,  2,  6,  6, 14, 14,  6,  6, 14, 14,  6,  6, 
	 2,  6,  6, 10,  2,  6,  6, 14, 14,  6,  6, 14, 14,  6,  6, 
	 2,  6,  6, 10,  2,  6,  6, 14, 14,  6,  6, 14, 14,  6,  6, 
	
	 0,  4,  4,  8,  0,  4,  4, 12,  0,  4,  4, 12,  0,  4,  4, 
	 1,  5,  5,  9,  1,  5,  5, 13,  1,  5,  5, 13,  1,  5,  5, 
	 0,  4,  4,  8,  0,  4,  4, 12,  0,  4,  4, 12,  0,  4,  4, 
	 2,  6,  6, 10,  2,  6,  6, 14,  2,  6,  6, 14,  2,  6,  6, 
	 2,  6,  6, 10,  2,  6,  6, 14,  2,  6,  6, 14,  2,  6,  6, 
	 2,  6,  6, 10,  2,  6,  6, 14,  2,  6,  6, 14,  2,  6,  6, 
	 2,  6,  6, 10,  2,  6,  6, 14,  2,  6,  6, 14,  2,  6,  6, 
	 3,  7,  7, 11,  3,  7,  7, 15,  3,  7,  7, 15,  3,  7,  7, 
	 3,  7,  7, 11,  3,  7,  7, 15,  3,  7,  7, 15,  3,  7,  7, 
	 3,  7,  7, 11,  3,  7,  7, 15,  3,  7,  7, 15,  3,  7,  7, 
	 3,  7,  7, 11,  3,  7,  7, 15,  3,  7,  7, 15,  3,  7,  7, 
	 2,  6,  6, 10,  2,  6,  6, 14,  2,  6,  6, 14,  2,  6,  6, 
	 2,  6,  6, 10,  2,  6,  6, 14,  2,  6,  6, 14,  2,  6,  6, 
	 2,  6,  6, 10,  2,  6,  6, 14,  2,  6,  6, 14,  2,  6,  6, 
	 2,  6,  6, 10,  2,  6,  6, 14,  2,  6,  6, 14,  2,  6,  6, 
	
	 0,  4,  4,  8,  0,  4,  4, 12,  0,  4,  4, 12,  0,  4,  4, 
	 1,  5,  5,  9,  1,  5,  5, 13,  1,  5,  5, 13,  1,  5,  5, 
	 0,  4,  4,  8,  0,  4,  4, 12,  0,  4,  4, 12,  0,  4,  4, 
	 2,  6,  6, 10,  2,  6,  6, 14,  2,  6,  6, 14,  2,  6,  6, 
	 2,  6,  6, 10,  2,  6,  6, 14,  2,  6,  6, 14,  2,  6,  6, 
	 2,  6,  6, 10,  2,  6,  6, 14,  2,  6,  6, 14,  2,  6,  6, 
	 2,  6,  6, 10,  2,  6,  6, 14,  2,  6,  6, 14,  2,  6,  6, 
	 3,  7,  7, 11,  3,  7,  7, 15, 15,  7,  7, 15, 15,  7,  7, 
	 0,  4,  4,  8,  0,  4,  4, 15,  0,  4,  4, 15,  0,  4,  4, 
	 3,  7,  7, 11,  3,  7,  7, 15, 15,  7,  7, 15, 15,  7,  7, 
	 0,  4,  4,  8,  0,  4,  4, 15,  0,  4,  4, 15,  0,  4,  4, 
	 2,  6,  6, 10,  2,  6,  6, 14,  2,  6,  6, 14,  2,  6,  6, 
	 2,  6,  6, 10,  2,  6,  6, 14,  2,  6,  6, 14,  2,  6,  6, 
	 2,  6,  6, 10,  2,  6,  6, 14,  2,  6,  6, 14,  2,  6,  6, 
	 2,  6,  6, 10,  2,  6,  6, 14,  2,  6,  6, 14,  2,  6,  6, 
	
	 0,  4,  0,  8,  0,  4,  0, 12,  0,  4,  0,  8,  0,  4,  0, 
	 1,  5,  5,  9,  1,  5,  5, 13,  1,  5,  5, 13,  1,  5,  5, 
	 0,  5,  0,  8,  0,  5,  0, 13,  0,  5,  0,  8,  0,  5,  0, 
	 2,  6,  2, 10,  2,  6,  2, 14,  2,  6,  2, 10,  2,  6,  2, 
	 2,  6,  2, 10,  2,  6,  2, 14,  2,  6,  2, 10,  2,  6,  2, 
	 2,  6,  2, 10,  2,  6,  2, 14,  2,  6,  2, 10,  2,  6,  2, 
	 2,  6,  2, 10,  2,  6,  2, 14,  2,  6,  2, 10,  2,  6,  2, 
	 3,  7,  7, 11,  3,  7,  7, 15,  3,  7,  7, 15,  3,  7,  7, 
	 3,  7,  7, 11,  3,  7,  7, 15,  3,  7,  7, 15,  3,  7,  7, 
	 3,  7,  7, 11,  3,  7,  7, 15,  3,  7,  7, 15,  3,  7,  7, 
	 3,  7,  7, 11,  3,  7,  7, 15,  3,  7,  7, 15,  3,  7,  7, 
	 2,  7,  2, 10,  2,  7,  2, 15,  2,  7,  2, 10,  2,  7,  2, 
	 2,  7,  2, 10,  2,  7,  2, 15,  2,  7,  2, 10,  2,  7,  2, 
	 2,  7,  2, 10,  2,  7,  2, 15,  2,  7,  2, 10,  2,  7,  2, 
	 2,  7,  2, 10,  2,  7,  2, 15,  2,  7,  2, 10,  2,  7,  2, 
	
	 0,  4,  0,  8,  0,  8,  0, 12,  0,  4,  0,  8,  0,  8,  0, 
	 1,  5,  1,  9,  1,  9,  1, 13,  1,  5,  1,  9,  1,  9,  1, 
	 1,  5,  1,  9,  1,  9,  1, 13,  1,  5,  1,  9,  1,  9,  1, 
	 2,  6,  2, 10, 10, 10, 10, 14,  2, 14,  2, 10, 10, 10, 10, 
	 0,  4,  0, 10,  0, 10,  0, 14,  0,  4,  0, 10,  0, 10,  0, 
	 1,  5,  1,  9,  1,  9,  1, 13,  1,  5,  1,  9,  1,  9,  1, 
	 1,  5,  1,  9,  1,  9,  1, 13,  1,  5,  1,  9,  1,  9,  1, 
	 3,  7,  3, 11, 11, 11, 11, 15,  3, 15,  3, 11, 11, 11, 11, 
	 3,  7,  3, 11, 11, 11, 11, 15,  3, 15,  3, 11, 11, 11, 11, 
	 1,  5,  1, 11,  1, 11,  1, 15,  1,  5,  1, 11,  1, 11,  1, 
	 1,  5,  1, 11,  1, 11,  1, 15,  1,  5,  1, 11,  1, 11,  1, 
	 3,  7,  3, 11, 11, 11, 11, 15,  3, 15,  3, 11, 11, 11, 11, 
	 3,  7,  3, 11, 11, 11, 11, 15,  3, 15,  3, 11, 11, 11, 11, 
	 1,  5,  1, 11,  1, 11,  1, 15,  1,  5,  1, 11,  1, 11,  1, 
	 1,  5,  1, 11,  1, 11,  1, 15,  1,  5,  1, 11,  1, 11,  1, 
	
	 0,  4,  0,  8,  8,  8,  8, 12,  0, 12,  0,  8,  8,  8,  8, 
	 1,  5,  1,  9,  9,  9,  9, 13,  1, 13,  1,  9,  9,  9,  9, 
	 1,  5,  1,  9,  9,  9,  9, 13,  1, 13,  1,  9,  9,  9,  9, 
	 2,  6,  2, 10, 10, 10, 10, 14,  2, 14,  2, 10, 10, 10, 10, 
	 0,  4,  0,  8,  8,  8,  8, 12,  0, 12,  0,  8,  8,  8,  8, 
	 1,  5,  1,  9,  9,  9,  9, 13,  1, 13,  1,  9,  9,  9,  9, 
	 1,  5,  1,  9,  9,  9,  9, 13,  1, 13,  1,  9,  9,  9,  9, 
	 3,  7,  3, 11, 11, 11, 11, 15,  3, 15,  3, 11, 11, 11, 11, 
	 3,  7,  3, 11, 11, 11, 11, 15,  3, 15,  3, 11, 11, 11, 11, 
	 1,  5,  1,  9,  9,  9,  9, 13,  1, 13,  1,  9,  9,  9,  9, 
	 1,  5,  1,  9,  9,  9,  9, 13,  1, 13,  1,  9,  9,  9,  9, 
	 3,  7,  3, 11, 11, 11, 11, 15,  3, 15,  3, 11, 11, 11, 11, 
	 3,  7,  3, 11, 11, 11, 11, 15,  3, 15,  3, 11, 11, 11, 11, 
	 1,  5,  1,  9,  9,  9,  9, 13,  1, 13,  1,  9,  9,  9,  9, 
	 1,  5,  1,  9,  9,  9,  9, 13,  1, 13,  1,  9,  9,  9,  9, 
	
	 0,  4,  0,  8,  8,  8,  8, 12, 12, 12, 12,  8,  8,  8,  8, 
	 1,  5,  1,  9,  9,  9,  9, 13, 13, 13, 13,  9,  9,  9,  9, 
	 1,  5,  1,  9,  9,  9,  9, 13, 13, 13, 13,  9,  9,  9,  9, 
	 2,  6,  2, 10, 10, 10, 10, 14, 14, 14, 14, 10, 10, 10, 10, 
	 0,  4,  0,  8,  8,  8,  8, 12, 12, 12, 12,  8,  8,  8,  8, 
	 1,  5,  1,  9,  9,  9,  9, 13, 13, 13, 13,  9,  9,  9,  9, 
	 1,  5,  1,  9,  9,  9,  9, 13, 13, 13, 13,  9,  9,  9,  9, 
	 3,  7,  3, 11, 11, 11, 11, 15, 15, 15, 15, 11, 11, 11, 11, 
	 0,  4,  0,  8,  8,  8,  8, 12, 12, 12, 12,  8,  8,  8,  8, 
	 1,  5,  1,  9,  9,  9,  9, 13, 13, 13, 13,  9,  9,  9,  9, 
	 1,  5,  1,  9,  9,  9,  9, 13, 13, 13, 13,  9,  9,  9,  9, 
	 3,  7,  3, 11, 11, 11, 11, 15, 15, 15, 15, 11, 11, 11, 11, 
	 0,  4,  0,  8,  8,  8,  8, 12, 12, 12, 12,  8,  8,  8,  8, 
	 1,  5,  1,  9,  9,  9,  9, 13, 13, 13, 13,  9,  9,  9,  9, 
	 1,  5,  1,  9,  9,  9,  9, 13, 13, 13, 13,  9,  9,  9,  9, 
	
	 0,  4,  0,  8,  8,  8,  8, 12,  0, 12,  0,  8,  8,  8,  8, 
	 1,  5,  1,  9,  9,  9,  9, 13,  1, 13,  1,  9,  9,  9,  9, 
	 1,  5,  1,  9,  9,  9,  9, 13,  1, 13,  1,  9,  9,  9,  9, 
	 2,  6,  2, 10, 10, 10, 10, 14,  2, 14,  2, 10, 10, 10, 10, 
	 0,  4,  0,  8,  8,  8,  8, 12,  0, 12,  0,  8,  8,  8,  8, 
	 1,  5,  1,  9,  9,  9,  9, 13,  1, 13,  1,  9,  9,  9,  9, 
	 1,  5,  1,  9,  9,  9,  9, 13,  1, 13,  1,  9,  9,  9,  9, 
	 3,  7,  3, 11, 11, 11, 11, 15, 15, 15, 15, 11, 11, 11, 11, 
	 0,  4,  0,  8,  8,  8,  8, 15,  0, 15,  0,  8,  8,  8,  8, 
	 1,  5,  1,  9,  9,  9,  9, 13,  1, 13,  1,  9,  9,  9,  9, 
	 1,  5,  1,  9,  9,  9,  9, 13,  1, 13,  1,  9,  9,  9,  9, 
	 3,  7,  3, 11, 11, 11, 11, 15, 15, 15, 15, 11, 11, 11, 11, 
	 0,  4,  0,  8,  8,  8,  8, 15,  0, 15,  0,  8,  8,  8,  8, 
	 1,  5,  1,  9,  9,  9,  9, 13,  1, 13,  1,  9,  9,  9,  9, 
	 1,  5,  1,  9,  9,  9,  9, 13,  1, 13,  1,  9,  9,  9,  9, 
	
	 0,  4,  4,  8,  8,  4,  4, 12, 12, 12, 12, 12, 12, 12, 12, 
	 1,  5,  5,  9,  9,  5,  5, 13, 13, 13, 13, 13, 13, 13, 13, 
	 0,  4,  4,  8,  8,  4,  4, 12, 12, 12, 12, 12, 12, 12, 12, 
	 2,  6,  6, 10, 10,  6,  6, 14, 14, 14, 14, 14, 14, 14, 14, 
	 0,  4,  4,  8,  8,  4,  4, 12, 12, 12, 12, 12, 12, 12, 12, 
	 2,  6,  6, 10, 10,  6,  6, 14, 14, 14, 14, 14, 14, 14, 14, 
	 0,  4,  4,  8,  8,  4,  4, 12, 12, 12, 12, 12, 12, 12, 12, 
	 3,  7,  7, 11, 11,  7,  7, 15, 15, 15, 15, 15, 15, 15, 15, 
	 0,  4,  4,  8,  8,  4,  4, 12, 12, 12, 12, 12, 12, 12, 12, 
	 1,  5,  5,  9,  9,  5,  5, 13, 13, 13, 13, 13, 13, 13, 13, 
	 0,  4,  4,  8,  8,  4,  4, 12, 12, 12, 12, 12, 12, 12, 12, 
	 2,  6,  6, 10, 10,  6,  6, 14, 14, 14, 14, 14, 14, 14, 14, 
	 0,  4,  4,  8,  8,  4,  4, 12, 12, 12, 12, 12, 12, 12, 12, 
	 2,  6,  6, 10, 10,  6,  6, 14, 14, 14, 14, 14, 14, 14, 14, 
	 0,  4,  4,  8,  8,  4,  4, 12, 12, 12, 12, 12, 12, 12, 12, 
	
	 0,  4,  4,  8,  8,  8,  8, 12, 12, 12, 12, 12, 12, 12, 12, 
	 1,  5,  5,  9,  9,  9,  9, 13, 13, 13, 13, 13, 13, 13, 13, 
	 0,  4,  4,  8,  8,  8,  8, 12, 12, 12, 12, 12, 12, 12, 12, 
	 2,  6,  6, 10, 10, 10, 10, 14, 14, 14, 14, 14, 14, 14, 14, 
	 0,  4,  4,  8,  8,  8,  8, 12, 12, 12, 12, 12, 12, 12, 12, 
	 1,  5,  5,  9,  9,  9,  9, 13, 13, 13, 13, 13, 13, 13, 13, 
	 0,  4,  4,  8,  8,  8,  8, 12, 12, 12, 12, 12, 12, 12, 12, 
	 3,  7,  7, 11, 11, 11, 11, 15, 15, 15, 15, 15, 15, 15, 15, 
	 0,  4,  4,  8,  8,  8,  8, 12, 12, 12, 12, 12, 12, 12, 12, 
	 1,  5,  5,  9,  9,  9,  9, 13, 13, 13, 13, 13, 13, 13, 13, 
	 0,  4,  4,  8,  8,  8,  8, 12, 12, 12, 12, 12, 12, 12, 12, 
	 2,  6,  6, 10, 10, 10, 10, 14, 14, 14, 14, 14, 14, 14, 14, 
	 0,  4,  4,  8,  8,  8,  8, 12, 12, 12, 12, 12, 12, 12, 12, 
	 1,  5,  5,  9,  9,  9,  9, 13, 13, 13, 13, 13, 13, 13, 13, 
	 0,  4,  4,  8,  8,  8,  8, 12, 12, 12, 12, 12, 12, 12, 12, 
	
	 0,  4,  4,  8,  8,  4,  4, 12, 12, 12, 12, 12, 12, 12, 12, 
	 1,  5,  5,  9,  9,  5,  5, 13, 13, 13, 13, 13, 13, 13, 13, 
	 0,  4,  4,  8,  8,  4,  4, 12, 12, 12, 12, 12, 12, 12, 12, 
	 2,  6,  6, 10, 10, 10, 10, 14, 14, 14, 14, 14, 14, 14, 14, 
	 0,  4,  4,  8,  8,  4,  4, 12, 12, 12, 12, 12, 12, 12, 12, 
	 1,  5,  5, 10, 10,  5,  5, 13, 13, 13, 13, 13, 13, 13, 13, 
	 0,  4,  4,  8,  8,  4,  4, 12, 12, 12, 12, 12, 12, 12, 12, 
	 3,  7,  7, 11, 11,  7,  7, 15, 15, 15, 15, 15, 15, 15, 15, 
	 0,  4,  4,  8,  8,  4,  4, 12, 12, 12, 12, 12, 12, 12, 12, 
	 1,  5,  5,  9,  9,  5,  5, 13, 13, 13, 13, 13, 13, 13, 13, 
	 0,  4,  4,  8,  8,  4,  4, 12, 12, 12, 12, 12, 12, 12, 12, 
	 2,  6,  6, 10, 10, 10, 10, 14, 14, 14, 14, 14, 14, 14, 14, 
	 0,  4,  4,  8,  8,  4,  4, 12, 12, 12, 12, 12, 12, 12, 12, 
	 1,  5,  5, 10, 10,  5,  5, 13, 13, 13, 13, 13, 13, 13, 13, 
	 0,  4,  4,  8,  8,  4,  4, 12, 12, 12, 12, 12, 12, 12, 12, 
	
	 0,  4,  0,  8,  8,  8,  8, 12, 12, 12, 12,  8,  8,  8,  8, 
	 1,  5,  5,  9,  9,  9,  9, 13, 13, 13, 13, 13, 13, 13, 13, 
	 0,  5,  0,  8,  8,  8,  8, 13, 13, 13, 13,  8,  8,  8,  8, 
	 2,  6,  2, 10, 10, 10, 10, 14, 14, 14, 14, 10, 10, 10, 10, 
	 0,  4,  0,  8,  8,  8,  8, 12, 12, 12, 12,  8,  8,  8,  8, 
	 1,  5,  5,  9,  9,  9,  9, 13, 13, 13, 13, 13, 13, 13, 13, 
	 0,  5,  0,  8,  8,  8,  8, 13, 13, 13, 13,  8,  8,  8,  8, 
	 3,  7,  7, 11, 11, 11, 11, 15, 15, 15, 15, 15, 15, 15, 15, 
	 0,  4,  0,  8,  8,  8,  8, 12, 12, 12, 12,  8,  8,  8,  8, 
	 1,  5,  5,  9,  9,  9,  9, 13, 13, 13, 13, 13, 13, 13, 13, 
	 0,  5,  0,  8,  8,  8,  8, 13, 13, 13, 13,  8,  8,  8,  8, 
	 2,  7,  2, 10, 10, 10, 10, 15, 15, 15, 15, 10, 10, 10, 10, 
	 0,  4,  0,  8,  8,  8,  8, 12, 12, 12, 12,  8,  8,  8,  8, 
	 1,  5,  5,  9,  9,  9,  9, 13, 13, 13, 13, 13, 13, 13, 13, 
	 0,  5,  0,  8,  8,  8,  8, 13, 13, 13, 13,  8,  8,  8,  8, 
	
	 0,  4,  4,  8,  0,  4,  4, 12, 12,  4,  4, 12, 12,  4,  4, 
	 1,  5,  5,  9,  1,  5,  5, 13, 13,  5,  5, 13, 13,  5,  5, 
	 0,  4,  4,  8,  0,  4,  4, 12, 12,  4,  4, 12, 12,  4,  4, 
	 2,  6,  6, 10, 10,  6,  6, 14, 14, 14, 14, 14, 14, 14, 14, 
	 0,  4,  4, 10,  0,  4,  4, 14, 14,  4,  4, 14, 14,  4,  4, 
	 2,  6,  6, 10, 10,  6,  6, 14, 14, 14, 14, 14, 14, 14, 14, 
	 0,  4,  4, 10,  0,  4,  4, 14, 14,  4,  4, 14, 14,  4,  4, 
	 3,  7,  7, 11, 11,  7,  7, 15, 15, 15, 15, 15, 15, 15, 15, 
	 0,  4,  4,  8,  0,  4,  4, 12, 12,  4,  4, 12, 12,  4,  4, 
	 1,  5,  5, 11,  1,  5,  5, 15, 15,  5,  5, 15, 15,  5,  5, 
	 0,  4,  4,  8,  0,  4,  4, 12, 12,  4,  4, 12, 12,  4,  4, 
	 2,  6,  6, 10, 10,  6,  6, 14, 14, 14, 14, 14, 14, 14, 14, 
	 0,  4,  4, 10,  0,  4,  4, 14, 14,  4,  4, 14, 14,  4,  4, 
	 2,  6,  6, 10, 10,  6,  6, 14, 14, 14, 14, 14, 14, 14, 14, 
	 0,  4,  4, 10,  0,  4,  4, 14, 14,  4,  4, 14, 14,  4,  4, 
	
	 0,  4,  0,  8,  0,  4,  0, 12,  0,  4,  0,  8,  0,  4,  0, 
	 1,  5,  5,  9,  1,  5,  5, 13,  1,  5,  5, 13,  1,  5,  5, 
	 0,  5,  0,  8,  0,  5,  0, 13,  0,  5,  0,  8,  0,  5,  0, 
	 2,  6,  2, 10, 10, 10, 10, 14,  2, 14,  2, 10, 10, 10, 10, 
	 0,  4,  0, 10,  0, 10,  0, 14,  0,  4,  0, 10,  0, 10,  0, 
	 1,  5,  5, 10, 10,  5,  5, 13,  1,  5,  5, 10, 10,  5,  5, 
	 0,  5,  0, 10,  0,  5,  0, 13,  0,  5,  0, 10,  0,  5,  0, 
	 3,  7,  7, 11, 11,  7,  7, 15, 15, 15, 15, 15, 15, 15, 15, 
	 0,  4,  0,  8,  0,  4,  0, 15,  0, 15,  0, 15,  0, 15,  0, 
	 1,  5,  5, 11,  1,  5,  5, 15, 15,  5,  5, 15, 15,  5,  5, 
	 0,  5,  0,  8,  0,  5,  0, 15,  0,  5,  0, 15,  0,  5,  0, 
	 2,  7,  2, 10, 10, 10, 10, 15, 15, 15, 15, 10, 10, 10, 10, 
	 0,  4,  0, 10,  0, 10,  0, 15,  0, 15,  0, 10,  0, 10,  0, 
	 1,  5,  5, 10, 10,  5,  5, 15, 15,  5,  5, 10, 10,  5,  5, 
	 0,  5,  0, 10,  0,  5,  0, 15,  0,  5,  0, 10,  0,  5,  0
	
};

/****************************************************************************/
/*							Inline functions								*/
/****************************************************************************/

/*
*	Allocates a distance mapping queue item
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	inline	QITEM	*MallocQitem( void )
{
	return( (QITEM *) MallocPool( memQitem, sizeof( QITEM ) ) );
}

/****************************************************************************/
/*							Public functions								*/
/****************************************************************************/

/*
*	Finds the minimum distance between two objects
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	If the two objects are more than limit apart, the returned value
*	is DIST_INF. Otherwise, the returned value is the minimum distance
*	and the RECT structure will contain the matching pixels.
*/

int		DistMin( OBJECT ref, OBJECT act, RECT *coords, int maxd )
{
DLINK	work;
QITEM	*item;

	item	  = MallocQitem();
	item->min = 0;
	
	// First, check if the two objects' bounding boxes overlap or not.
	// If they do, we do a search involving all pixels in the objects.
	// Otherwise a perimeter search will do.
	
	if ( RectOverlap( &ref->bbox, &act->bbox ) ) {
	
		item->ref = ref->atree;
		item->act = act->atree;
	}
	else {
	
		item->ref = ref->ptree;
		item->act = act->ptree;
	}
	
	DlinkInit( &work );
	DlinkAddTail( &work, item );
	
	coords->tlx = ref->bbox.tlx;
	coords->tly = ref->bbox.tly;
	coords->brx = act->bbox.tlx;
	coords->bry = act->bbox.tly;
	
	maxd = DistMinPrepare( &work, coords, maxd );
	maxd = DistMinResolve( &work, coords, maxd );
	
	return( maxd );
}

/*
*	Finds the overlap between objects
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Finds the overlap between two objects. Returns the number of
*	pixels overlapping. If the 'pixs' pointer is not NULL, then
*	a list of pixels will be placed onto the list, indicating
*	the overlap positions.
*/

int		DistOvl( OBJECT ref, OBJECT act, DLINK *pixs )
{
DLINK	work;
QITEM	*item;
RECT	coords;
int		area;
	
	// First, check if the two objects' bounding boxes overlap or not.
	// If not, then we have nothing to do.
	
	if ( ! RectOverlap( &ref->bbox, &act->bbox ) ) return( 0 );
	
	// Objects possibly overlap. 
	// Prepare the data and do the search.
	
	item	   = MallocQitem();
	item->ref  = ref->atree;
	item->act  = act->atree;
	coords.tlx = ref->bbox.tlx;
	coords.tly = ref->bbox.tly;
	coords.brx = act->bbox.tlx;
	coords.bry = act->bbox.tly;
	
	DlinkInit( &work );
	DlinkAddTail( &work, item );
	DistOvlPrepare( &work, &coords );
	area = DistOvlResolve( &work, &coords, pixs );
	return( area );
}

/****************************************************************************/
/*							Local functions									*/
/****************************************************************************/

/*
*	Calculates the distance minimum at block level
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Receives a list containig a top level block pair
*	(a block that is larger than the entire image), that 
*	is, the roots of the objects' spatial division trees.
*	It also receives a rectange that contains the absolute 
*	position pair from which the block offsets are calculated.
*	In addition, it gets the maximum acceptable distance.
*	In the rectangle block the top-left coordinates should be 
*	that of the object represented by the 'ref' field of the 
*	queue items, and the bottom-right coordinates should belong 
*	to the 'act' fields (its top-left coordinates, of course).
*
*	It will remove the top block from the queue and put all pairs
*	of quad-pixel block pairs onto it that have a chance to contain
*	pixels that are as close or closer than the requested distance.
*	Also returns an upper bound for the maximum distance to be
*	considered at the pixel level That bound is not larger than the
*	max distance received. 
*	If it can prove without going down to pixel level that the
*	requested distance can not be achieved at all, then the queue
*	will be empty and DIST_INF.
*
*	The rectangle structure is always left unchanged.
*/

static	int		DistMinPrepare( DLINK *old, RECT *offs, int maxd )
{
DLINK	new;
QITEM	*pair, *work;
int		size, dist;
int		r, rx, ry, rxm, rym;
int		a, ax, ay, axm, aym;

	for ( size = 1 << MAXLEV ; size > 1 ; size >>= 1 ) {	
		
		// Initialise the result queue
		
		DlinkInit( &new );
		
		while ( ( pair = DlinkRemoveFirst( old ) ) ) {
		
			rx = pair->ref->x + offs->tlx;
			ry = pair->ref->y + offs->tly;
			ax = pair->act->x + offs->brx;
			ay = pair->act->y + offs->bry;
			
			// If the min. distance of the pair is larger than our current, 
			// don't do anything.
			
			if ( pair->min > maxd ) {
			
				Free( pair );
				continue;
			}
			
			// Now check the 16 combinations and keep the possibles
			// We can't really do much else, the only thing we know
			// at this point is that if the minimal distance between
			// a quadrant pair is larger than the maximum of any other
			// pairs, we can throw away the pair.
			
			for ( r = 0 ; r < 4 ; r++ ) {
		
				if ( ! pair->ref->quad[ r ] ) continue;
			
				rxm = rx + ( ( ( 10 >> r ) & 1 ) ? size : 0 );
				rym = ry + ( ( ( 12 >> r ) & 1 ) ? size : 0 );
			
				for ( a = 0 ; a < 4 ; a++ ) {
		
					if ( ! pair->act->quad[ a ] ) continue;
			
					axm = ax + ( ( ( 10 >> a ) & 1 ) ? size : 0 );
					aym = ay + ( ( ( 12 >> a ) & 1 ) ? size : 0 );
			
					// We have a possible pair. Get the minimum possible
					// distance between them
				
					dist = SquareDistMin( rxm, rym, axm, aym, size - 1 );
		
					// Now, we keep a pair only if min. dist <= maxd 

					if ( dist <= maxd ) {
				
						// Create a new pair, schedule it for the next round
						// We put it to the beginning of the queue, so that
						// the next round starts with the tightest block,
						// increasing the chance of throwing things away
				
						work	  = MallocQitem();
						work->ref = pair->ref->quad[ r ];
						work->act = pair->act->quad[ a ];
						work->min = dist;
						DlinkAddHead( &new, work );
				
						// Update maxd -> calculate max distances
				
						dist = SquareDistMax( rxm, rym, axm, aym, size - 1 );

						// If we found a smaller upper limit, keep it
						
						if ( dist < maxd ) maxd = dist;
					}
				}
			}
			
			// Release the pair that we processed
		
			Free( pair );
		}
		
		// Swap queues
		
		*old = new;
	}
	
	return( DlinkFirst( old ) ? maxd : DIST_INF );
}

/*
*	Calculates the distance minimum at pixel level
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Receives a list of 4-pixel block pairs, a rectangle
*	that contains the absolute position pair from which 
*	the block offsets are calculated and the maximum 
*	distance that is acceptable. In the rectangle block
*	the top-left coordinates should be that of the object
*	represented by the 'ref' field of the queue items, and
*	the bottom-right coordinates should belong to the 'act'
*	fields.
*
*	If it can achieve the requested distance, or shorter, it
*	returns that and the rectangle will be filled with the
*	absolute coordinates of the two pixels that were found.
*	If there is no possible pixel pairing so that their
*	distance is no greater than the requested minimum, then
*	it will return DIST_INF and the rectangle will not be touched.
*	
*	The block pair queue will be emptied and all of its members freed.
*/

static	int		DistMinResolve( DLINK *old, RECT *offs, int maxd )
{
int		xo, yo;
int		dx, dy, dd;
int		i, seli;
QITEM	*pair, *selp;

	selp = NULL;
	seli = 0;
	xo	 = offs->tlx - offs->brx;
	yo	 = offs->tly - offs->bry;
		
	while ( ( pair = DlinkRemoveFirst( old ) ) ) {
		
		// If we reached zero distance, we don't have to bother any more
			
		if ( ! maxd ) {
			
			Free( pair );
			continue;
		}
			
		// If the min. distance of the pair is larger than our current, 
		// just ignore the pair
			
		if ( pair->min > maxd ) {
			
			Free( pair );
			continue;
		}
			
		// Calculate the distance vector between the blocks
		// and the difference between the vector components
			
		dx = pair->act->x - pair->ref->x - xo;
		dy = pair->act->y - pair->ref->y - yo;
		dd = Abs( dx ) - Abs( dy );
		
		// Calculate the closest pixel pair
		
		i  = dx < 0 ? 18 : dx > 0 ? 9 : 0;
		i += dy < 0 ?  6 : dy > 0 ? 3 : 0;
		i += dd < 0 ?  2 : dd > 0 ? 1 : 0;
		i  = index_single[ i ];
		i += pair->ref->m << 4;
		i -= 16 + pair->ref->m - pair->act->m;
		i  = resol_single[ i ];
			
		// Calculate the real distance between these pixels

		dx += mod_single_dx[ i ];
		dy += mod_single_dy[ i ];
		dd  = Sqr( dx ) + Sqr( dy );
		
		if ( dd < maxd || ( dd == maxd && ! selp ) ) {
		
			// We have a possible winner. If we had a previous choice,
			// we can get rid of it now.
			
			if ( selp ) Free( selp );
			
			// We don't calculate anything now, just note all the
			// data that we need to work out the final results later.
			// Also, make note of the new target distance.
			
			selp = pair;
			seli = i;
			maxd = dd;
		}
		else {
			
			// Release the pair that we have processed
			
			Free( pair );
		}
	}
	
	if ( selp ) {
	
		// There was a solution. Calculate the absolute pixel
		// positions and fill the rectange with it
		
		offs->tlx += selp->ref->x + mod_single_arx[ seli ];
		offs->tly += selp->ref->y + mod_single_ary[ seli ];
		offs->brx += selp->act->x + mod_single_aax[ seli ];
		offs->bry += selp->act->y + mod_single_aay[ seli ];
		Free( selp );
		return( maxd );
	}
	else {
	
		// No solution, return infinity
		
		return( DIST_INF );
	}
}

/*
*	Overlap calculations, squares larger than a pixel
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	DistOvlPrepare( DLINK *old, RECT *offs )
{
DLINK	new;
QITEM	*pair, *work;
int		size;
int		r, rx, ry, rxm, rym;
int		a, ax, ay, axm, aym;

	for ( size = 1 << MAXLEV ; size > 1 ; size >>= 1 ) {	
		
		// Initialise the result queue
		
		DlinkInit( &new );
		
		while ( ( pair = DlinkRemoveFirst( old ) ) ) {
		
			// Calculate the base offset between the blocks
			
			rx = pair->ref->x + offs->tlx;
			ry = pair->ref->y + offs->tly;
			ax = pair->act->x + offs->brx;
			ay = pair->act->y + offs->bry;
			
			// Go through the combinations 
			
			for ( r = 0 ; r < 4 ; r++ ) {
			
				if ( ! pair->ref->quad[ r ] ) continue;
				
				rxm = rx + ( ( ( 10 >> r ) & 1 ) ? size : 0 );
				rym = ry + ( ( ( 12 >> r ) & 1 ) ? size : 0 );
			
				for ( a = 0 ; a < 4 ; a++ ) {
		
					if ( ! pair->act->quad[ a ] ) continue;
			
					axm = ax + ( ( ( 10 >> a ) & 1 ) ? size : 0 );
					aym = ay + ( ( ( 12 >> a ) & 1 ) ? size : 0 );
			
					if ( Abs( rxm - axm ) < size && Abs( rym - aym ) < size ) {
			
						// These quadrants overlap. Put them onto the
						// new queue
					
						work	  = MallocQitem();
						work->ref = pair->ref->quad[ r ];
						work->act = pair->act->quad[ a ];
						DlinkAddHead( &new, work );
					}
				}
			}
			
			// Release the pair that we processed
		
			Free( pair );
		}
		
		// Swap queues
		
		*old = new;
	}
}

/*
*	Overlap calculations, single pixel level
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	int		DistOvlResolve( DLINK *old, RECT *offs, DLINK *pix )
{
QITEM	*pair;
int		r, rx, ry;
int		a, dx, dy;
int		i, pixno;
PIXEL	pixel;

	pixno = 0;
	if ( pix ) DlinkInit( pix );
	
	while ( ( pair = DlinkRemoveFirst( old ) ) ) {
		
		rx = pair->ref->x + offs->tlx;
		ry = pair->ref->y + offs->tly;
		dx = ( pair->act->x + offs->brx ) - rx;
		dy = ( pair->act->y + offs->bry ) - ry;
			
		// Go through the combinations 
			
		for ( i = 0 ; i < 16 ; i++ ) {
			
			if ( ! pair->act->quad[ i >> 2 ] ) continue;
			if ( ! pair->ref->quad[ i  & 3 ] ) continue;

			if ( dx + mod_single_dx[ i ] ) continue;
			if ( dy + mod_single_dy[ i ] ) continue;
			
			// These pixels exist and overlap.
			
			pixno++;
			
			// If we have to create a list, do so
			
			if ( pix ) {
			
				pixel = MallocPixel();
				pixel->x = rx + mod_single_arx[ i ];
				pixel->y = ry + mod_single_ary[ i ];
				DlinkAddTail( pix, pixel );
			}
		}
		
		// Release the pair that we processed
		
		Free( pair );
	}
	
	return( pixno );
}
