/****************************************************************************/
/* Copyright 2004,  Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	Useful functions for distance calculations								*/
/*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~								*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zoltn Kcsi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#ifndef __DISTFUNC_H__
#define __DISTFUNC_H__

/*
*	Integer square
*	~~~~~~~~~~~~~~
*/

static inline int		Sqr( short x )
{
	return( x * x );
}

/*
*	Absolute value
*	~~~~~~~~~~~~~~
*/

static inline short		Abs( short x )
{
	return( x < 0 ? -x : x );
}

/*
*	Tells if two rectangles overlap in X direction
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static inline int		RectOverlapX( RECT *a, RECT *b )
{
	if ( a->tlx > b->brx ) return( FALSE );
	if ( b->tlx > a->brx ) return( FALSE );
	return( TRUE );
}

/*
*	Tells if two rectangles overlap in Y direction
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static inline int		RectOverlapY( RECT *a, RECT *b )
{
	if ( a->tly > b->bry ) return( FALSE );
	if ( b->tly > a->bry ) return( FALSE );
	return( TRUE );
}

/*
*	Tells if two rectangles overlap or not
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static inline int		RectOverlap( RECT *a, RECT *b )
{
	return( RectOverlapX( a, b ) && RectOverlapY( a, b ) );
}

/*
*	Tells the minimum distance between two rectangles
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static inline int		RectMinDist( RECT *a, RECT *b )
{
RECT	*t;

	if ( RectOverlap( a, b ) ) return( 0 );
	
	if ( RectOverlapX( a, b ) ) {
	
		if ( RectOverlapY( a, b ) ) {
		
			return( 0 );
		}
		else {
		
			if ( a->bry < b->tly ) {
			
				return( Sqr( a->bry - b->tly ) );
			}
			else {
			
				return( Sqr( a->tly - b->bry ) );
			}
		}
	}
	else {
	
		if ( RectOverlapY( a, b ) ) {
		
			if ( a->brx < b->tlx ) {
			
				return( Sqr( a->brx - b->tlx ) );
			}
			else {
			
				return( Sqr( a->tlx - b->brx ) );
			}
		}
		else {
		
			if ( a->bry < b->tly ) {
			
				if ( a->brx < b->tlx ) {
				
					return( Sqr( a->bry - b->tly ) + Sqr( a->brx - b->tlx ) );
				}
				else {
				
					return( Sqr( a->bry - b->tly ) + Sqr( b->brx - a->tlx ) );
				}
			}
			else {
			
				if ( a->brx < b->tlx ) {
				
					return( Sqr( b->bry - a->tly ) + Sqr( a->brx - b->tlx ) );
				}
				else {
				
					return( Sqr( b->bry - a->tly ) + Sqr( b->brx - a->tlx ) );
				}
			}
		}
	}
}

/*
*	Calculate min distances
*	~~~~~~~~~~~~~~~~~~~~~~~
*	Returns the square of the minimum distance between 2 squares
*	that are s by s large and their top-left coordinates are
*	at (x1,y1) and (x2,y2), respectively.
*/

static	inline	int		SquareDistMin( short x1, short y1, short x2, short y2, 
									   short s )
{
short	dx;
short	dy;

	dx = Abs( x1 - x2 );
	dy = Abs( y1 - y2 );
	
	if ( dx <= s )
				
		if ( dy <= s )
					
			return( 0 );
		else
			return( Sqr( dy - s ) );
	else
		if ( dy <= s )
						
			return( Sqr( dx - s ) );
		else
			return( Sqr( dx - s ) + 
					Sqr( dy - s ) );
}

/*
*	Calculate max distances
*	~~~~~~~~~~~~~~~~~~~~~~~
*	Returns the square of the maximum distance between 2 squares
*	that are s+1 by s+1 large and their top-left coordinates are
*	at (x1,y1) and (x2,y2), respectively.
*/

static	inline	int		SquareDistMax( short x1, short y1, short x2, short y2, 
									   short s )
{
	return( Sqr( Abs( x1 - x2 ) + s ) + Sqr( Abs( y1 - y2 ) + s ) );
}

#endif
