/****************************************************************************/
/* Copyright 2004,  Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	Gamma corrects the R and G layers of an image							*/
/*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~							*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zoltn Kcsi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#include <math.h>
#include <setjmp.h>

#include <types.h>
#include <image.h>
#include <progress.h>
#include <process.h>

/****************************************************************************/
/*							Public functions								*/
/****************************************************************************/

/*
*	Gamma corrects and equalises an image
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters
*
*		old:	Original image
*		gamm:	Gamma value
*	
*	Return value
*
*		A pointer to the new image or NULL if could not finish
*
*	Receives a 2-layer image. For both layers, scale the highest values
*	to 255 and run a gamma correction on all pixels with the given
*	gamma value. Return the resulting image or NULL, if the processing
*	was aborted.
*/

IMAGE	*GammaImage( IMAGE *image, int equal, double gamm )
{
int		gam_tab[ 256 ];
int		x, y;
IMAGE	*res;
int		maxr, maxg;
double	multr, multg;
jmp_buf	state;

	gam_tab[ 0 ] = 0;
	
	for ( x = 1 ; x < 256 ; x++ ) {
	
		gam_tab[ x ] = 0.5 + 255 * exp( log( x / 255.0 ) / gamm );
	}
	
	res = ImageCreate( image->y, image->x, 2 );
	
	if ( setjmp( state ) ) {
	
		// Aborted
		
		ImageDestroy( res );
		return( NULL );
	}
	
	ProgressInit( "Gamma correction", state );
	
	if ( equal ) {
	
		maxr = 0;
		maxg = 0;
		
		for ( y = 0 ; y < image->y ; y++ ) {
	
			ProgressUpdate( y, image->y );
		
			for ( x = 0 ; x < image->x ; x++ ) {
	
				if ( image->im[R][y][x] > maxr ) maxr = image->im[R][y][x];
				if ( image->im[G][y][x] > maxg ) maxg = image->im[G][y][x];
			}
		}
		
		multr = 255.0 / maxr;
		multg = 255.0 / maxg;
	}
	else {
	
		multr = 1.0;
		multg = 1.0;
	}
	
	for ( y = 0 ; y < image->y ; y++ ) {
	
		ProgressUpdate( image->y + y, 2 * image->y );
		
		for ( x = 0 ; x < image->x ; x++ ) {
			
			maxr = (int)( image->im[ R ][ y ][ x ] * multr + 0.5 );
			maxg = (int)( image->im[ G ][ y ][ x ] * multg + 0.5 );
			
			res->im[ R ][ y ][ x ] = gam_tab[ maxr ];
			res->im[ G ][ y ][ x ] = gam_tab[ maxg ];
		}
	}
	
	ProgressDone();
	return( res );
}
