/****************************************************************************/
/* Copyright 2004, © Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	Object extraction - Select closest neighbour							*/
/*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~							*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zoltán Kócsi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#include <stdio.h>
#include <math.h>
#include <setjmp.h>

#include <types.h>
#include <memory.h>
#include <dlinklist.h>
#include <objects.h>
#include <process.h>
#include <progress.h>
#include "process_intern.h"

/****************************************************************************/
/*						Local definitions									*/
/****************************************************************************/

#define	SQR( x )	((x)*(x))

/****************************************************************************/
/*						Public functions									*/
/****************************************************************************/

/*
*	Selects the closest neighbour for each object
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters
*		list		A list of objects
*		neig		A list of segments (distance objects)
*
*	Return value:
*		The number of segments
*
*	The method is simply selecting the closest neighbour.
*	It should be noted that it is not a commutative relation,
*	if A's closest neighbour is B it does not mean that B's
*	closest neighbour is A. Therefore, we do not try to eliminate
*	duplicates, we calculate the closest for each object and return 
*	that.
*/

int		Neighbour( DLINK *list, DLINK *neig )
{
OBJECT	this, that;
int		n, min, d, x1, x2, y1, y2, xs, ys;
DIST	*segm;

	// Initialise resulting list
	
	DlinkInit( neig );
	
	// The following initialisation if to keep gcc's data flow analysis happy
	
	xs = 0;
	ys = 0;
	
	// Run through the list and for each object run through the list
	// again, selecting the closest neighbour. It is fairly time
	// consuming O(n^2) but still faster than the other two.
	
	for ( n = 0, this = DlinkFirst( list ) ; this ; this = DlinkNext( this ) ) {
		
		min = DIST_INF;
		x1  = this->xc + this->bbox.tlx;
		y1  = this->yc + this->bbox.tly;
		
		for ( that = DlinkFirst( list ) ; that ; that = DlinkNext( that ) ) {
			
			// Don't check the distance between I and me
			
			if ( this == that ) continue;
			
			// Calculate the distance
			
			x2 = that->xc + that->bbox.tlx;
			y2 = that->yc + that->bbox.tly;
			
			d = SQR( x1 - x2 ) + SQR( y1 - y2 );
			
			// Store the minimum
			
			if ( min > d ) {
				
				min = d;
				xs  = x2;
				ys  = y2;
			}
		}
		
		// Create a distance object and fill it in
		
		segm = MallocDistance();
		segm->pair.tlx = x1;
		segm->pair.tly = y1;
		segm->pair.brx = xs;
		segm->pair.bry = ys;
		segm->dist	   = min;
		DlinkAddTail( neig, segm );
		
		// Increment the object counter
		
		n++;
	}
	
	// Done
	
	return( n );
}

