/****************************************************************************/
/* Copyright 2004, © Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	Object extraction - Select closest neighbour							*/
/*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~							*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zoltán Kócsi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#include <stdio.h>
#include <math.h>
#include <setjmp.h>

#include <types.h>
#include <memory.h>
#include <dlinklist.h>
#include <objects.h>
#include <process.h>
#include <progress.h>
#include "process_intern.h"

/****************************************************************************/
/*						Local definitions									*/
/****************************************************************************/

#define	SQR( x )	((x)*(x))

/****************************************************************************/
/*						Public functions									*/
/****************************************************************************/

/*
*	Selects the closest neighbour for each object
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	Parameters
*		list		A list of objects
*		rats		A list of segments (distance objects)
*
*	Return value:
*		The number of segments
*
*	The method pairs every object with every other object.
*	The duplicates are eliminated.
*/

int		Ratsnest( DLINK *list, DLINK *rats )
{
OBJECT	this, that;
int		n, d, x1, x2, y1, y2;
DIST	*segm;

	// Initialise resulting list
	
	DlinkInit( rats );
	
	// Do the pairing
	
	for ( n = 0, this = DlinkFirst( list ) ; this ; this = DlinkNext( this ) ) {
		
		x1  = this->xc + this->bbox.tlx;
		y1  = this->yc + this->bbox.tly;
		
		for ( that = DlinkNext( this ) ; that ; that = DlinkNext( that ) ) {
			
			// Calculate the distance
			
			x2 = that->xc + that->bbox.tlx;
			y2 = that->yc + that->bbox.tly;
			
			d = SQR( x1 - x2 ) + SQR( y1 - y2 );
			
			// Create a distance object and fill it in
		
			segm = MallocDistance();
			segm->pair.tlx = x1;
			segm->pair.tly = y1;
			segm->pair.brx = x2;
			segm->pair.bry = y2;
			segm->dist	   = d;
			DlinkAddTail( rats, segm );
		
			// Increment the object counter
		
			n++;
		}
	}
	
	// Done
	
	return( n );
}

