/****************************************************************************/
/* Copyright 1998, (c) Bendor Research Pty. Ltd.	All rights reserved.	*/
/*																			*/
/* Permission is granted to use this software for any purpose as long as	*/
/* this copyright notice remains intact in all sources.						*/
/* Bendor Research Pty. Ltd. does not guarantee the quality of this			*/
/* software and does not assume any liability whatsoever for any damages	*/
/* arising out of the use of this software.									*/
/*																			*/
/****************************************************************************/
/*																			*/
/*	This file contains the doubly-linked list handler						*/
/*																			*/
/*	History:																*/
/*																			*/
/*	10/11/98	Zoltan Kocsi	Original revision							*/
/*																			*/
/****************************************************************************/

#include "dlinklist.h"

#define	HEAD( x )	((x)->head)
#define	TAIL( x )	((x)->tail)

#define	NEXT( x ) (((DLINK *) (x))->head)
#define	PREV( x ) (((DLINK *) (x))->tail)

/*
*	Inserts an element to the head of the list
*/

void	DlinkAddHead( DLINK *list, void *element )
{
	PREV( element ) = NULL;
	
	if ( ( NEXT( element ) = HEAD( list ) ) )
	
		PREV( NEXT( element ) ) = element;
	else
		TAIL( list ) = element;
		
	HEAD( list ) = element;
}

/*
*	Inserts an element to the tail of the list
*/

void	DlinkAddTail( DLINK *list, void *element )
{
	NEXT( element ) = NULL;
	
	if ( ( PREV( element ) = TAIL( list ) ) )
	
		NEXT( PREV( element ) ) = element;
	else
		HEAD( list ) = element;
		
	TAIL( list ) = element;
}

/*
*	Inserts an element after the given one
*/

void	DlinkAddAfter( DLINK *list, void *element, void *after )
{
	if ( ! after || ! NEXT( after ) ) {
	
		DlinkAddTail( list, element );
	}
	else {
	
		NEXT( element ) = NEXT( after );
		PREV( element ) = after;
		NEXT( after ) = element;
		PREV( NEXT( element ) ) = element;
	}
}

/*
*	Inserts an element before the given one
*/

void	DlinkAddBefore( DLINK *list, void *element, void *before )
{
	if ( ! before || ! PREV( before ) ) {
	
		DlinkAddHead( list, element );
	}
	else {
	
		PREV( element ) = PREV( before );
		NEXT( element ) = before;
		PREV( before ) = element;
		NEXT( PREV( element ) ) = element;
	}
}

/*
*	Removes the first element of the list
*/

void	*DlinkRemoveFirst( DLINK *list )
{
void	*member;

	if ( ( member = HEAD( list ) ) ) {
	
		if ( ( HEAD( list ) = NEXT( member ) ) )
		
			PREV( NEXT( member ) ) = NULL;
		else
			TAIL( list ) = NULL;
	}
	
	return( member );
}

/*
*	Removes the last element of the list
*/

void	*DlinkRemoveLast( DLINK *list )
{
void	*member;

	if ( ( member = TAIL( list ) ) ) {
	
		if ( ( TAIL( list ) = PREV( member ) ) )
		
			NEXT( PREV( member ) ) = NULL;
		else
			HEAD( list ) = NULL;
	}
	
	return( member );
}

/*
*	Removes the given element from the list
*/

void	DlinkRemove( DLINK *list, void *element )
{
	if ( NEXT( element ) )
	
		PREV( NEXT( element ) ) = PREV( element );
	else
		TAIL( list ) = PREV( element );
	
	if ( PREV( element ) )
	
		NEXT( PREV( element ) ) = NEXT( element );
	else
		HEAD( list ) = NEXT( element );
}

/*
*	Appends the second list to the first one. The second list
*	becomes empty. Returns the pointer to the resulting list.
*/

DLINK	*DlinkCat( DLINK *list1, DLINK *list2 )
{
	if ( HEAD( list2 ) ) {
	
		if ( ! HEAD( list1 ) ) {
	
			*list1 = *list2;
		}
		else {
		
			PREV( HEAD( list2 ) ) = TAIL( list1 );
			NEXT( TAIL( list1 ) ) = HEAD( list2 );
			TAIL( list1 )         = TAIL( list2 );
		}
		
		DlinkInit( list2 );
	}	
	return( list1 );
}

/*
*	This function splits the first list at the given element and puts the
*	list starting with the given element to the second list.
*/

void	DlinkSplit( DLINK *list1, DLINK *list2, void *element )
{
	if ( ! HEAD( list1 ) || ! element ) {
	
		DlinkInit( list2 );
	}
	else {
	
		TAIL( list2 ) = TAIL( list1 );
		
		if ( ! ( TAIL( list1 ) = PREV( element ) ) )
		
			HEAD( list1 ) = NULL;
		else
			NEXT( TAIL( list1 ) ) = NULL;
		
		HEAD( list2 ) = element;
		PREV( HEAD( list2 ) ) = NULL;
	}
}
