/****************************************************************************/
/* Copyright 2004,  Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	File selection box code													*/
/*	~~~~~~~~~~~~~~~~~~~~~~~													*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zoltn Kcsi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <glib.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>

#include <types.h>
#include "file.h"

/****************************************************************************/
/*						Local definitions									*/
/****************************************************************************/

#define	MAXNAME		2048

/****************************************************************************/
/*						Prototypes											*/
/****************************************************************************/

static	void	GuiFileOpenDestroy( GtkWidget *box );
static	void	GuiFileCancel( GtkWidget *box, GtkWidget *dummy );
static	void	GuiFileOpenOk( GtkWidget *box, GtkWidget *dummy );

/****************************************************************************/
/*						Local data											*/
/****************************************************************************/

/*
*	Normally, static data would be a big no-no, but since the file dialog
*	is modal, only one can be opened at any given time, so it's OK.
*/

static	char	work_dir[ MAXNAME ] = "";
static	char	dflt_dir[ MAXNAME ] = "";
static	char	file_nam[ MAXNAME ] = "";
static	void	(*callback)( char *, void *par );
static	void	*param;

static	GtkWidget	*ok, *fs;

/****************************************************************************/
/*						Public functions									*/
/****************************************************************************/

/*
*	File menu for loading image
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	If the user successfully selects a file, then the 'action'
*	callback will be called with the filename. If the user cancels
*	or closes the dialog, nothing happens.
*/

void	GuiFileGetName( char *title, char *mask, 
						void (*action)( char *, void * ), void *par )
{
GtkWidget	*fsel;
	
	// Non GTK business
	
	if ( ! work_dir[ 0 ] ) getcwd( work_dir, MAXNAME );
	if ( ! dflt_dir[ 0 ] ) strcpy( dflt_dir, work_dir );
	chdir( dflt_dir );
	callback = action;
	param	 = par;
		
	// GTK business
		
	fsel = gtk_file_selection_new( title );
	
	if ( mask ) {
	
		gtk_file_selection_complete( GTK_FILE_SELECTION( fsel ), mask );
	}
		
	ok = GTK_FILE_SELECTION( fsel )->ok_button;
	fs = fsel;
	
	gtk_signal_connect( GTK_OBJECT( fsel ), "destroy",
						GTK_SIGNAL_FUNC( GuiFileOpenDestroy ), NULL );
	
	gtk_signal_connect_object( GTK_OBJECT( GTK_FILE_SELECTION( fsel )->ok_button ), 
						"clicked",
						GTK_SIGNAL_FUNC( GuiFileOpenOk ), (gpointer)fsel );
	
	gtk_signal_connect_object( GTK_OBJECT(GTK_FILE_SELECTION( fsel )->cancel_button ),
						"clicked",
						GTK_SIGNAL_FUNC( GuiFileCancel ), (gpointer)fsel );
	
	gtk_widget_show( fsel );
	gtk_grab_add( fsel );
}

/****************************************************************************/
/*						Local functions										*/
/****************************************************************************/

/*
*	Call-back for destruction of the file select window
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	GuiFileOpenDestroy( GtkWidget *box )
{
	if ( box ) {
	
		chdir( work_dir );
		gtk_grab_remove( box );
		gtk_widget_destroy( box );
	}
}

/*
*	Callback for cancel
*	~~~~~~~~~~~~~~~~~~~
*/

static	void	GuiFileCancel( GtkWidget *box, GtkWidget *dummy )
{
	GuiFileOpenDestroy( box );
}

/*
*	Call-back when the user selected a file
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

static	void	GuiFileOpenOk( GtkWidget *box, GtkWidget *dummy )
{
char	*name;
char	*slash;

	// Get the filename
	
	name = (char *) gtk_file_selection_get_filename(GTK_FILE_SELECTION(box));
	strcpy( file_nam, name );
	
	// Search for the directory part of the filename
	// so that next time we open there
	
	slash = rindex( file_nam, '/' );
	
	if ( slash && slash != file_nam && slash[ 1 ] ) {
	
		*slash = 0;
		strcpy( dflt_dir, file_nam );
		*slash = '/';
	}
		
	// Inform the caller that we obtained a filename

	if ( callback ) (*callback)( file_nam, param );
	GuiFileOpenDestroy( box );
}
