/****************************************************************************/
/* Copyright 2004,  Bendor Research Pty. Ltd.			All Rights Reserved	*/
/****************************************************************************/
/*																			*/
/*	Wrapper functions around the most frequently used GTK functions			*/
/*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~		*/
/*																			*/
/*	Who				When			Comment									*/
/*																			*/
/*	Zoltn Kcsi	2004 Mar 18		Original code							*/
/*																			*/
/****************************************************************************/

#include <gtkwrappers.h>

/*
*	Creates a horizontal packing box
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*GuiHbox( int homo, int border )
{
GtkWidget	*box;

	box = gtk_hbox_new( homo, border );
	gtk_widget_show( box );
	return( box );
}

/*
*	Creates a vertical packing box
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*GuiVbox( int homo, int border )
{
GtkWidget	*box;

	box = gtk_vbox_new( homo, border );
	gtk_widget_show( box );
	return( box );
}

/*
*	Adds an item to a packing box
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	GuiBoxAdd( GtkWidget *obj, GtkWidget *box, int wide, int fill )
{
	gtk_box_pack_start( GTK_BOX( box ), obj, wide, fill, 0 );
}

/*
*	Creates a horizontal separator line and adds it to a box
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	GuiHline( GtkWidget *box )
{
GtkWidget	*lin;

	lin = gtk_hseparator_new();
	gtk_widget_show( lin );
	gtk_box_pack_start( GTK_BOX( box ), lin, FALSE, TRUE, 0 );
}

/*
*	Creates a vertical separator line and adds it to a box
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

void	GuiVline( GtkWidget *box )
{
GtkWidget	*lin;

	lin = gtk_vseparator_new();
	gtk_widget_show( lin );
	gtk_box_pack_start( GTK_BOX( box ), lin, FALSE, TRUE, 0 );
}

/*
*	Creates a frame and puts something in it
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*GuiFrame( char *txt, GtkStyle *style, 
					   GtkWidget *content, int shadow )
{
GtkWidget	*fra;

	fra = gtk_frame_new( txt );
	gtk_widget_show( fra );
	gtk_frame_set_shadow_type( GTK_FRAME( fra ), shadow );
	gtk_frame_set_label_align( GTK_FRAME( fra ), 0.5, 0 );
	if ( style ) gtk_widget_set_style( fra, style );
	if ( content ) gtk_container_add( GTK_CONTAINER( fra ), content );
	return( fra );
}

/*
*	Creates a toggle button and puts it in a box
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*GuiToggle( char *txt, void *handler, void *param,
						GtkWidget *box, int wide, int fill )
{
GtkWidget	*but;

	but = gtk_check_button_new_with_label( txt );
	gtk_widget_show( but );
	gtk_box_pack_start( GTK_BOX( box ), but, wide, fill, 0 );
	gtk_signal_connect( GTK_OBJECT( but ), "toggled",
						GTK_SIGNAL_FUNC( handler ), (gpointer) param );
	return( but );
}

/*
*	Creates a press button and puts it in a box
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*GuiButton( char *txt, void *handler, void *param,
						GtkWidget *box, int wide, int fill )
{
GtkWidget	*but;

	but = gtk_button_new_with_label( txt );
	gtk_widget_show( but );
	gtk_box_pack_start( GTK_BOX( box ), but, wide, fill, 0 );
	gtk_signal_connect( GTK_OBJECT( but ), "clicked",
						GTK_SIGNAL_FUNC( handler ), (gpointer) param );
	return( but );
}

/*
*	Creates a label with the given style and buts it in a box
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*GuiLabel( char *txt, GtkStyle *style,
					   GtkWidget *box, int wide, int fill )
{
GtkWidget	*lab;

	lab = gtk_label_new( txt );
	gtk_widget_show( lab );
	if ( style ) gtk_widget_set_style( lab, style );
	if ( box ) gtk_box_pack_start( GTK_BOX( box ), lab, wide, fill, 0 );
	return( lab );
}

/*
*	Creates a radio button and places it in a box
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*GuiRadio( char *txt, GSList **grp, void *handler, void *param,
					   GtkWidget *box, int wide, int fill )
{
GtkWidget	*rad;

	rad  = gtk_radio_button_new_with_label( *grp, txt );
	*grp = gtk_radio_button_group( GTK_RADIO_BUTTON( rad ) );
	gtk_widget_show( rad );
	gtk_box_pack_start( GTK_BOX( box ), rad, wide, fill, 0 );
	gtk_signal_connect( GTK_OBJECT( rad ), "toggled",
						GTK_SIGNAL_FUNC( handler ), (gpointer) param );
	return( rad );
}

/*
*	Creates a menu and adds it to a box
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*	The menu is surrounded by an etched out frame
*/

GtkWidget	*GuiMenu( char *txt, GtkWidget **menu, GtkWidget **list,
					  GtkWidget *box, int wide, int fill )
{
GtkWidget	*item;

	*list = gtk_menu_new();
	*menu = gtk_option_menu_new();
	item  = gtk_frame_new( txt );
	gtk_widget_show( *list );
	gtk_widget_show( *menu );
	gtk_widget_show( item );				
	gtk_option_menu_set_menu( GTK_OPTION_MENU( *menu ), *list );
	gtk_frame_set_label_align( GTK_FRAME( item ), 0.5, 0 );
	gtk_frame_set_shadow_type( GTK_FRAME( item ), GTK_SHADOW_ETCHED_OUT );
	gtk_widget_set_usize( item, CPLX_XSIZE, CPLX_YSIZE );
	gtk_container_add( GTK_CONTAINER( item ), *menu );
	gtk_box_pack_start( GTK_BOX( box ), item, wide, fill, 0 );
	return( item );
}

/*
*	Adds a menu item to menu
*	~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*GuiMenuItem( char *txt, GtkWidget *list, GSList **group,
						  void *handler, void *param )
{
GtkWidget	*item;

	item = gtk_radio_menu_item_new_with_label( *group, txt );
	gtk_widget_show( item );
	*group = gtk_radio_menu_item_group( GTK_RADIO_MENU_ITEM( item ) );
	gtk_menu_append( GTK_MENU( list ), item );
	gtk_signal_connect( GTK_OBJECT( item ), "activate",
						GTK_SIGNAL_FUNC( handler ), (gpointer) param );
	return( item );
}

/*
*	Creates a slider and puts it in a box
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*GuiSlider( char *txt, GtkObject **adj, GtkWidget **lab,
						void *handler, void *param,
					    GtkWidget *box, int wide, int fill )
{			
GtkObject	*adjus;
GtkWidget	*item, *scale, *label, *lfram, *hpack;

	hpack = gtk_hbox_new( FALSE, 5 );
	gtk_widget_show( hpack );
	
	adjus = gtk_adjustment_new( 0.5, 0, 1.01, 0.01, 0.01, 0.01 );
	scale = gtk_hscale_new( GTK_ADJUSTMENT( adjus ) );
	gtk_widget_show( scale );
	gtk_scale_set_draw_value( GTK_SCALE( scale ), FALSE );
	gtk_scale_set_digits(  GTK_SCALE( scale ), 2 );
	gtk_box_pack_start( GTK_BOX( hpack ), scale, TRUE, TRUE, 0 );
	
	label = gtk_label_new( "" );
	gtk_label_set_justify( GTK_LABEL( label ), GTK_JUSTIFY_CENTER );
	gtk_widget_show( label );
	gtk_widget_set_usize( label, 50, 0 );
	
	lfram = gtk_frame_new( NULL );
	gtk_widget_show( lfram );
	gtk_frame_set_shadow_type( GTK_FRAME( lfram ), GTK_SHADOW_IN );
	gtk_frame_set_label_align( GTK_FRAME( lfram ), 1.0, 0 );
	gtk_container_set_border_width( GTK_CONTAINER( lfram ), 0 );
	gtk_container_add( GTK_CONTAINER( lfram ), label );
	gtk_box_pack_start( GTK_BOX( hpack ), lfram, FALSE, TRUE, 0 );
	
	item  = gtk_frame_new( txt );
	gtk_widget_set_usize( item, CPLX_XSIZE, CPLX_YSIZE );
	gtk_widget_show( item );
	gtk_frame_set_label_align( GTK_FRAME( item ), 0.5, 0 );
	gtk_frame_set_shadow_type( GTK_FRAME( item ), GTK_SHADOW_ETCHED_OUT );
	gtk_container_add( GTK_CONTAINER( item ), hpack );
	gtk_box_pack_start( GTK_BOX( box ), item, wide, fill, 0 );
		
	*adj = adjus;
	*lab = label;
	
	gtk_signal_connect( GTK_OBJECT( adjus ), "value_changed", 
						GTK_SIGNAL_FUNC( handler ), (gpointer) param );
						
	return( scale );
}

/*
*	Creates a spin box and puts it in a box
*	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

GtkWidget	*GuiSpinbox( char *txt, int min, int max, 
						GtkWidget **spp, GtkObject **adj,
						void *handler, void *param,
					    GtkWidget *box, int wide, int fill )
{
GtkWidget	*spin, *label, *cont;
GtkObject	*adjus;
	
	adjus = gtk_adjustment_new( min, min, max, 1, 1, 1 );
	spin = gtk_spin_button_new( GTK_ADJUSTMENT( adjus ), 1, 0 );
	
	gtk_spin_button_set_digits( GTK_SPIN_BUTTON( spin ), 0 );
	gtk_spin_button_set_numeric( GTK_SPIN_BUTTON( spin ), 1 );
	gtk_spin_button_set_snap_to_ticks( GTK_SPIN_BUTTON( spin ), 1 );
	
	gtk_signal_connect( GTK_OBJECT( adjus ), "value_changed",
						GTK_SIGNAL_FUNC( handler ), (gpointer) param );
	
	cont = GuiHbox( TRUE, 0 );
	GuiLabel( txt, NULL, cont, wide, fill );
	GuiBoxAdd( spin, cont, wide, fill );
	GuiBoxAdd( cont, box, wide, fill );
	gtk_widget_show( spin );
		
	*adj = adjus;
	*spp = spin;
	
	return( cont );				
}
